# -*- coding: utf-8 -*-
import decimal
from django.contrib import admin, messages
from django.utils.translation import ugettext_lazy as _
from django.conf.urls.defaults import *
from django import forms

from jumbo.apps.autocomplete import autocomplete
from jumbo.apps.autocomplete.fields import AutocompleteModelChoiceField
from jumbo.forms.models import AdminModelForm
from jumbo.admin.options import OrderedModelAdmin, ExtendibleModelAdmin
from jumbo.core.urlresolvers import reverse

from bom.models import EngeneeringBOM, Element

from product.models import Product

class EngeneeringBOMAdminForm(AdminModelForm):
    class Meta:
        model = EngeneeringBOM

    def __init__(self, *args, **kws):
        super(EngeneeringBOMAdminForm, self).__init__(*args, **kws)

        self.fields['product'] = AutocompleteModelChoiceField(
            'bom_product', label=_("Product"),
#            on_select_token='on_select_product',
#            on_delete_token='on_delete_product',
        )

class ElementAdminForm(AdminModelForm):
    class Meta:
        model = Element

    def __init__(self, *args, **kws):
        super(ElementAdminForm, self).__init__(*args, **kws)
        self.fields['product'] = AutocompleteModelChoiceField(
            'element_product', label=_("Product"),
        )

def get_autocomplete_product(request, query):
    return Product.objects.filter(status=1).order_by('code')

autocomplete.register (
    'bom_product' , model = Product, views = get_autocomplete_product,
    search_fields=('name', 'code',), limit=10
)
autocomplete.register (
    'element_product' , model = Product, views = get_autocomplete_product,
    search_fields=('name', 'code',), limit=10
)

class EngeneeringBOMAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("bom/css/admin_bom.css",)}

    form = EngeneeringBOMAdminForm
    list_display = ('product', 'set_quantity', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon')
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                'status', 'product',
                ('date_begin', 'date_end')
            )
        })
    )
    actions = ('create_ddt',)

    def set_quantity(self, obj):
        return """<INPUT type="text" name="quantity_%s" size="7" maxlength="7">""" % obj.id
    set_quantity.short_description = _('Qty')
    set_quantity.allow_tags = True

    def create_ddt(self, request, queryset):
        """
        Crea DDT e Righe DDT
        Per ogni Distinta Base di Prodotto creo un DDT
        """
        ddts_created = 0
        ddts_not_created = 0
        rows_updated = 0
        rows_not_updated = 0
        post = request.POST
        for obj in queryset:
            quantity = post.get('quantity_%s' % obj.id, None)
            # verifico che venga passato un valore numerico
            try:
                quantity = decimal.Decimal(quantity.replace(',','.'))
            except Exception, e:
                messages.error(request, _("Wrong Quantity"))
                ddts_not_created += 1
                continue
            ddt, error = obj.create_ddt(quantity)
            if error:
                ddts_not_created += 1
                messages.error(request, error)
            else:
                ddts_created += 1

    def get_actions(self, request):
        actions = super(EngeneeringBOMAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        return actions

class ElementAdmin(ExtendibleModelAdmin):
    form = ElementAdminForm
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('bom', 'product', 'unit', 'quantity',),
        })
    )
    use_fancybox = True
#    father_foreignkey_fancybox = 'bom'

admin.site.register(Element, ElementAdmin)
admin.site.register(EngeneeringBOM, EngeneeringBOMAdmin)
