## -*- coding: utf-8 -*-
import datetime

from django.db import models
from django.utils.translation import ugettext_lazy as _
#from django.db.models import Q, connection
from ddt.models import Ddt, DdtRow
from jumbo.utils.functions import get_settings
import mptt

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel, PeriodicityModel, HTMLField
#from jumbo.utils.functions import get_settings, slugify
from organization.models import Organization, Vat

from product.models import Product, UNIT_TYPE

## BOM: Bill Of Materials - Distinta Base

## EngeneeringBOM - Distinta Base Tecnica
## La distinta base tecnica è la prima parte del processo produttivo che coinvolge
## aziende che producono beni. Questa è l'elenco della componentistica che 
## costituisce il bene da produrre; in seguito questo elenco di componenti verrà 
## organizzato per entrare nel ciclo produttivo ed essere realizzato. Solitamente
## tale distinta viene stilata dal progettista attraverso l'estrapolazione dei 
## componenti dal modello solido creato per il bene da produrre.

######################################
## DISTINTA BASE TECNICA             #
######################################
from store.models import StoreCause

class EngeneeringBOM(DateModel, UserModel, StatusModel):
    product = models.ForeignKey(
        Product,
        related_name = 'DBs'
    )
    date_begin = models.DateField(
        verbose_name = _("Date Begin")
    )
    date_end = models.DateField(
        verbose_name = _("Date End")
    )

    class Meta:
        verbose_name = _("Enegeneering BOM")
        verbose_name_plural = _("Enegeneering BOM")
        ordering = ('product__name',)
        permissions = (
            ("view_bom", "Can view bom"),
        )

    def __unicode__(self):
        return self.product.name

    def create_ddt(self, quantity, date_d=None):
        if not date_d:
            date_d = datetime.date.today()

        try:
            owner = Organization.objects.get(organization_type__category='com')
            store_cause_id = get_settings("bom.settings", "BOM_STORE_CAUSE_ID")
            store_cause = StoreCause.objects.get(pk=store_cause_id)
            vat_id = get_settings("bom.settings", "BOM_VAT_ID")
            vat = Vat.objects.get(pk=vat_id)
        except:
            return False, "set SETTINGS in LOCAL_SETTINGS.py"

        ddt = Ddt(
            owner=owner, recipient=owner,
            store_cause=store_cause, date_d=date_d,
        )
        ddt.save()

        # inserisco nel ddt il padre

        ddt_row = DdtRow(
            ddt=ddt, product=self.product, quantity=quantity, vat=vat,
            m_unit=self.product.m_unit, store_cause=store_cause,
        )
        ddt_row.save()
        for element in self.elements.all():
            ddt_row = DdtRow(
                ddt=ddt, product=element.product, quantity=-quantity, vat=vat,
                m_unit=element.product.m_unit, store_cause=store_cause,
            )
            ddt_row.save()
        return ddt, ""

class Element(DateModel, UserModel, StatusModel):
    bom = models.ForeignKey(
        EngeneeringBOM,
        related_name = 'elements'
    )
    product = models.ForeignKey(
        Product,
        related_name = 'elements'
    )
    parent = models.ForeignKey(
        'self', 
        null=True, blank=True, 
        related_name='children'
    )
    quantity = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Quantity"),
    )
    perc_scrap = models.DecimalField (
        max_digits = 4,
        decimal_places = 2,
        default = 0,
        verbose_name = _("% Scrap"),
    )
    price = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=0,
        verbose_name=_("price"),
    )
    unit = models.CharField (
        max_length = 2,
        choices = UNIT_TYPE,
        default = "pz",
        db_index = True,
        verbose_name = _("Unit"),
    )

    class Meta:
        verbose_name = _("BOM Element")
        verbose_name_plural = _("BOM Elements")
        ordering = ('product__name',)

    def __unicode__(self):
        return self.product.name

mptt.register(Element, order_insertion_by=['product'])

#class ManufacturingBOM - Distinta Base Produzione
    #fat_component
    #son_component
    #lft
    #rgt
    #quantity
    #perc_scrap
#...
#lavorazione (interna, acquisto, conto lavorazione)
#costo
#fornitore

## SalesBOM - Distinta Base Ordinato
## ServiceBOM - Distinta Base Manutenzione
