# -*- coding: utf-8 -*-
from datetime import date, timedelta

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jumbo.db import models as jumbo_models

from organization.models import Organization
from invoice.models import Invoice

###############################
# ESTRATTO CONTO              #
###############################
class Commision (DateModel, UserModel, StatusModel):
    ## ForeignKey
    organization = models.ForeignKey (
        Organization,
        related_name = "commisions",
        db_index = True,
        verbose_name = _("Organization"),
    )
    invoice = models.ForeignKey(
        Invoice,
        blank = True, null = True,
        related_name = "commisions",
        verbose_name = _("Invoice"),
    )
    ## campi modello
    operation_date = models.DateField(
        db_index = True,
        verbose_name = _("Operation Date"),
    )
    currency_date = models.DateField(
        db_index = True,
        verbose_name = _("Currency Date"),
    )
    description = jumbo_models.HTMLField(
        verbose_name = _("Description"),
    )
    amount = models.DecimalField(
        max_digits = 9, decimal_places = 2,
        db_index = True,
        verbose_name = _("Amount"),
    )
    note = models.TextField(
        null = True, blank = True,
        verbose_name = _("Note"),
    )

    def __unicode__(self):
        return u"%s" % self.description

    class Meta:
        verbose_name = _("Commision")
        verbose_name_plural = _("Commisions")