import decimal
from dajax.core import Dajax
from dajaxice.core import dajaxice_functions
from ddt.models import Ddt, UNIT_TYPE
from organization.models import Organization
from product.models import Product
from store.models import StoreCause

###############################
# OWNER                       #
###############################
def ajax_on_select_owner (request):
    dajax = Dajax()
    return dajax.json()
dajaxice_functions.register(ajax_on_select_owner)

def ajax_on_delete_owner (request):
    dajax = Dajax()
    return dajax.json()
dajaxice_functions.register(ajax_on_delete_owner)

###############################
# RECIPIENT                   #
###############################
def ajax_on_select_recipient (
    request, recipient_id, contact_id=None, store_cause_id=None, delivery_id=None,
    vector_id=None, shipping_address_id=None, set_default=False
):
    dajax = Dajax()
    if not recipient_id:
        return dajax.json()

    organization = Organization.objects.get(id=recipient_id)

    # CONTACT
    option = ""
    for i, contact in enumerate(organization.contacts.filter(status=1)):
        selected = ""
        if contact_id:
            if contact.id == int(contact_id):
                selected = " selected"
        option += u"<option value='%s'%s>%s</option>" % (contact.id, selected, contact)
    dajax.assign('#id_contact', 'innerHTML', option)

    # STORE CAUSE
    option = ""
    category = [organization.organization_type.category]
    if category == ['com']:
        category = ['com', 'whe']
    print category
    cats = StoreCause.objects.filter(category__in=category, status=1, ddt=True)
    cat_default = None
    try:
        cat_default = StoreCause.objects.get(default=True, status=1, category=category, ddt=True)
    except:
        cat_defaults = StoreCause.objects.filter(status=1, category=category, ddt=True)
        if cat_defaults:
            cat_default = cat_defaults[0]
    for i, cat in enumerate(cats):
        selected = ""
        if store_cause_id:
            if cat.id == int(store_cause_id):
                selected = " selected"
        elif set_default and cat == cat_default:
            selected = " selected"
        option += u"<option value='%s'%s>%s</option>" % (cat.id, selected, cat.name)
    dajax.assign('#id_store_cause', 'innerHTML', option)

    # VECTOR
    option = "<option value=''></option>"
    vectors = organization.vectors.filter(vector__status=1)
    if vectors:
        vec = vectors
    else:
        vec = Organization.objects.filter(organization_type__category='vec', status=1)

    for i, v in enumerate(vec):
        selected = ""
        if vector_id:
            if v.id == int(vector_id):
                selected = " selected"
        elif set_default and i == 0:
            selected = " selected"
        option += "<option value='%s'%s>%s</option>" % (v.id, selected, unicode(v))
    dajax.assign('#id_vector', 'innerHTML', option)

    # SHIPPING ADDRESS
    option = "<option value=''></option>"
    addresses = organization.addresses.filter(address_scopes=3)

    for i, a in enumerate(addresses):
        selected = ""
        if shipping_address_id:
            if a.id == int(shipping_address_id):
                selected = " selected"
        elif set_default and i == 0:
            selected = " selected"
        option += "<option value='%s'%s>%s</option>" % (a.id, selected, unicode(a))
    dajax.assign('#id_shipping_address', 'innerHTML', option)

    return dajax.json()
dajaxice_functions.register(ajax_on_select_recipient)

def ajax_on_delete_recipient(request):
    dajax = Dajax()
    option = "<option value=''></option>"
    dajax.assign('#id_contact', 'innerHTML', option)
    dajax.assign('#id_store_cause', 'innerHTML', option)
    dajax.assign('#id_vector', 'innerHTML', option)
    dajax.assign('#id_shipping_address', 'innerHTML', option)
    return dajax.json()
dajaxice_functions.register(ajax_on_delete_recipient)

###############################
# PRODUCT                     #
###############################
def ajax_on_select_product(
        request, prod_id, ddt_id=None, m_unit=None,
        store_cause_id=None, description=None, set_default=False
):
    dajax = Dajax()

    if not prod_id:
        return dajax.json()

    product = Product.objects.get(id=prod_id)
    ddt = Ddt.objects.get(id=ddt_id)

    # M_UNIT
    option = "<option value=''></option>"
    if product.m_unit and not m_unit:
        m_unit = product.m_unit
    for i, mu in enumerate(UNIT_TYPE):
        selected = ""
        if m_unit:
            if mu[0] == m_unit:
                selected = " selected"
        elif set_default and i == 0:
            selected = " selected"
        option += u"<option value='%s'%s>%s</option>" % (mu[0], selected, mu[1])
    dajax.assign('#id_m_unit', 'innerHTML', option)

    # STORE CAUSE
    option = "<option value=''></option>"
    store_cause = ddt.store_cause
    option += u"<option value='%s' selected>%s</option>" % (store_cause.id, store_cause)
    dajax.assign('#id_store_cause', 'innerHTML', option)

    # DESCRIPTIONS
    if product.short_description and not description:
        dajax.assign('#id_description', 'value', product.short_description)
    return dajax.json()
dajaxice_functions.register(ajax_on_select_product)

def ajax_on_delete_product(request):
    dajax = Dajax()
    dajax.assign('#id_quantity', 'value', "")
    dajax.assign('#id_description', 'value', "")
    option = "<option value=''></option>"
    dajax.assign('#id_store_cause', 'innerHTML', option)
    dajax.assign('#id_m_unit', 'innerHTML', option)

    return dajax.json()
dajaxice_functions.register(ajax_on_delete_product)
