import decimal
from django.db.models import Q
from django.utils.html import strip_tags
from dajax.core import Dajax
from dajaxice.core import dajaxice_functions
from invoice.models import Invoice, UNIT_TYPE
from jumbo.apps.project.models import Project

from jumbo.shortcuts import render_to_string

from organization.models import Organization
from organization.models import  Vat
from organization.models import OrganizationAddress
from store.models import StoreCause
from payment.models import PaymentForm
from product.models import  Product

def ajax_on_select_owner(request, org_id, contact_id=None, payment_form_id=None,
                         store_cause_id=None, invoice_address_id=None, bank_id=None, set_default=False):
    dajax = Dajax()
    if not org_id:
        return dajax.json()

    o = Organization.objects.get(id=org_id)

    # CONTACT
    option = "<option value=''></option>"
    for i, contact in enumerate(o.contacts.all()):
        selected = ""
        if contact_id:
            if contact.id == int(contact_id):
                selected = " selected"
        elif set_default and i == 0:
            selected = " selected"
        option += "<option value='%s'%s>%s</option>" % (contact.id, selected, unicode(contact))
    dajax.assign('#id_contact', 'innerHTML', option)

    # PAYMENT FORM
    option = "<option value=''></option>"
    if o.payment_form:
        org_payment_id = o.payment_form.id
    else:
        org_payment_id = 0

    payments = PaymentForm.objects.filter(status=1)

    for i, pay in enumerate(payments):
        selected = ""
        if payment_form_id:
            if pay.id == int(payment_form_id):
                selected = " selected"
        elif set_default and pay.id == org_payment_id:
            selected = " selected"
        option += "<option value='%s'%s>%s</option>" % (pay.id, selected, unicode(pay))
    dajax.assign('#id_payment_form', 'innerHTML', option)

    # BANK
    option = "<option value=''></option>"
    q1 = Q(id=bank_id)
    q2 = Q(status=1)
    if bank_id:
        banks = o.banks.filter(q1|q2).distinct()
    else:
        banks = o.banks.filter(q2)

    for i, bank in enumerate(banks):
        selected = ""
        if bank_id:
            if bank.id == int(bank_id):
                selected = " selected"
        elif set_default and i == 0:
            selected = " selected"
        option += "<option value='%s'%s>%s</option>" % (bank.id, selected, unicode(bank))
    dajax.assign('#id_bank', 'innerHTML', option)

    # INVOICE ADDRESS
    option = "<option value=''></option>"
    invoice_addresses = o.get_invoice_addresses()
    if invoice_addresses:
        if invoice_address_id:
            q = OrganizationAddress.objects.get(id=invoice_address_id)
            invoice_addresses.append(q)
    else:
        qa = Q(id=invoice_address_id)
        qb = Q(status=1)
        addresses = o.addresses.filter(address_scopes__id=1)
        if invoice_address_id:
            invoice_addresses = o.addresses.filter(qa|qb).distinct()
        else:
            invoice_addresses = addresses.filter(qb)

    for i, a in enumerate(invoice_addresses):
        selected = ""
        if invoice_address_id:
            if a.id == int(invoice_address_id):
                selected = " selected"
        elif set_default and i == 0:
            selected = " selected"
        option += "<option value='%s'%s>%s</option>" % (a.id, selected, unicode(a))
    dajax.assign('#id_invoice_address', 'innerHTML', option)

    # STORE CAUSE
    option = "<option value=''></option>"
    category = o.organization_type.category
    cats = StoreCause.objects.filter(category=category, status=1)
    cat_default = None
    try:
        cat_default = StoreCause.objects.get(default=True, status=1, category=category)
    except:
        cat_defaults = StoreCause.objects.filter(status=1, category=category)
        if cat_defaults:
            cat_default = cat_defaults[0]

    for i, cat in enumerate(cats):
        selected = ""
        if store_cause_id:
            if cat.id == int(store_cause_id):
                selected = " selected"
        elif set_default and cat == cat_default:
            selected = " selected"
        option += "<option value='%s'%s>%s</option>" % (cat.id, selected, unicode(cat.name))
    dajax.assign('#id_store_cause', 'innerHTML', option)

    return dajax.json()
dajaxice_functions.register(ajax_on_select_owner)

def ajax_on_delete_owner(request):
    dajax = Dajax()
    option = "<option value=''></option>"
    dajax.assign('#id_contact', 'innerHTML', option)
    dajax.assign('#id_payment_form', 'innerHTML', option)
    dajax.assign('#id_bank', 'innerHTML', option)
    dajax.assign('#id_store_cause', 'innerHTML', option)
    dajax.assign('#id_invoice_address', 'innerHTML', option)

    return dajax.json()
dajaxice_functions.register(ajax_on_delete_owner)

##################################
# PRODUCT                        #
##################################
def ajax_on_select_product(
        request, prod_id, invoice_id=None, vat_id=None, m_unit=None,
        store_cause_id=None, description=None, long_description=None,
        project_id=None, set_default=False
):
    dajax = Dajax()

    if not prod_id:
        return dajax.json()

    product = Product.objects.get(id=prod_id)
    invoice = Invoice.objects.get(id=invoice_id)
    pricelist = None
    perc_discount1 = 0
    perc_discount2 = 0
    try:
#        try:
#            if invoice.price_list:
#                pricelist = invoice.price_list.price_list
#                perc_discount1 = invoice.price_list.discount1
#                perc_discount2 = invoice.price_list.discount2
#        except Invoice.DoesNotExist:
#            raise Http404

        # PRICELISTITEM
        try:
            # invoice.price_list.price_list -> il primo price_list e' l'organization_price_list,
            # mentre il secondo e' il listino associato

            pricelistitem = product.get_active_pricelistitem(pricelist=pricelist)
            price = pricelistitem.last_price
        except Exception, e:
            price = decimal.Decimal(0)
        dajax.assign('#id_price', 'value', price.quantize(decimal.Decimal('0.01')))
        dajax.assign('#id_perc_discount1', 'value', perc_discount1)
        dajax.assign('#id_perc_discount2', 'value', perc_discount2)
        dajax.assign('#id_fee', 'value', 0)
        dajax.assign('#id_discount', 'value', 0)

        # VAT
        option = ""
        if product.vat and not vat_id:
            vat_id = product.vat.id
        vat = Vat.objects.filter(status=1)
        for i, v in enumerate(vat):
            selected = ""
            if vat_id:
                if v.id == int(vat_id):
                    selected = " selected"
            elif set_default and i == 0:
                selected = " selected"
            option += u"<option value='%s'%s>%s</option>" % (v.id, selected, v)
        dajax.assign('#id_vat', 'innerHTML', option)

        # PROJECT
        option = "<option value=''></option>"
        owner = invoice.owner
        recipient = invoice.recipient
        category = owner.organization_type.category
        projects = Project.objects.filter(status=1)
        if category == "cus":
            projects = projects.filter(organization__in=(owner, recipient))
        for i, proj in enumerate(projects):
            selected = ""
            if project_id:
                if proj.id == int(project_id):
                    selected = " selected"
            option += u"<option value='%s'%s>%s</option>" % (proj.id, selected, proj)
        dajax.assign('#id_project', 'innerHTML', option)

        # M_UNIT
        option = "<option value=''></option>"
        if product.m_unit and not m_unit:
            m_unit = product.m_unit
        for i, mu in enumerate(UNIT_TYPE):
            selected = ""
            if m_unit:
                if mu[0] == m_unit:
                    selected = " selected"
            elif set_default and i == 0:
                selected = " selected"
            option += u"<option value='%s'%s>%s</option>" % (mu[0], selected, mu[1])
        dajax.assign('#id_m_unit', 'innerHTML', option)

        # STORE CAUSE
        option = "<option value=''></option>"
        proc_store_cause = invoice.store_cause
        prod_store_cause = product.store_cause
        category = invoice.owner.organization_type.category

        if prod_store_cause and not store_cause_id and category == "cus":
            store_cause_id = prod_store_cause.id
        if proc_store_cause and not store_cause_id:
            store_cause_id = proc_store_cause.id

        store_causes = StoreCause.objects.filter(status=1, category=category)
        for i, sc in enumerate(store_causes):
            selected = ""
            if store_cause_id:
                if sc.id == int(store_cause_id):
                    selected = " selected"
            elif set_default and i == 0:
                selected = " selected"
            option += u"<option value='%s'%s>%s</option>" % (sc.id, selected, sc)
        dajax.assign('#id_store_cause', 'innerHTML', option)

        # DESCRIPTIONS
        if product.short_description and not description:
            dajax.assign('#id_description', 'value', product.short_description)
        if product.full_description and not long_description:
            full_description = strip_tags(product.full_description)
            dajax.assign('#id_long_description', 'value', full_description)
        return dajax.json()
    except Exception, e:
        #print e
        pass

dajaxice_functions.register(ajax_on_select_product)

def ajax_on_delete_product(request):
    dajax = Dajax()
    dajax.assign('#id_price', 'value', "")
    dajax.assign('#id_project', 'value', "")
    dajax.assign('#id_quantity', 'value', "")
    dajax.assign('#id_fee', 'value', "")
    dajax.assign('#id_discount', 'value', "")
    dajax.assign('#id_description', 'value', "")
    dajax.assign('#id_long_description', 'value', "")
    dajax.assign('#id_perc_discount1', 'value', "")
    dajax.assign('#id_perc_discount2', 'value', "")
    option = "<option value=''></option>"
    dajax.assign('#id_vat', 'innerHTML', option)
    dajax.assign('#id_store_cause', 'innerHTML', option)
    dajax.assign('#id_m_unit', 'innerHTML', option)
    return dajax.json()

dajaxice_functions.register(ajax_on_delete_product)

def reload_invoice_rows(request, invoice_id):
    dajax = Dajax()
    i = Invoice.objects.get(id=invoice_id)
    str_invoice = render_to_string("admin/invoice/invoice/rows_inline.html", original=i)
    dajax.assign("#invoice_rows", "innerHTML", str_invoice)

    return dajax.json()
dajaxice_functions.register(reload_invoice_rows)