from django.conf import settings
from django.utils.translation import ugettext as _
from jumbo.core.urlresolvers import reverse
from invoice.models import Invoice, InvoiceRow

class InvoiceList(object):
    list_display = ['get_date_i', 'number', 'get_detail', 'get_payment_form_code', 'get_store_cause_code', 'get_income', 'get_fee', 'get_discount', 'get_vat', 'get_total', 'get_commision', 'create_commision', 'get_edit_icon', 'get_invoice_rows']
    sort_field = {
        'date_i'        : 'date_i',
        'number'        : 'number',
        'store_cause'   : 'store_cause__name',
        'owner'         : 'owner__name',
    }
    default_order_by = ('-date_i', '-number')
    model = Invoice

    def get_detail(self, result):
        owner = result.owner
        contact = result.contact or ""
        return "<b>%s</b><br/>%s" % (owner, contact)
    get_detail.short_description = _("Bill to")

    def get_payment_form_code(self, result):
        return result.payment_form.code
    get_payment_form_code.short_description = _("Payment Form")

    def get_store_cause_code(self, result):
        return result.store_cause.code
    get_store_cause_code.short_description = _("Store Cause")

    def get_date_i(self, result):
        return result.date_i.strftime("%d/%m/%Y")
    get_date_i.short_description = _('Date')

    def get_commision(self, result):
        row = "<img alt='False' src='/media/img/admin/icon-no.gif'>"
        if result.commision:
            row = "<img alt='True' src='/media/img/admin/icon-yes.gif'>"
        return row
    get_commision.short_description = _('CC')

    def get_income(self, result):
        return result.totals['income']
    get_income.short_description = _('Total Income')

    def get_fee(self, result):
        return result.totals['fee']
    get_fee.short_description = _('Total Fee')

    def get_discount(self, result):
        return result.totals['discount']
    get_discount.short_description = _('Total Discount')

    def get_vat(self, result):
        return result.totals['vat']
    get_vat.short_description = _('Total VAT')

    def get_total(self, result):
        return result.totals['total']
    get_total.short_description = _('Total')

    def get_edit_icon(self, result):
        return "<a href=%(reverse_url)s><img src='%(url)sjumbo/img/edit.gif' alt='%(window_title)s' /></a>" % {
            'reverse_url':reverse('invoice-invoice-edit', result.id),
            'url':settings.MEDIA_URL,
            'window_title':_("Edit Invoice")
        }
    get_edit_icon.short_description = _("Edit")

    def create_commision(self, result):
        return "<a href=%(reverse_url)s>x</a>" % {
            'reverse_url':reverse('invoice-invoice-create_commision', result.id),
            'window_title':_("Create Invoice")
        }
    create_commision.short_description = _("COM")

    def get_invoice_rows(self, result):
        pr = result.invoicerows.count()
        num_pr = None
        if pr > 0:
            num_pr = """<a href="%(reverse_url)s" rel="#overlay">%(num_invoice)s</a>""" % {
                'reverse_url': reverse('invoice-invoice-invoicerows', result.id),
                'num_invoice': pr,
            }

        return "%s" % num_pr
    get_invoice_rows.short_description = _('PR')

    def change_filter(self, filtro):
        if filtro.has_key('date_i_begin__gte'):
            date_i_begin__gte = filtro['date_i_begin__gte']
            del filtro['date_i_begin__gte']
            filtro['date_i__gte'] = date_i_begin__gte

        if filtro.has_key('date_i_end__lte'):
            date_i_end__lte = filtro['date_i_end__lte']
            del filtro['date_i_end__lte']
            filtro['date_i__lte'] = date_i_end__lte

        return filtro

class InvoiceRowList(object):
    list_display = [
        'get_date_i', 'get_number', 'get_organization', 'procedurerow', 'ddtrow', 'get_product_project',
        'quantity', 'price', 'fee', 'discount', 'get_commision', 'get_total_income', 'get_total_fee',
        'get_total_discount', 'get_total', 'get_cause_code', 'get_vat_code', 'get_edit_icon'
    ]
    #default_order_by = ('-invoice__date_i', '-invoice__number', 'ordering', 'id')
    default_order_by = ('project', 'invoice__owner', '-invoice__date_i')
    default_group_by = ('project')
    model = InvoiceRow

    def get_commision(self, result):
        row = "<img alt='False' src='/media/img/admin/icon-no.gif'>"
        if result.commision:
            row = "<img alt='True' src='/media/img/admin/icon-yes.gif'>"
        return row
    get_commision.short_description = _('CC')

    def get_date_i(self, result):
        return result.invoice.date_i.strftime("%d/%m/%Y")
    get_date_i.short_description = _('Date')

    def get_number(self, result):
        return result.invoice.number
    get_number.short_description = _('Number')

    def get_organization(self, result):
        owner = result.invoice.owner
        recipient = result.invoice.recipient
        contact = result.invoice.contact
        if recipient == owner:
            recipient = ""
        if not recipient:
            recipient = ""
        if not contact:
            contact = ""

        return "%s<br/>%s<br/>%s" % (owner, recipient, contact)
    get_organization.short_description = _('Organization')

    def get_product_project(self, result):
        product = result.get_description()
        project = result.project
        return "<b>%s</b><br/>%s" % (product, project)
    get_product_project.short_description = _("Product")

    def get_vat_code(self, result):
        return result.vat.code
    get_vat_code.short_description = _('VC')

    def get_cause_code(sel, result):
        return result.store_cause.code
    get_cause_code.short_description = _('SC')

    def get_edit_icon(self, result):
        return "<a href=%(reverse_url)s><img src='%(url)sjumbo/img/edit.gif' alt='%(window_title)s' /></a>" % {
            'reverse_url':reverse('invoice-invoicerow-edit', result.id),
            'url':settings.MEDIA_URL,
            'window_title':_("Edit Invoice Row")
        }
    get_edit_icon.short_description = _("Edit")
