# -*- coding: utf-8 -*-
from django.db import connection
from invoice.models import Invoice

def dictfetchall(cursor):
    "Returns all rows from a cursor as a dict"
    desc = cursor.description
    return [
        dict(zip([col[0] for col in desc], row))
        for row in cursor.fetchall()
    ]

def total_invoice_per_year(years=None, others_where=''):
    """
    Returns totals per year
    years: is a list of years
    """
    where = ''
    if years:
        where = "AND DATE_TRUNC('year', date_i) = %s" % years

    cursor = connection.cursor()
    query = """
        SELECT
            COUNT(*) AS invoices,
            DATE_TRUNC('year', date_i) AS year,
            SUM(total_income) AS total_income,
            SUM(total_fee) AS total_fee,
            SUM(total_discount) AS total_discount,
            SUM(total_vat) AS total_vat,
            SUM(
                total_income + total_fee
                - total_discount
            ) AS total_no_vat,
            SUM(
                total_income + total_fee
                - total_discount + total_vat
            ) AS total
        FROM
            invoice_invoice
        WHERE 1=1
        %s %s
        GROUP BY year
        ORDER BY year DESC
    """ % (where, others_where)
    cursor.execute(query)
    dictionary = dictfetchall(cursor)
    return dictionary

def total_invoice_per_month(years=None, others_where=''):
    """
    Returns totals per month
    years: is a list of years
    """
    where = ''
    if years:
        where = "AND date_i IN %s" % years

    cursor = connection.cursor()
    query = """
        SELECT
            COUNT(*) AS invoices,
            DATE_TRUNC('year', date_i) AS year,
            DATE_TRUNC('month', date_i) AS month,
            SUM(total_income) AS total_income,
            SUM(total_fee) AS total_fee,
            SUM(total_discount) AS total_discount,
            SUM(total_vat) AS total_vat,
            SUM(
                total_income + total_fee
                - total_discount
            ) AS total_no_vat,
            SUM(
                total_income + total_fee
                - total_discount + total_vat
            ) AS total
        FROM
            invoice_invoice
        WHERE 1=1
            %s %s
        GROUP BY year, month
        ORDER BY year, month DESC
    """ % (where, others_where)
    cursor.execute(query)
    dictionary = dictfetchall(cursor)
    return dictionary

def total_invoice_per_year_per_organization(years=None, others_where='', limit=''):
    """
    Returns Top Ten Organizations per year
    years: is a list of years
    """
    where = ''
    if years:
        where = " AND DATE_PART('year', i.date_i) = %s" % years

    if limit:
        limit = "LIMIT %i" % limit

    cursor = connection.cursor()
    query = """
        SELECT
            COUNT(*) AS invoices,
            DATE_TRUNC('year', i.date_i) AS year,
            o.name,
            SUM(i.total_income) AS total_income,
            SUM(i.total_fee) AS total_fee,
            SUM(i.total_discount) AS total_discount,
            SUM(i.total_vat) AS total_vat,
            SUM(
                i.total_income + i.total_fee
                - i.total_discount
            ) AS total_no_vat,
            SUM(
                i.total_income + i.total_fee
                - i.total_discount + i.total_vat
            ) AS total
        FROM
            invoice_invoice i
            JOIN organization_organization o ON o.id=i.owner_id
        WHERE 1=1
            %s %s
        GROUP BY year, i.owner_id, o.name
        ORDER BY year, total DESC
        %s
    """ % (where, others_where, limit)
    cursor.execute(query)
    dictionary = dictfetchall(cursor)
    return dictionary

def total_invoice_per_month_per_organization(years=None, months=None, others_where='', limit=''):
    """
    Returns Top Ten Organizations per year
    years: is a list of years
    """
    where = ''

    if years:
        where += " AND DATE_PART('year', i.date_i) = %s" % years
    if months:
        where += " AND DATE_PART('month', i.date_i) = %s" % months
    if limit:
        limit = "LIMIT %i" % limit

    cursor = connection.cursor()
    query = """
        SELECT
            COUNT(*) AS invoices,
            DATE_TRUNC('year', i.date_i) AS year,
            DATE_TRUNC('month', i.date_i) AS month,
            o.name,
            SUM(i.total_income) AS total_income,
            SUM(i.total_fee) AS total_fee,
            SUM(i.total_discount) AS total_discount,
            SUM(i.total_vat) AS total_vat,
            SUM(
                i.total_income + i.total_fee
                - i.total_discount
            ) AS total_no_vat,
            SUM(
                i.total_income + i.total_fee
                - i.total_discount + i.total_vat
            ) AS total
        FROM
            invoice_invoice i
            JOIN organization_organization o ON o.id=i.owner_id
        WHERE 1=1
            %s %s
        GROUP BY year, month, i.owner_id, o.name
        ORDER BY year, month, total DESC
        %s
    """ % (where, others_where, limit)

    cursor.execute(query)
    dictionary = dictfetchall(cursor)
    return dictionary
