# -*- coding: utf-8 -*-

from django.http import Http404, HttpResponse, HttpResponseForbidden, HttpResponseRedirect
from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode
from django import template
from django.shortcuts import render_to_response
from jumbo.utils.functions import get_settings

from jumbo.views.generic.create_oo_file import create_file

from invoice.models import Invoice
from invoice.forms import SendMailForm

def invoice_send_mail(request, invoice_id):
    try:
        invoice = Invoice.objects.get(pk=invoice_id)
        opts = invoice._meta
    except Exception, e:
        raise Http404(_('Invoice with primary key %s does not exist.') % invoice_id)

    mails = set(invoice.owner.get_all_mails_for_invoice())
    if invoice.owner != invoice.recipient:
        for rec_email in invoice.recipient.get_all_mails_for_invoice():
            mails.add(rec_email)


    # Recupero la lingua del primo contatto per decidere in che lingua
    # creare e mandare la fattura
    odt_language = 'en'
    contacts = invoice.owner.get_invoice_contacts()[:1]

    if contacts and contacts[0].language:
        odt_language = contacts[0].language.code

    lmails = [ (email[0], email[1]) for email in mails ]

    if request.POST:
        form = SendMailForm(request.POST)

        form.fields['mails'].choices = lmails
        if form.is_valid():
            f = create_file(
                request, Invoice, invoice_id,
                file_model = 'admin/invoice/invoice/invoice.odt',
                file_type  = 'pdf',
                get_raw_string = True,
                language = odt_language,
                extra_context={'form':form}
            )

            all_mails = set()

            fromail = get_settings("invoice.settings", "INVOICE_MAIL_FROM")

            if invoice.creator.email:
                fromail = invoice.creator.email
                all_mails.add(invoice.creator.email)

            for mail in form.cleaned_data['mails']:
                if mail:
                    all_mails.add(mail)

            for mail in form.cleaned_data['other_mails']:
                if mail:
                    all_mails.add(mail)

            if all_mails:
                subject = render_to_string("admin/invoice/invoice/mail_subject.txt", {'invoice':invoice, 'form':form})
                # Email subject *must not* contain newlines
                subject = ''.join(subject.splitlines())
                body    = render_to_string("admin/invoice/invoice/mail_body.txt", {'invoice':invoice, 'form':form})

                email_msg = EmailMessage(subject, body, fromail, [], all_mails)

                email_msg.attach(
                    'fattura_%s.pdf' % invoice.date_p.strftime("%Y%m%d"),
                    f.read(),
                    'application/pdf'
                )

                email_msg.send(fail_silently=False)
            messages.success(request, _('Mail Sent Successfully.'))
            return HttpResponseRedirect(reverse('admin:invoice_invoice_changelist'))
    else:
        form = SendMailForm()
        form.fields['mails'].choices = lmails

    context = {
        'title': _('Send Mail - Invoice : %s') % force_unicode(invoice),
        'object_id': invoice_id,
        'obj': invoice,
        'is_popup': request.REQUEST.has_key('_popup'),
        'app_label': opts.app_label,
        'opts': opts,
        'form':form
    }
    context_instance = template.RequestContext(request, current_app='invoice')
    return render_to_response(
        'admin/invoice/invoice/sendmail.html', context,
        context_instance=context_instance
    )
