# -*- coding: utf-8 -*-
import datetime
import decimal

from django.contrib import admin
from django.template.defaultfilters import floatformat
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.utils.formats import date_format
from django.conf.urls.defaults import *
from django import forms
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.conf import settings
from django.utils.encoding import force_unicode
from django.utils.formats import number_format
from django.template.defaultfilters import floatformat
from django.db.models import Q
from django.forms.widgets import Select
from django.contrib import messages

from jumbo.forms.models import AdminModelForm
from jumbo.admin.options import OrderedModelAdmin, ExtendibleModelAdmin, SettingsAdmin
from jumbo.core.urlresolvers import reverse
from jumbo.forms import DateWidget
from jumbo.forms.fields import ChainedChoiceField
from jumbo.middleware.thread_local import get_request
from jumbo.views.generic.create_oo_file import create_file
from jumbo.utils.functions import get_settings

from jumbo.apps.autocomplete import autocomplete
from jumbo.apps.autocomplete import AutocompleteModelChoiceField, AutocompleteInlineCharField
from jumbo.apps.smart_selects.form_fields import ChainedModelChoiceField
from jumbo.apps.smart_selects.widgets import ChainedSelect
from jumbo.apps.project.models import Project

from organization.models import Organization, Contact, Vat, Bank, Currency, Agent
from organization.models import OrganizationAddress, Delivery, Commision, M2MtoVector
from store.models import StoreCause
from payment.models import PaymentForm
from procedure.models import Procedure, Settings
from procedure.models import ProcedureRow, VatRow, TotalOrganizationInvoiced
from product.models import OrganizationPriceList, Product
from procedure import views as procedure_views


#########################################
## PROCEDURE ROW INLINE                 #
#########################################
class ProcedureRowInlineAdminForm(AdminModelForm):
    class Meta:
        model = ProcedureRow

    def __init__(self, *args, **kws):
        super(ProcedureRowInlineAdminForm, self).__init__(*args, **kws)
        self.fields['price'] = forms.DecimalField(max_digits=11, decimal_places=4, localize=True)
        self.fields['quantity'] = forms.DecimalField(max_digits=9, decimal_places=2, localize=True)
        #self.fields['price'].localize = True
        #self.fields['quantity'].localize = True
        self.fields['product'] = AutocompleteInlineCharField('procedure_product', label=_("Product"))
        self.fields['store_cause'].widget =  ChainedSelect(
            queryset       = self.fields['store_cause'].queryset,
            choices        = self.fields['store_cause'].choices,
            app_name       = "store",
            model_name     = "StoreCause",
            chain_field    = "owner",
            model_field    = "organization",
            chain_function = "procedure.views.chained_store_cause",
            is_required    = "True",
            attrs          = {'class':'procedure_store_cause_inline'}
        )
        self.fields['vat'].widget =  ChainedSelect(
            queryset       = self.fields['vat'].queryset,
            choices        = self.fields['vat'].choices,
            app_name       = "organization",
            model_name     = "Vat",
            chain_field    = "owner",
            model_field    = "organization",
            chain_function = "procedure.views.chained_vat",
            is_required    = "True",
            attrs          = {'class':'procedure_vat_inline'}
        )


##########################################
## PROCEDURE                            ##
##########################################
class ProcedureSearchForm(forms.ModelForm):
    class Meta:
        model          = Procedure
        fields         = ('payment_form', 'store_cause')
        display_fields = (
                           ('owner__name__istartswith', 'owner__code'),
                           ('recipient__name__istartswith', 'recipient__code'),
                           ('payment_form', 'store_cause'),
                           ('number', 'owner_number'),
                           ('date_p__year', 'search_in_all_years'),
                           ('date_p__gte', 'date_p__lte'),
                           ('accepted', 'ddt', 'invoice')
                         )

    def __init__(self, *args, **kws):
        super(ProcedureSearchForm, self).__init__(*args, **kws)
        BOOLEAN = (('', '---'), ('1', _('Yes')), ('0', _('No')),)

        self.fields['owner__name__istartswith'] = forms.CharField(label=_("Owner"), required=False)
        self.fields['owner__code'] = forms.CharField(label=_("Owner Code"), required=False)
        self.fields['recipient__name__istartswith'] = forms.CharField(label=_("Recipient"), required=False)
        self.fields['recipient__code'] = forms.CharField(label=_("Recipient Code"), required=False)
        self.fields['date_p__gte'] = forms.DateField(label=_("Date Begin"), widget= DateWidget, required=False)
        self.fields['date_p__lte'] = forms.DateField(label=_("Date End"), widget= DateWidget, required=False)
        self.fields['number'] = forms.CharField(label=_("Number"), required=False)
        self.fields['owner_number'] = forms.CharField(label=_("Owner Number"), required=False)
        self.fields['accepted'] = forms.ChoiceField(label=_("Accepted"), choices=BOOLEAN, required=False)
        self.fields['ddt'] = forms.ChoiceField(label=_("DDT Complete"), choices=BOOLEAN, required=False)
        self.fields['invoice'] = forms.ChoiceField(label=_("Invoice Complete"), choices=BOOLEAN, required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['date_p__year'] = forms.CharField(label=_("Year"), required=False)

    # Modifico il filtro di ricerca in modo da filtrare sempre in base all'anno corrente, tranne nel caso in cui
    # nei parametri di ricerca venga specificato un anno oppure venga flaggata la checkbox search_in_all_years
    def change_search_filter(self, lookup_params, qs):
        if not lookup_params.has_key('date_p__year') and not lookup_params.has_key('search_in_all_years'):
            qs = qs.filter(date_p__year = datetime.date.today().year)

        if lookup_params.has_key('search_in_all_years'):
            del lookup_params['search_in_all_years']
        return lookup_params, qs


class ProcedureAdminForm(AdminModelForm):
    class Meta:
        model = Procedure

    class Media:
        js = ('procedure/js/procedure_form.js',)

    def __init__(self, *args, **kws):
        super(ProcedureAdminForm, self).__init__(*args, **kws)
        self.fields['date_p'].initial = datetime.date.today()
        self.fields['owner'] = AutocompleteModelChoiceField(
            'procedure_owner', label=_("Owner"),
            on_select_token='on_select_owner',
            on_delete_token='on_delete_owner',
        )
        self.fields['recipient'] = AutocompleteModelChoiceField(
            'procedure_recipient', label=_("Recipient"),
            on_select_token='on_select_recipient',
            on_delete_token='on_delete_recipient',
        )
        self.fields['contact'] = ChainedChoiceField(model=Contact, label=_("Contact"))
        self.fields['project'] = ChainedChoiceField(model=Project, label=_("Project"), required=False)
        self.fields['bank'] = ChainedChoiceField(model=Bank, label=_("Bank"), required=False)
        self.fields['payment_form'] = ChainedChoiceField(model=PaymentForm, label=_("Payment Form"))
        self.fields['store_cause'] = ChainedChoiceField(model=StoreCause, label=_("Store Cause"))
        self.fields['currency'] = ChainedChoiceField(model=Currency, label=_("Currency"))
        self.fields['vat'] = ChainedChoiceField(model=Vat, label=_("Vat"))
        self.fields['delivery'] = ChainedChoiceField(model=Delivery, label=_("Delivery"), required=False)
        self.fields['price_list'] = ChainedChoiceField(model=OrganizationPriceList, label=_("Price List"), required=False)
        self.fields['commision'] = ChainedChoiceField(model=Commision, label=_("Commision"), required=False)
        self.fields['vector'] = ChainedChoiceField(model=M2MtoVector, label=_("Vector"), required=False)
        self.fields['shipping_address'] = ChainedChoiceField(model=OrganizationAddress, label=_("Shipping Address"), required=False)


class ProcedureAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("procedure/css/admin_procedure.css",)}

    form               = ProcedureAdminForm
    search_form        = ProcedureSearchForm
    list_per_page      = 30
    actions            = ["set_accepted", "force_as_delivered", "force_as_invoiced", "set_as_delivered", "set_as_invoiced", "create_ddt", "create_invoice"]
    date_hierarchy     = 'date_p'
    fieldsets          = (
        (_('logs admin'), {
            'fields'  : ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None           , {
            'fields'  : (
                'status',
                ('date_p', 'number'),
                'owner_number',
                ('owner', 'recipient',),
                ('contact', 'project',),
                ('bank', 'payment_form',),
                'store_cause',
                'currency',
                'vat',
                'price_list',
                'commision',
                ('vector', 'delivery',),
                ('date_delivery', 'shipping_address',),
                ('accepted'), ('ddt',), ('invoice',),
                'description',
            )
        }),
        (_('note')     , {'fields'  : ('note',), 'classes' : ('collapse',)}),
    )
    search_fields      = ['owner__name', 'owner__code',
                          'recipient__name', 'recipient__code',
                          'number', 'owner_number']
    list_display       = (
        'get_date_p', 'number', 'get_organization',
        'get_store_cause', 'get_total', 'accepted', 'get_ddt', 'get_invoice',
        'create_odt', 'create_pdf',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    list_display_csv   = (
        'get_date_p', 'number', 'owner_number', 'owner', 'contact',
        'description', 'payment_form', 'store_cause', 'get_income',
        'get_fee', 'get_discount', 'get_vat', 'get_total', 'accepted',
        'ddt', 'invoice'
    )
    list_display_links = ['number']
    default_group_by   = 'owner'

    def get_ddt(self, obj):
        icon = "icon-no.gif"
        if obj.ddt:
            icon = "icon-yes.gif"
        ddtrows = """
            <a href="%(href)s?procedurerow__procedure__id=%(obj)s" title="%(title)s">
                <img alt="True" src="/media/img/admin/%(icon)s" />
            </a>
        """ % {
            'href'      :   reverse('admin:ddt_ddtrow_changelist'),
            'obj'       :   obj.id,
            'title'     :   ugt("List Ddt Rows"),
            'icon'      :   icon
        }
        return ddtrows
    get_ddt.short_description=_('DDT')
    get_ddt.allow_tags = True
    get_ddt.admin_order_field='ddt'

    def get_invoice(self, obj):
        icon = "icon-no.gif"
        if obj.invoice:
            icon = "icon-yes.gif"
        ddtrows = """
            <a href="%(href)s?ddtrow__procedurerow__procedure__id=%(obj)s" title="%(title)s">
                <img alt="True" src="/media/img/admin/%(icon)s" />
            </a>
        """ % {
            'href'      :   reverse('admin:invoice_invoicerow_changelist'),
            'obj'       :   obj.id,
            'title'     :   ugt("List Invoice Rows"),
            'icon'      :   icon
        }
        return ddtrows
    get_invoice.short_description=_('INV')
    get_invoice.allow_tags = True
    get_invoice.admin_order_field='invoice'

    def save_model(self, request, obj, form, change):
        # Se tutte le righe ordine sono state evase allora setto ddt = True
        # anche alla procedure
        if obj.is_alldelivered():
            obj.ddt = True
        obj.save()


    def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
        PROCEDURE_SEND_MAIL_FORM = get_settings('procedure.settings', 'PROCEDURE_SEND_MAIL_FORM')
        context.update({'PROCEDURE_SEND_MAIL_FORM':PROCEDURE_SEND_MAIL_FORM})
        return super(ProcedureAdmin, self).render_change_form(request, context, add, change, form_url, obj)


    def response_add(self, request, obj, post_url_continue='../%s/'):
        """
        Determines the HttpResponse for the add_view stage.
        """
        opts = obj._meta
        pk_value = obj._get_pk_val()

        # Here, we distinguish between different save types by checking for
        # the presence of keys in request.POST.
        if request.POST.has_key("_save_and_add_procedure_row"):
            self.message_user(request, ugt('Now you can add procedure row.'))
            if request.POST.has_key("_popup"):
                post_url_continue += "?_popup=1"
            return HttpResponseRedirect(post_url_continue % pk_value)

        return super(ProcedureAdmin, self).response_add(request, obj, post_url_continue)


    def response_change(self, request, obj):
        """
        Determines the HttpResponse for the change_view stage.
        """
        opts = obj._meta
        pk_value = obj._get_pk_val()

        # Here, we distinguish between different save types by checking for
        # the presence of keys in request.POST.
        if request.POST.has_key("_save_and_mail"):
            msg = _('The %(name)s "%(obj)s" was saved successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}
            self.message_user(request, msg)
            return HttpResponseRedirect(reverse('admin:procedure_procedure_sendmail', obj.id))

        return super(ProcedureAdmin, self).response_change(request, obj)


    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(ProcedureAdmin, self).get_urls()
        my_urls = patterns('',
            url(
                r'^procedurerows_detail/(?P<procedure_id>\d+)/$',
                self.admin_site.admin_view(procedure_views.procedurerows),
                name = 'procedure_procedure_procedurerows_detail'
            ),
            url(
                r'^send_mail/(?P<procedure_id>\d+)/$',
                self.admin_site.admin_view(procedure_views.procedure_send_mail),
                name = 'procedure_procedure_sendmail'
            ),
            url(
                r'^get_price/(?P<owner_id>\d+)/(?P<product_id>\d+)/$',
                self.admin_site.admin_view(procedure_views.get_price),
            ),
            url(
                r'^create_odt/(?P<obj_id>\d+)/$',
                self.admin_site.admin_view(create_file),
                {
                    'model':            Procedure,
                    'file_model':       'admin/procedure/procedure/procedure.odt',
                    'file_type':        'odt',
                },
                name='procedure_procedure_create_odt'
            ),
            url(
                r'^create_pdf/(?P<obj_id>\d+)/$',
                self.admin_site.admin_view(create_file),
                {
                    'model':            Procedure,
                    'file_model':       'admin/procedure/procedure/procedure.odt',
                    'file_type':        'pdf',
                },
                name='procedure_procedure_create_pdf'
            )
        )
        return my_urls + urls


    def get_date_p(self, obj):
        return obj.date_p.strftime("%d/%m/%Y")
    get_date_p.short_description = _('Date')
    get_date_p.admin_order_field = 'date_p'


    def get_organization(self, obj):
        org = """<a href="%(href)s" title="%(title)s">%(owner)s</a><br/>%(contact)s<br/>%(description)s""" % {
            'href'      :   reverse('admin:organization_organization_detail', obj.owner.id),
            'title'     :   ugt("View Organization Detail"),
            'owner'     :   obj.owner.name,
            'contact'   :   obj.contact or "",
            'description':  obj.description or ""
        }
        return org
    get_organization.short_description = _("Bill to")
    get_organization.allow_tags = True
    get_organization.admin_order_field = 'owner'


    def get_detail(self, obj):
        detail = """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/search.png' alt='%(window_title)s' title='%(window_title)s'/></a>""" % {
            'href'  : reverse('admin:procedure_procedure_detail', obj.id),
            'title' : ugt("View Procedure Detail"),
            'url':settings.MEDIA_URL,
            'window_title' : ugt("View Procedure Detail")
        }
        return detail
    get_detail.short_description = _("Det.")
    get_detail.allow_tags = True


    def get_payment_form(self, obj):
        return """<span title="%s">%s</span>""" % (obj.payment_form.name, obj.payment_form.code)
    get_payment_form.short_description = _("Payment Form")
    get_payment_form.allow_tags = True
    get_payment_form.admin_order_field = 'payment_form__code'


    def get_store_cause(self, obj):
        return """<span title="%s">%s</span>""" % (obj.store_cause.name, obj.store_cause.code)
    get_store_cause.short_description = _("Store Cause")
    get_store_cause.allow_tags = True
    get_store_cause.admin_order_field = 'store_cause__code'


    def get_income(self, obj):
        return floatformat(obj.totals['income'], -2)
    get_income.short_description = _('Total Income')


    def get_fee(self, obj):
        return floatformat(obj.totals['fee'], -2)
    get_fee.short_description = _('Total Fee')


    def get_discount(self, obj):
        return floatformat(obj.totals['discount'], -2)
    get_discount.short_description = _('Total Discount')


    def get_vat(self, obj):
        return floatformat(obj.totals['vat'], -2)
    get_vat.short_description = _('Total VAT')

    def get_total_no_vat(self, obj):
        return floatformat(obj.totals['total_no_vat'], -2)
    get_total_no_vat.short_description = _('Total no VAT')

    def get_total(self, obj):
        return floatformat(obj.totals['total'], -2)
    get_total.short_description = _('Total')

    def create_odt(self, obj):
        return """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/writer.gif' alt='%(title)s' title='%(title)s'/></a>""" % {
            'href' : reverse('admin:procedure_procedure_create_odt', obj.id),
            'url':settings.MEDIA_URL,
            'title': ugt("Create Order ODT"),
        }
    create_odt.short_description = _("ODT")
    create_odt.allow_tags = True


    def create_pdf(self, obj):
        return """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/pdf_icon.png' alt='%(title)s' title='%(title)s'/></a>""" % {
            'href' : reverse('admin:procedure_procedure_create_pdf', obj.id),
            'url':settings.MEDIA_URL,
            'title': ugt("Create Order PDF")
        }
    create_pdf.short_description = _("PDF")
    create_pdf.allow_tags = True


    def send_mail(self, obj):
        return """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/mail.png' alt='%(title)s' title='%(title)s'/></a>""" % {
            'href' : reverse('admin:procedure_procedure_sendmail', obj.id),
            'url':settings.MEDIA_URL,
            'title': ugt("Send Mail")
        }
    send_mail.short_description = _("Mail")
    send_mail.allow_tags = True


    def get_procedure_rows(self, obj):
        n_rows = obj.rows.count()
        link = ""
        if n_rows > 0:
            link = """<a href="%(href)s" class="overlay" title="%(title)s">%(num_pr)s</a>""" % {
                'href'  : reverse('admin:procedure_procedure_procedurerows_detail', obj.id),
                'num_pr': n_rows,
                'title' : ugt("View Detail Rows")
            }

        return link
    get_procedure_rows.short_description = _('PR')
    get_procedure_rows.allow_tags = True

    def filter_procedure_rows(self, obj):
        n_rows = obj.rows.count()
        link = ""
        if n_rows > 0:
            link = """<a href="%(href)s?&procedure__id=%(obj_id)s" class="overlay" title="%(title)s">%(num_pr)s</a>""" % {
                'href'  : reverse('admin:procedure_procedurerow_changelist'),
                'obj_id': obj.id,
                'num_pr': n_rows,
                'title' : ugt("View Detail Rows")
            }

        return link
    filter_procedure_rows.short_description = _('PR')
    filter_procedure_rows.allow_tags = True

    def set_accepted(self, request, queryset):
        rows_updated = 0
        for row in queryset:
            row.accepted = True
            row.save()
            rows_updated += 1
        if rows_updated == 1:
            message_bit = "1 Procedure was"
        else:
            message_bit = "%s Procedures were" % rows_updated
        self.message_user(request, "%s successfully accepted." % message_bit)
    set_accepted.short_description = _("Set Procedures as accepted")

    def set_as_delivered(self, request, queryset):
        """
        segna l'ordine e le relative righe come consegnate e non crea il ddt
        """
        rows_updated = 0
        for p in queryset:
            for r in p.rows.all():
                r.ddt = True
                r.save()
            p.set_alldelivered()
            rows_updated += 1
        if rows_updated == 1:
            message_bit = "1 Procedure was"
        else:
            message_bit = "%s Procedures were" % rows_updated
        self.message_user(request, "%s successfully delivery closed." % message_bit)
    set_as_delivered.short_description = _("Set Procedures as delivered")

    def set_as_invoiced(self, request, queryset):
        """
        segna l'ordine e le relative righe come fatturate e non crea la fattura
        """
        rows_updated = 0
        for p in queryset:
            for r in p.rows.all():
                r.invoice=True
                r.save()
            p.set_allinvoiced()
            rows_updated += 1
        if rows_updated == 1:
            message_bit = "1 Procedure was"
        else:
            message_bit = "%s Procedures were" % rows_updated
        self.message_user(request, "%s successfully invoice closed." % message_bit)
    set_as_invoiced.short_description = _("Set Procedures as invoiced")

    def force_as_delivered(self, request, queryset):
        """
        forza l'ordine e le relative righe ordine come consegnate, non crea il
        ddt e cambia la quantità ordinata:
        questo viene utilizzato nel caso in cui si vuole chiudere un ordine e le
        relative righe ordine di materiale non ancora consegnate e modificare
        contemporaneamente anche le quantità della righe ordine
        """
        rows_updated = 0
        for p in queryset:
            for r in p.rows.all():
                r.force_as_delivered()
            p.set_alldelivered()
            rows_updated += 1
        if rows_updated == 1:
            message_bit = "1 Procedure was"
        else:
            message_bit = "%s Procedures were" % rows_updated
        self.message_user(request, "%s successfully delivery closed." % message_bit)
    force_as_delivered.short_description = _("Force Procedures as delivered")

    def force_as_invoiced(self, request, queryset):
        """
        forza l'ordine e le relative righe ordine come fatturate, non crea la
        fattura e cambia la quantità ordinata:
        questo viene utilizzato nel caso in cui si vuole chiudere un ordine e le
        relative righe ordine di materiale non ancora fatturate e modificare
        contemporaneamente anche le quantità della righe ordine
        """
        rows_updated = 0
        for p in queryset:
            for r in p.rows.all():
                r.force_as_invoiced()
            p.set_allinvoiced()
            rows_updated += 1
        if rows_updated == 1:
            message_bit = "1 Procedure was"
        else:
            message_bit = "%s Procedures were" % rows_updated
        self.message_user(request, "%s successfully invoice closed." % message_bit)
    force_as_invoiced.short_description = _("Force Procedures as invoiced")

    def create_ddt(self, request, queryset):
        """
        Crea DDT e RigheDDT partendo da una lista di PRATICHE selezionate
        """
        change = ()
        ddts_created = 0
        rows_updated = 0
        rows_not_updated = 0
        ddt = None

        for obj in queryset.order_by(
            'owner', 'recipient', 'bank', 'payment_form', 'store_cause', 'date_p', 'number', 'id'
        ):
            for row in obj.rows.all():
                ## cambio dati intestazione -> nuovo DDT
                if (obj.owner, obj.recipient, obj.bank, obj.payment_form, obj.store_cause) != change:
                    try:
                        ddt, error = row.create_ddt()
                        if not ddt and error:
                            messages.error(request, error)
                            rows_not_updated += 1
                            continue
                        else:
                            change = (obj.owner, obj.recipient, obj.bank, obj.payment_form, obj.store_cause)
                            ddts_created += 1
                            rows_updated += 1
                    except Exception, e:
                        messages.error(request, e)
                        rows_not_updated += 1
                        continue

                ## stesso intestatario e stessa causale -> vecchio DDT
                else:
                    try:
                        res, error = row.add_to_ddt(ddt=ddt)
                        if not res and error:
                            messages.error(request, error)
                            rows_not_updated += 1
                            continue

                        rows_updated += 1
                    except Exception, e:
                        messages.error(request, e)
                        rows_not_updated += 1
                        continue

        if rows_updated == 0 and rows_not_updated == 0:
            messages.error(request, _("No DDT Row was created"))

        if rows_not_updated > 0:
            if rows_not_updated == 1:
                message_bit = _("1 DDT Row was")
            else:
                message_bit = _("%s DDT Rows were") % rows_not_updated
            messages.error(request, _("%s not created.") % message_bit)

        if rows_updated > 0:
            if rows_updated == 1:
                message_bit = _("1 DDT Row was")
            else:
                message_bit = _("%s DDT Rows were") % rows_updated
            messages.info(request, _("%s successfully created.") % message_bit)

        if ddts_created > 0:
            if ddts_created == 1:
                message_bit = _("1 DDT was")
            else:
                message_bit = _("%s DDT were") % ddts_created
            messages.info(request, _("%s successfully created.") % message_bit)

        return HttpResponseRedirect(request.META['HTTP_REFERER'])
    create_ddt.short_description = _("Create DDT")


    def create_invoice(self, request, queryset):
        rows_updated = 0
        for obj in queryset:
            resp = obj.create_or_append_to_invoice()
            if resp:
                rows_updated += 1
        if rows_updated == 1:
            message_bit = "1 Invoice was"
        else:
            message_bit = "%s Invoices were" % rows_updated
        self.message_user(request, "%s successfully created." % message_bit)
    create_invoice.short_description = _("Create Invoice")

    def get_actions(self, request):
        actions = super(ProcedureAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        return actions

##########################################
## PROCEDURE ROW                        ##
##########################################
class ProcedureRowSearchForm(forms.ModelForm):
    class Meta:
        model          = ProcedureRow
        fields         = ('code', 'store_cause')
        display_fields = (
                           ('procedure__owner__name__istartswith', 'procedure__owner__code'),
                           ('procedure__recipient__name__istartswith', 'procedure__recipient__code'),
                           ('procedure__number__iexact', 'procedure__owner_number'),
                           ('store_cause',),
                           ('procedure__date_p__year', 'search_in_all_years'),
                           ('date_p__gte', 'date_p__lte'),
                           ('product__code', 'code'),
                           ('procedure__accepted', 'procedure__ddt', 'procedure__invoice')
                         )

    def __init__(self, *args, **kws):
        super(ProcedureRowSearchForm, self).__init__(*args, **kws)
        BOOLEAN = (('', '---'), ('1', _('Yes')), ('0', _('No')),)

        self.fields['procedure__owner__name__istartswith'] = forms.CharField(label=_("Owner"), required=False)
        self.fields['procedure__owner__code'] = forms.CharField(label=_("Owner Code"), required=False)
        self.fields['procedure__recipient__name__istartswith'] = forms.CharField(label=_("Recipient"), required=False)
        self.fields['procedure__recipient__code'] = forms.CharField(label=_("Recipient Code"), required=False)
        self.fields['procedure__accepted'] = forms.ChoiceField(label=_("Procedures Accepted"), choices=BOOLEAN, required=False)
        self.fields['procedure__ddt'] = forms.ChoiceField(label=_("Procedures Delivered"), choices=BOOLEAN, required=False)
        self.fields['procedure__invoice'] = forms.ChoiceField(label=_("Procedures Invoiced"), choices=BOOLEAN, required=False)
        self.fields['date_p__gte'] = forms.DateField(label=_("Date Begin"), widget= DateWidget, required=False)
        self.fields['date_p__lte'] = forms.DateField(label=_("Date End"), widget= DateWidget, required=False)
        self.fields['procedure__number__iexact'] = forms.CharField(label=_("Number"), required=False)
        self.fields['procedure__owner_number'] = forms.CharField(label=_("Owner Number"), required=False)
        self.fields['product__code'] = forms.CharField(label=_("Product Code"), required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['procedure__date_p__year'] = forms.CharField(label=_("Year"), required=False)

    # Modifico il filtro di ricerca in modo da filtrare sempre in base all'anno corrente, tranne nel caso in cui
    # nei parametri di ricerca venga specificato un anno oppure venga flaggata la checkbox search_in_all_years
    def change_search_filter(self, lookup_params, qs):
        if not lookup_params.has_key('procedure__date_p__year') and not lookup_params.has_key('search_in_all_years'):
            qs = qs.filter(procedure__date_p__year = datetime.date.today().year)

        if lookup_params.has_key('search_in_all_years'):
            del lookup_params['search_in_all_years']
        return lookup_params, qs


class ProcedureRowAdminForm(AdminModelForm):
    class Meta:
        model = ProcedureRow

    class Media:
        js = ('procedure/js/procedurerow_form.js',)

    def __init__(self, *args, **kws):
        super(ProcedureRowAdminForm, self).__init__(*args, **kws)

        self.fields['product'] = AutocompleteModelChoiceField(
            'procedure_product', label=_("Product"),
            on_select_token='on_select_product',
            on_delete_token='on_delete_product',
        )

        procedure_id = self.get_object_id('procedure')
        procedure = Procedure.objects.get(pk=procedure_id)
        product_id = self.get_object_id('product')
        projects = Project.objects.filter(status=1)
        vats = Vat.objects.none()
        store_causes = StoreCause.objects.none()

        if procedure.store_cause.category == "cus":
            projects = projects.filter(organization__in=(procedure.owner, procedure.recipient))
        if self.fields.get('project'):
            self.fields['project'].queryset = projects
        if not product_id:
            self.fields['vat'].queryset = vats
            self.fields['store_cause'].queryset = store_causes
            self.fields['m_unit'].choices = []


class ProcedureRowAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("procedure/css/admin_procedure.css",)}

    form             = ProcedureRowAdminForm
    search_form      = ProcedureRowSearchForm
    list_per_page    = 30
    fieldsets        = (
                       (None, {'fields': ('status', ('procedure', 'project'), ('product', 'code'), ('store_cause', 'vat'),
                              ('quantity', 'price'), 'm_unit', ('fee', 'discount'), ('perc_discount1', 'perc_discount2'),
                              ('ddt', 'invoice'), 'description','long_description', 'note')}),
                       )
    search_fields    = ('procedure__number', 'product__code',)

    list_display     = ['get_date_p', 'get_number', 'get_detail', 'get_product_code',
                        'quantity', 'price', 'fee', 'discount', 'ddt', 'invoice',
                        'str_total_income', 'str_total_fee', 'str_total_discount',
                        'str_total', 'get_cause_code', 'get_vat_code', 'set_ddt_quantity', 'get_edit_icon', 'get_delete_icon']
    list_display_csv = ['get_date_p', 'get_number', 'get_detail', 'get_product_code',
                        'quantity', 'price', 'fee', 'discount', 'ddt', 'invoice',
                        'str_total_income', 'str_total_fee', 'str_total_discount',
                        'str_total', 'get_cause_code', 'get_vat_code']
    actions            = ["create_ddt", "force_as_delivered", "force_as_invoiced", "set_as_delivered", "set_as_invoiced"]

    # For inline with fancybox
    use_fancybox = True
    father_foreignkey_fancybox = 'procedure'
    default_group_by   = 'product'

    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(ProcedureRowAdmin, self).get_urls()
        my_urls = patterns('',
            url(r'^product/(?P<product_id>\d+)/save/$'   , self.admin_site.admin_view(procedure_views.save_product_row)
                                                        , name='procedure_procedurerows_productsave'),
            url(r'^product/(?P<product_id>\d+)/delete/$'   , self.admin_site.admin_view(procedure_views.delete_product_row)
                                                           , name='procedure_procedurerows_productdelete'),
        )
        return my_urls + urls

    def set_ddt_quantity(self, obj):
        return """<INPUT type="text" name="quantity_%s" size="7" maxlength="7">""" % obj.id
    set_ddt_quantity.short_description = _('DDT Qty')
    set_ddt_quantity.allow_tags = True

    def get_date_p(self, obj):
        return obj.procedure.date_p.strftime("%d/%m/%Y")
    get_date_p.short_description = _('Date')
    get_date_p.admin_order_field = 'procedure__date_p'

    def get_number(self, obj):
        return obj.procedure.number or ""
    get_number.short_description = _('Number')
    get_number.admin_order_field = 'procedure__number'

    def get_detail(self, obj):
        owner = obj.procedure.owner
        recipient = obj.procedure.recipient or ""
        contact = obj.procedure.contact or ""
        if recipient == owner:
            recipient = ""
        return "%s<br/>%s<br/>%s" % (owner, recipient, contact)
    get_detail.short_description = _("Bill to")
    get_detail.allow_tags = True
    get_detail.admin_order_field = 'procedure__owner'

    #def get_product_detail(self, obj):
    #    product = obj.get_description()
    #    project = obj.project or ""
    #    return """<b><a href="%s/">%s</a></b><br/>%s""" % (obj.id, product, project)
    #get_product_detail.short_description = _("Product")
    #get_product_detail.allow_tags = True
    #get_product_detail.admin_order_field = 'description'

    def get_product_code(self, obj):
        code = obj.code or obj.product.code
        return """<b><a href="%s/">%s</a></b>""" % (obj.id, code)
    get_product_code.short_description = _("Product Code")
    get_product_code.allow_tags = True

    def get_cause_code(sel, obj):
        return obj.store_cause.code
    get_cause_code.short_description = _('SC')

    def get_vat_code(self, obj):
        return obj.vat.code
    get_vat_code.short_description = _('VC')

    def get_ddt_date(self, obj):
        ddtrows = obj.ddtrows.all().order_by('-ddt__date_d')
        if ddtrows:
            return date_format(ddtrows[0].ddt.date_d)
        return ''
    get_ddt_date.short_description = _("Last DDT")

    def str_total_income(self, obj):
        return floatformat(obj.get_total_income(), -2)
    str_total_income.short_description = _('Total Income')

    def str_total_fee(self, obj):
        return floatformat(obj.get_total_fee(), -2)
    str_total_fee.short_description = _('Total Fee')

    def str_total_discount(self, obj):
        return floatformat(obj.get_total_discount(), -2)
    str_total_discount.short_description = _('Total Discount')

    def str_total(self, obj):
        return floatformat(obj.get_total(), -2)
    str_total.short_description = _('Total')

    def create_ddt(self, request, queryset):
        """
        Crea DDT e Righe DDT da Righe Pratica selezionate
        """
        change = ()
        ddts_created = 0
        ddts_not_created = 0
        rows_updated = 0
        rows_not_updated = 0
        post = request.POST

        ## ordino il queryset
        queryset = queryset.order_by(
            'procedure__owner', 'procedure__recipient',
            'procedure__bank', 'procedure__payment_form',
            'procedure__store_cause', 'procedure__date_p',
            'procedure__number', 'procedure__id'
        )
        for obj in queryset:
            ddt_qty = post.get('quantity_%s' % obj.id, None)
            try:
                ddt_qty = decimal.Decimal(ddt_qty.replace(',','.'))
            except Exception, e:
                messages.error(request, _("Wrong Quantity"))
                rows_not_updated += 1
                continue

            ## cambio dati intestazione -> nuovo DDT
            if ((
                obj.procedure.owner, obj.procedure.recipient,
                obj.procedure.bank, obj.procedure.payment_form,
                obj.procedure.store_cause
            ) != change):
                try:
                    ddt, error = obj.create_ddt(quantity=ddt_qty)

                    if not ddt and error:
                        messages.error(request, error)
                        rows_not_updated += 1
                        ddts_not_created += 1
                        continue
                    else:
                        rows_updated += 1
                        ddts_created += 1
                        change = (
                            obj.procedure.owner, obj.procedure.recipient,
                            obj.procedure.bank, obj.procedure.payment_form,
                            obj.procedure.store_cause
                        )

                except Exception, e:
                    messages.error(request, e)
                    ddts_not_created += 1
                    rows_not_updated += 1
                    continue

            ## stessi dati intestazione -> vecchio DDT
            else:
                try:
                    res, error = obj.add_to_ddt(ddt=ddt, quantity=ddt_qty)

                    if not res and error:
                        messages.error(request, error)
                        rows_not_updated += 1
                        continue

                    rows_updated += 1
                except Exception, e:
                    messages.error(request, e)
                    rows_not_updated += 1
                    continue

        if rows_updated == 0 and rows_not_updated == 0:
            messages.error(request, _("No DDT Row was created"))
        if ddts_created == 0 and ddts_not_created == 0:
            messages.error(request, _("No DDT was created"))

        if rows_not_updated > 0:
            if rows_not_updated == 1:
                message_bit = _("1 DDT Row was")
            else:
                message_bit = _("%s DDT Rows were") % rows_not_updated
            messages.error(request, _("%s not created.") % message_bit)

        if ddts_not_created > 0:
            if ddts_not_created == 1:
                message_bit = _("1 DDT was")
            else:
                message_bit = _("%s DDT were") % ddts_not_created
            messages.error(request, _("%s not created.") % message_bit)

        if rows_updated > 0:
            if rows_updated == 1:
                message_bit = _("1 DDT Row was")
            else:
                message_bit = _("%s DDT Rows were") % rows_updated
            messages.info(request, _("%s successfully created.") % message_bit)

        if ddts_created > 0:
            if ddts_created == 1:
                message_bit = _("1 DDT was")
            else:
                message_bit = _("%s DDT were") % ddts_created
            messages.info(request, _("%s successfully created.") % message_bit)

        return HttpResponseRedirect(request.META['HTTP_REFERER'])
    create_ddt.short_description = _("Create DDT")

    def set_as_delivered(self, request, queryset):
        """
        segna la riga ordine come consegnata e non crea il ddt
        """
        rows_updated = 0
        for r in queryset:
            r.ddt = True
            r.save()
            rows_updated += 1
        if rows_updated == 1:
            message_bit = "1 Procedure Row was"
        else:
            message_bit = "%s Procedure Rows were" % rows_updated
        self.message_user(request, "%s successfully delivery closed." % message_bit)
    set_as_delivered.short_description = _("Set Procedure Rows as delivered")

    def set_as_invoiced(self, request, queryset):
        """
        segna la riga ordine come fatturata e non crea la fattura
        """
        rows_updated = 0
        for r in queryset:
            r.invoice = True
            r.save()
            rows_updated += 1
        if rows_updated == 1:
            message_bit = "1 Procedure Row was"
        else:
            message_bit = "%s Procedure Rows were" % rows_updated
        self.message_user(request, "%s successfully invoice closed." % message_bit)
    set_as_invoiced.short_description = _("Set Procedure Rows as invoiced")

    def force_as_delivered(self, request, queryset):
        """
        forza la riga ordine come consegnata, non crea il ddt e cambia la quantità ordinata:
        questo viene utilizzato nel caso in cui si vuole chiudere una riga ordine di
        materiale non ancora consegnato e modificare contemporaneamente anche la
        quantità della riga ordine
        """
        rows_updated = 0
        for r in queryset:
            r.force_as_delivered()
            r.procedure.set_alldelivered()
            rows_updated += 1
        if rows_updated == 1:
            message_bit = "1 Procedure Row was"
        else:
            message_bit = "%s Procedure Rows were" % rows_updated
        self.message_user(request, "%s successfully delivery closed." % message_bit)
    force_as_delivered.short_description = _("Force Procedure Rows as delivered")

    def force_as_invoiced(self, request, queryset):
        """
        forza la riga ordine come fatturata, non crea la fattura e cambia la quantità ordinata:
        questo viene utilizzato nel caso in cui si vuole chiudere una riga ordine di
        materiale non ancora fatturata e modificare contemporaneamente anche la
        quantità della riga ordine
        """
        rows_updated = 0
        for r in queryset:
            r.force_as_invoiced()
            r.procedure.set_allinvoiced()
            rows_updated += 1
        if rows_updated == 1:
            message_bit = "1 Procedure Row was"
        else:
            message_bit = "%s Procedure Rows were" % rows_updated
        self.message_user(request, "%s successfully invoice closed." % message_bit)
    force_as_invoiced.short_description = _("Force Procedure Rows as invoiced")

    def get_actions(self, request):
        actions = super(ProcedureRowAdmin, self).get_actions(request)
        del actions['set_active']
        del actions['set_disactive']
        return actions

class TotalOrganizationInvoicedAdmin(ExtendibleModelAdmin):
    list_display = ['organization', 'year', 'total']
    search_fields = ['organization__code', 'organization__name', 'year']

########################################
# VAT INLINE                           #
########################################
class VatRowAdminForm(AdminModelForm):
    class Meta:
        model = VatRow


class VatRowInline(admin.TabularInline):
    model = VatRow
    form = VatRowAdminForm
    extra = 0
    fields = ('vat', 'total_income', 'total_fee', 'total_discount', 'total_vat',)

########################################
# REGISTER                             #
########################################
admin.site.register(Settings, SettingsAdmin)
admin.site.register(ProcedureRow, ProcedureRowAdmin)
admin.site.register(Procedure, ProcedureAdmin)
admin.site.register(VatRow)
admin.site.register(TotalOrganizationInvoiced, TotalOrganizationInvoicedAdmin)

########################################
# AUTOCOMPLETE REGISTER                #
########################################
def get_procedure_product(request, query):
    return Product.objects.filter(status=1).order_by('code')

def get_autocomplete_procedure_owner(request, query):
    results = []
    try:
        org_exact = Organization.objects.get(code=query)
        results.append([org_exact.id, "%s - %s" % (org_exact.code, org_exact.name)])
    except:
        pass

    q1 = Q(name__istartswith=query)
    q2 = Q(code__istartswith=query)
    for other_org in Organization.objects.exclude(code=query).filter(status=1).filter(q1|q2)[:10]:
        results.append([other_org.id, "%s - %s" % (other_org.code, other_org.name)])
    return results

autocomplete.register('procedure_owner'    , model = Organization, views = get_autocomplete_procedure_owner)
autocomplete.register('procedure_recipient', model = Organization, views = get_autocomplete_procedure_owner)
autocomplete.register('procedure_product'  , model = Product, views = get_procedure_product
                                           , search_fields=('name', 'code', 'production_code__icontains')
                                           , limit=10)
