# -*- coding: utf-8 -*-

from django.utils.translation import ugettext as _
from django.core.urlresolvers import reverse
from admin_tools.menu import items, Menu
from jumbo.conf.admin_menu import GenericMenuItem
from jumbo.middleware.thread_local import get_current_user

#STRUTTURA MENU PROCEDURE
#
#I Livello  |    II livello                |   III livello             |   IV livello
#
# PROCEDURE |-->  customer_procedure_main  |-->  customer_procedure    |-->   add_customer_procedure
#           |                              |                           |      list_customer_procedure
#           |                              |                           |      customer_not_accepted_procedure
#           |                              |                           |      customer_not_delivered_procedure
#           |                              |                           |      customer_delivered_procedure
#           |                              |                           |      customer_not_invoiced_procedure
#           |                              |                           |      customer_invoiced_procedure

#           |                              |-->  customer_procedurerow |-->   list_customer_procedurerow
#           |                              |                           |      list_customer_procedurerow_groupbycode
#           |                              |                           |      delivered_customer_procedurerow
#           |                              |                           |      not_delivered_customer_procedurerow

#           |-->  supplier_procedure_main  |-->  supplier_procedure    |-->   add_supplier_procedure
#           |                              |                           |      list_supplier_procedure
#           |                              |                           |      supplier_not_accepted_procedure
#           |                              |                           |      supplier_not_delivered_procedure
#           |                              |                           |      supplier_delivered_procedure
#           |                              |                           |      supplier_not_invoiced_procedure
#           |                              |                           |      supplier_invoiced_procedure

#           |                              |-->  supplier_procedurerow |-->   list_supplier_procedurerow
#           |                              |                           |      list_supplier_procedurerow_groupbycode
#           |                              |                           |      delivered_supplier_procedurerow
#           |                              |                           |      not_delivered_supplier_procedurerow


def add_extended_menuitems(children, current_app):
    user = get_current_user()
    gm = GenericMenuItem('procedure', [])

#-------------MENU-CUSTOMER-PROCEDURE----------------------------------------------------------#

#------------IV LIVELLO nel menu nero----------------------------------------------------------#

    # creo manualmente i figli per gli ordini cliente
    add_customer_procedure = items.MenuItem(
        title=_('Add procedure'),
        children=[], # nessun figlio perchè non ci sono quinti livelli
        url = '%s?category=cus' % reverse('admin:procedure_procedure_add')
    )
    list_customer_procedure = items.MenuItem(
        title=_('List procedures'),
        children=[],
        url = '%s?owner__organization_type__category=cus' % reverse('admin:procedure_procedure_changelist')
    )
    list_customer_procedurerow_groupbycode = items.MenuItem(
        title=_('List procedures row by code'),
        children=[],
        url = '%s?groupby=code' % reverse('admin:procedure_procedurerow_changelist')
    )
    customer_not_accepted_procedure= items.MenuItem(
        title=_('Not accepted procedures'),
        children=[],
        url = '%s?owner__organization_type__category=cus&accepted=0' % reverse('admin:procedure_procedure_changelist')
    )
    customer_not_delivered_procedure= items.MenuItem(
        title=_('Not delivered procedures'),
        children=[],
        url = '%s?owner__organization_type__category=cus&accepted=1&ddt=0' % reverse('admin:procedure_procedure_changelist')
    )
    customer_delivered_procedure= items.MenuItem(
        title=_('Delivered procedures'),
        children=[],
        url = '%s?owner__organization_type__category=cus&accepted=1&ddt=1' % reverse('admin:procedure_procedure_changelist')
    )
    customer_not_invoiced_procedure= items.MenuItem(
        title=_('Not invoiced procedures'),
        children=[],
        url = '%s?owner__organization_type__category=cus&accepted=1&invoice=0' % reverse('admin:procedure_procedure_changelist')
    )
    customer_invoiced_procedure= items.MenuItem(
        title=_('Invoiced procedures'),
        children=[],
        url = '%s?owner__organization_type__category=cus&accepted=1&invoice=1' % reverse('admin:procedure_procedure_changelist')
    )

    # creo manualmente i figli per le righe ordine cliente
    list_customer_procedurerow = items.MenuItem(
        title=_('List rows'),
        children=[],
        url = '%s?procedure__owner__organization_type__category=cus' % reverse('admin:procedure_procedurerow_changelist')
    )
    delivered_customer_procedurerow= items.MenuItem(
        title=_('Delivered rows'),
        children=[],
        url = '%s?procedure__owner__organization_type__category=cus&ddt=1' % reverse('admin:procedure_procedurerow_changelist')
    )
    not_delivered_customer_procedurerow= items.MenuItem(
        title=_('Not delivered rows'),
        children=[],
        url = '%s?procedure__owner__organization_type__category=cus&ddt=0' % reverse('admin:procedure_procedurerow_changelist')
    )
    invoiced_customer_procedurerow= items.MenuItem(
        title=_('Invoiced procedures rows'),
        children=[],
        url = '%s?procedure__owner__organization_type__category=cus&invoice=1' % reverse('admin:procedure_procedurerow_changelist')
    )
    not_invoiced_customer_procedurerow= items.MenuItem(
        title=_('Not invoiced procedures rows'),
        children=[],
        url = '%s?procedure__owner__organization_type__category=cus&invoice=0' % reverse('admin:procedure_procedurerow_changelist')
    )

#------------III LIVELLO nel menu nero----------------------------------------------------------#

    # creo manualmente il menu padre per gli ordini cliente
    customer_procedure=items.MenuItem(
        title=_('Procedures'),
        children=[],
        url = '%s?owner__organization_type__category=cus' % reverse('admin:procedure_procedure_changelist')
    )

    # creo manualmente il menu padre per le righe d'ordine cliente
    customer_procedurerow=items.MenuItem(
        title=_('Procedure rows'),
        children=[],
        url = '%s?owner__organization_type__category=cus' % reverse('admin:procedure_procedurerow_changelist')
    )

#------------II LIVELLO nel menu nero----------------------------------------------------------#

    customer_procedure_main=items.MenuItem(
        title=_('Customer procedure'),
        children=[],
        url = '%s?owner__organization_type__category=cus' % reverse('admin:procedure_procedure_changelist')
    )

#------------GESTIONE PERMESSI----------------------------------------------------------------#

    # appendo i figli creati sopra solo se l'utente ha i relativi permessi
    if user.has_perm('procedure.add_customer_procedure'):
        customer_procedure.children.append(add_customer_procedure)
    if user.has_perm('procedure.view_customer_procedure'):
        customer_procedure.children.append(list_customer_procedure)
        customer_procedure.children.append(customer_not_accepted_procedure)
        customer_procedure.children.append(customer_not_delivered_procedure)
        customer_procedure.children.append(customer_delivered_procedure)
        customer_procedure.children.append(customer_not_invoiced_procedure)
        customer_procedure.children.append(customer_invoiced_procedure)
        customer_procedure_main.children.append(customer_procedure)
    if user.has_perm('procedure.view_customer_procedurerow'):
        customer_procedurerow.children.append(list_customer_procedurerow)
        customer_procedurerow.children.append(list_customer_procedurerow_groupbycode)
        customer_procedurerow.children.append(delivered_customer_procedurerow)
        customer_procedurerow.children.append(not_delivered_customer_procedurerow)
        customer_procedurerow.children.append(invoiced_customer_procedurerow)
        customer_procedurerow.children.append(not_invoiced_customer_procedurerow)
        customer_procedure_main.children.append(customer_procedurerow)
    if user.has_perm('procedure.view_customer_procedure'):
        # senza permessi di view su customer_procedure non appendo nemmeno il padre
        gm.add_father(customer_procedure_main)

#--------FINE-MENU-CUSTOMER-PROCEDURE--------------------------------------------------------#




#---------MENU-SUPPLIER-PROCEDURE--------  --------------------------------------------------#

#------------IV LIVELLO nel menu nero----------------------------------------------------------#

    # creo manualmente i figli per gli ordini fornitore
    add_supplier_procedure = items.MenuItem(
        title=_('Add procedure'),
        children=[],
        url = '%s?category=sup' % reverse('admin:procedure_procedure_add')
    )
    list_supplier_procedure = items.MenuItem(
        title=_('List procedures'),
        children=[],
        url = '%s?owner__organization_type__category=sup' % reverse('admin:procedure_procedure_changelist')
    )
    supplier_not_accepted_procedure= items.MenuItem(
        title=_('Not accepted procedures'),
        children=[],
        url = '%s?owner__organization_type__category=sup&accepted=0' % reverse('admin:procedure_procedure_changelist')
    )
    supplier_not_delivered_procedure= items.MenuItem(
        title=_('Not delivered procedures'),
        children=[],
        url = '%s?owner__organization_type__category=sup&accepted=1&ddt=0' % reverse('admin:procedure_procedure_changelist')
    )
    supplier_delivered_procedure= items.MenuItem(
        title=_('Delivered procedures'),
        children=[],
        url = '%s?owner__organization_type__category=sup&accepted=1&ddt=1' % reverse('admin:procedure_procedure_changelist')
    )
    supplier_not_invoiced_procedure= items.MenuItem(
        title=_('Not invoiced procedures'),
        children=[],
        url = '%s?owner__organization_type__category=sup&accepted=1&invoice=0' % reverse('admin:procedure_procedure_changelist')
    )
    supplier_invoiced_procedure= items.MenuItem(
        title=_('Invoiced procedures'),
        children=[],
        url = '%s?owner__organization_type__category=sup&accepted=1&invoice=1' % reverse('admin:procedure_procedure_changelist')
    )

    # creo manualmente i figli per le righe ordine fornitore
    list_supplier_procedurerow = items.MenuItem(
        title=_('List procedures row'),
        children=[],
        url = '%s?procedure__owner__organization_type__category=sup' % reverse('admin:procedure_procedurerow_changelist')
    )
    list_supplier_procedurerow_groupbycode = items.MenuItem(
        title=_('List procedures row by code'),
        children=[],
        url = '%s?groupby=code' % reverse('admin:procedure_procedurerow_changelist')
    )
    delivered_supplier_procedurerow= items.MenuItem(
        title=_('Delivered procedures rows'),
        children=[],
        url = '%s?procedure__owner__organization_type__category=sup&ddt=1' % reverse('admin:procedure_procedurerow_changelist')
    )
    not_delivered_supplier_procedurerow= items.MenuItem(
        title=_('Not delivered procedures rows'),
        children=[],
        url = '%s?procedure__owner__organization_type__category=sup&ddt=0' % reverse('admin:procedure_procedurerow_changelist')
    )
    invoiced_supplier_procedurerow= items.MenuItem(
        title=_('Invoiced procedures rows'),
        children=[],
        url = '%s?procedure__owner__organization_type__category=sup&invoice=1' % reverse('admin:procedure_procedurerow_changelist')
    )
    not_invoiced_supplier_procedurerow= items.MenuItem(
        title=_('Not invoiced procedures rows'),
        children=[],
        url = '%s?procedure__owner__organization_type__category=sup&invoice=0' % reverse('admin:procedure_procedurerow_changelist')
    )

#------------III LIVELLO nel menu nero----------------------------------------------------------#

    # creo manualmente il menu padre per gli ordini fornitore
    supplier_procedure=items.MenuItem(
        title=_('Procedures'),
        children=[],
        url = '%s?owner__organization_type__category=sup' % reverse('admin:procedure_procedure_changelist')
    )

    supplier_procedurerow=items.MenuItem(
        title=_('Procedures rows'),
        children=[],
        url = '%s?owner__organization_type__category=cus' % reverse('admin:procedure_procedurerow_changelist')
    )

#------------II LIVELLO nel menu nero----------------------------------------------------------#

    supplier_procedure_main=items.MenuItem(
        title=_('Supplier procedure'),
        children=[],
        url = '%s?owner__organization_type__category=sup' % reverse('admin:procedure_procedure_changelist')
    )

#------------GESTIONE PERMESSI----------------------------------------------------------------#

    # appendo i figli creati sopra solo se l'utente ha i relativi permessi
    if user.has_perm('procedure.add_supplier_procedure'):
        supplier_procedure.children.append(add_supplier_procedure)
    if user.has_perm('procedure.view_supplier_procedure'):
        supplier_procedure.children.append(list_supplier_procedure)
        supplier_procedure.children.append(supplier_not_accepted_procedure)
        supplier_procedure.children.append(supplier_not_delivered_procedure)
        supplier_procedure.children.append(supplier_delivered_procedure)
        supplier_procedure.children.append(supplier_not_invoiced_procedure)
        supplier_procedure.children.append(supplier_invoiced_procedure)
        supplier_procedure_main.children.append(supplier_procedure)
    if user.has_perm('procedure.view_supplier_procedurerow'):
        supplier_procedurerow.children.append(list_supplier_procedurerow)
        supplier_procedurerow.children.append(list_supplier_procedurerow_groupbycode)
        supplier_procedurerow.children.append(delivered_supplier_procedurerow)
        supplier_procedurerow.children.append(not_delivered_supplier_procedurerow)
        supplier_procedurerow.children.append(invoiced_supplier_procedurerow)
        supplier_procedurerow.children.append(not_invoiced_supplier_procedurerow)
        supplier_procedure_main.children.append(supplier_procedurerow)
    if user.has_perm('procedure.view_supplier_procedure'):
        # senza permessi di view su customer_procedure non appendo nemmeno il padre
        gm.add_father(supplier_procedure_main)

#------------FINE-MENU-SUPPLIER-PROCEDURE- -------------------------------------------------------#


    menu = gm.get_menu()
    [ children.append(proc_menu) for proc_menu in menu ]


#------------I LIVELLO nel menu separato----------------------------------------------------------#


# Personalizzo il menu dei primi livelli
def get_root_menu(children):
    user = get_current_user()
    if not user.has_perm("procedure.view_procedure"):
        return None

    root_menu = items.MenuItem(
        title=_('Procedures'),
        children=[],
        url = reverse('admin:procedure_procedure_changelist')
    )
    children.append(root_menu)