# -*- coding: utf-8 -*-

import re, os, os.path
import decimal

from django import template
from django.http import Http404, HttpResponse, HttpResponseForbidden, HttpResponseRedirect
from django.utils.translation import ugettext as _
from django.core.urlresolvers import reverse
from django.utils import simplejson
from django.utils.encoding import force_unicode
from django.shortcuts import render_to_response
from django import forms
from django.core.mail import EmailMessage
from django.contrib import messages
from django.utils.formats import number_format
from django.template.loader import render_to_string

from jumbo.utils.functions import get_settings
from jumbo.views.generic.create_oo_file import create_file
from jumbo.forms.fields import MultiEmailField
from procedure.models import Procedure, ProcedureRow
from organization.models import Organization, Currency, Vat, Delivery, Commision, M2MtoVector
from payment.models import PaymentForm
from store.models import StoreCause
from product.models import OrganizationPriceList, PriceListItem, Product


def get_result(queryset, obj_default=None):
    ### restituisce un dizionario per popolare le select che
    result = []
    if not queryset:
        return result
    for obj in queryset:
        result.append({'value': obj.pk, 'display': unicode(obj), 'is_default': (obj == obj_default)})
    return result


def procedure_cus(request):
    pass


def procedurerows(request, procedure_id):
    try:
        pr = Procedure.objects.get(pk=procedure_id)
    except Exception, e:
        #FIXME
        #return render_to_response('admin/generic_error.html', error=_("You must enable the Asterisk Application."))
        return None

    return render_to_response('admin/procedure/procedure/procedure_procedurerows.html', pr=pr)


def chained_payment_form(request, app, model, field, value):
    owner_id = value
    payments = PaymentForm.objects.filter(status=1)
    payment = Organization.objects.get(pk=owner_id).payment_form or payments[0]
    return get_result(payments, payment)


def chained_store_cause(request, app, model, field, value):
    owner_id = value
    category = Organization.objects.get(pk=owner_id).organization_type.category
    cats = StoreCause.objects.filter(category=category, status=1)
    cat_default = None
    try:
        cat_default = StoreCause.objects.get(default=True, status=1, category=category)
    except:
        cat_defaults = StoreCause.objects.filter(status=1, category=category)
        if cat_defaults:
            cat_default = cat_defaults[0]
    result = []
    for cat in cats:
        try:
            name = unicode(cat.name)
        except:
            name = None
        result.append({'value': cat.pk, 'display': name, 'is_default': (cat == cat_default)})
    return result


#def chained_store_cause_row(request, app, model, field, value):
    #store_cause_id = value
    #store_cause = StoreCause.objects.get(pk=store_cause_id)
    #store_causes = StoreCause.objects.filter(status=1, category=store_cause.category)
    #return get_result(store_causes, store_cause)


def chained_bank(request, app, model, field, value):
    owner_id = value
    banks = Organization.objects.get(pk=owner_id).banks.filter(status=1)
    return get_result(banks)


def chained_currency(request, app, model, field, value):
    owner_id = value
    currency = Organization.objects.get(pk=owner_id).currency
    if currency:
        curr = Currency.objects.filter(pk=currency.id)
    else:
        curr = Currency.objects.filter(status=1)
    return get_result(curr, currency)


def chained_vat(request, app, model, field, value):
    owner_id = value
    vat = Organization.objects.get(pk=owner_id).vat
    if vat:
        v = Vat.objects.filter(pk=vat.id)
    else:
        v = Vat.objects.filter(status=1)
    return get_result(v, vat)


def chained_delivery(request, app, model, field, value):
    owner_id = value
    delivery = Organization.objects.get(pk=owner_id).delivery
    if delivery:
        dlv = Delivery.objects.filter(pk=delivery.id)
    else:
        dlv = Delivery.objects.filter(status=1)
    return get_result(dlv, delivery)


def chained_price_lists(request, app, model, field, value):
    owner_id = value
    price_lists = Organization.objects.get(pk=owner_id).price_lists.all()
    result = []
    if price_lists:
        pl = price_lists
    else:
        return result
    default = pl[0]
    for obj in pl:
        try:
            name = unicode(obj) #, obj.discount1, obj.discount2))
        except:
            name = None
        result.append({'value': obj.pk, 'display': name, 'is_default': (obj == default)})
    return result


def chained_commision(request, app, model, field, value):
    owner_id = value
    commisions = Organization.objects.get(pk=owner_id).commisions.all()
    if commisions:
        com = commisions
    else:
        com = Commision.objects.all()
    result = []
    default = com[0]
    for obj in com:
        result.append({'value': obj.pk, 'display': unicode('%s - %s' % (obj.agent.name, obj.commision)), 'is_default': (obj == default)})
    return result


def chained_vector(request, app, model, field, value):
    owner_id = value
    vectors = Organization.objects.get(pk=owner_id).vectors.filter(vector__status=1)
    if vectors:
        vec = vectors
    else:
        vec = Organization.objects.filter(organization_type__category='vec', status=1)
    return get_result(vec)


def chained_shipping_address(request, app, model, field, value):
    recipient_id = value
    addresses = Organization.objects.get(pk=recipient_id).addresses.filter(address_scopes=3)
    if addresses:
        return get_result(addresses, addresses[0])
    return []


def get_price(request, owner_id, product_id):
    price = PriceListItem.objects.filter(product__id=product_id, price_list__organizations__in=owner_id)
    return price


def save_product_row(request, product_id):
    qty         = request.GET['qty'].replace(',', '.')
    vat         = request.GET['vat']
    sc          = request.GET['store_cause']
    pr_id       = request.GET['order_id']
    prrow_id    = request.GET['prrow_id']
    price       = request.GET['price'].replace(',', '.')
    opl         = request.GET['org_price_list']
    on_change   = request.GET['on_change']
    result = "OK"
    error_message = ""

    no_discount = False
    if request.GET['no_discount'] == 'true':
        no_discount = True

    product = Product.objects.get(pk=product_id)

    # on_change e' undefined quando viene scelto un prodotto
    if on_change == 'undefined':
        try:
            pl = OrganizationPriceList.objects.get(pk = opl)
            price = pl.get_product_price(product)
        except:
            price = 0
    else:
        try:
            price = decimal.Decimal(price)
        except Exception, e:
            price = 0

    try:
        qty = decimal.Decimal(qty)
    except Exception, e:
        qty = 0

    if prrow_id:
        pr = ProcedureRow.objects.get(pk=prrow_id)
    else:
        pr = ProcedureRow()
    pr.procedure_id = pr_id
    pr.product_id = product_id
    if vat != 'undefined':
        pr.vat_id = vat
    pr.store_cause_id = sc
    pr.quantity = qty
    pr.m_unit = product.m_unit
    pr.price = price
    pr.not_apply_discount = no_discount
    pr.save()

    totals = Procedure.objects.get(pk=pr_id).totals

    json = simplejson.dumps({"price" : number_format(price, 4),
                             "net_price" : number_format(pr.get_net_price(), 4),
                             "total_income_row" : number_format(pr.get_total_income(), 4),
                             "row_id": request.GET['row_id'],
                             "order_id": request.GET['order_id'],
                             "procedurerow_id": pr.id,
                             "total_income": number_format(totals['income'], 4),
                             "total_fee": number_format(totals['fee'], 2),
                             "total_discount": number_format(totals['discount'], 2),
                             "total_vat": number_format(totals['vat'], 2),
                             "total_total": number_format(totals['total'], 4),
                             "result": result,
                             "error_message": error_message
                             })
    return HttpResponse(json, mimetype='application/json')


def delete_product_row(request, product_id):
    prid = request.GET['procedurerow_id']
    if prid:
        try:
            pr = ProcedureRow.objects.get(id = prid)
            pr.delete()
            totals = pr.procedure.totals
            json = simplejson.dumps({
                             "total_income": "%.2f" % totals['income'],
                             "total_fee": "%.2f" % totals['fee'],
                             "total_discount": "%.2f" % totals['discount'],
                             "total_vat": "%.2f" % totals['vat'],
                             "total_total": "%.2f" % totals['total']
                    })
        except Exception, e:
            json = simplejson.dumps({})
            pass

    return HttpResponse(json, mimetype='application/json')


class SendMailForm(forms.Form):
    mails       = forms.MultipleChoiceField(required=False, label=_("Send To"))
    other_mails = MultiEmailField(required=False, label=_("Other Mail Addresses"))
    attention_of = forms.CharField(required=False, label=_("Attention of"))

    def clean(self):
        cleaned_data = self.cleaned_data

        mails = cleaned_data.get('mails', [])
        other_mails = cleaned_data.get('other_mails', '')

        if not mails and not other_mails:
            raise forms.ValidationError("Field Mails or Other Mail Addresses is required.")

        return cleaned_data


def procedure_send_mail(request, procedure_id):
    try:
        pr = Procedure.objects.get(pk=procedure_id)
        opts = pr._meta
    except Exception, e:
        raise Http404(_('The object with primary key %s does not exist.') % procedure_id)

    mails = set(pr.owner.get_all_mails_for_procedure())
    if pr.owner != pr.recipient:
        for rec_email in pr.recipient.get_all_mails_for_procedure():
            mails.add(rec_email)


    # Recupero la lingua del primo contatto per decidere in che lingua
    # creare e mandare la fattura
    odt_language = 'en'
    contacts = pr.owner.get_procedure_contacts()[:1]

    if contacts and contacts[0].language:
        odt_language = contacts[0].language.code

    lmails = [ (email[0], email[1]) for email in mails ]

    if request.POST:
        form = SendMailForm(request.POST)
        form.fields['mails'].choices = lmails
        if form.is_valid():
            f = create_file(request, Procedure, procedure_id,
                            file_model = 'admin/procedure/procedure/procedure.odt',
                            file_type  = 'pdf',
                            get_raw_string = True,
                            language = odt_language,
                            extra_context={'form':form}
                            )

            all_mails = set()

            fromail = get_settings("procedure.settings", "PROCEDURE_SEND_MAIL_FROM_EMAIL")
            if pr.creator.email:
                fromail = pr.creator.email
                all_mails.add(pr.creator.email)

            for mail in form.cleaned_data['mails']:
                if mail:
                    all_mails.add(mail)

            for mail in form.cleaned_data['other_mails']:
                if mail:
                    all_mails.add(mail)

            if all_mails:
                subject = render_to_string("admin/procedure/procedure/mail_subject.txt", {'pr':pr, 'form':form})
                # Email subject *must not* contain newlines
                subject = ''.join(subject.splitlines())
                body    = render_to_string("admin/procedure/procedure/mail_body.txt", {'pr':pr, 'form':form})

                email_msg = EmailMessage(subject, body, fromail, [], all_mails)

                email_msg.attach('ordine_%s.pdf' % pr.date_p.strftime("%Y%m%d"),
                                 f.read(),
                                'application/pdf')

                email_msg.send(fail_silently=False)
            messages.success(request, _('Mail Sent Successfully.'))
            return HttpResponseRedirect(reverse('admin:procedure_procedure_changelist'))
    else:
        form = SendMailForm()
        form.fields['mails'].choices = lmails

    context = {
        'title': _('Send Mail - Procedure : %s') % force_unicode(pr),
        'object_id': procedure_id,
        'obj': pr,
        'is_popup': request.REQUEST.has_key('_popup'),
        'app_label': opts.app_label,
        'opts': opts,
        'form':form
    }
    context_instance = template.RequestContext(request, current_app='procedure')
    return render_to_response('admin/procedure/procedure/sendmail.html', context,
                              context_instance=context_instance)
