# -*- coding: utf-8 -*-
from django.db import connection
from invoice.models import Invoice

def dictfetchall(cursor):
    "Returns all rows from a cursor as a dict"
    desc = cursor.description
    return [
        dict(zip([col[0] for col in desc], row))
        for row in cursor.fetchall()
    ]

def year_topten_sold_products(years=None, others_where='', limit=''):
    """
    Returns Top Ten Organizations per year
    years: is a list of years
    """
    where = ''
    if years:
        where = " AND DATE_PART('year', i.date_i) = %s" % years
    
    if limit:
        limit = "LIMIT %i" % limit
    
    cursor = connection.cursor()
    query = """
        SELECT
            COUNT(*) AS products,
            DATE_TRUNC('year', i.date_i) AS year,
            p.name,
            SUM(ir.price * ir.quantity) AS total_income,
            SUM(ir.fee * ir.quantity) AS total_fee,
            SUM(ir.discount * ir.quantity) AS total_discount,
            SUM((ir.price + ir.fee - ir.discount) * ir.quantity * (v.aliquot_perc / 100)) AS total_vat,
            SUM((ir.price + ir.fee - ir.discount) * ir.quantity) AS total_no_vat,
            SUM((ir.price + ir.fee - ir.discount) * ir.quantity * (1 + v.aliquot_perc / 100)) AS total
        FROM
            product_product p
            JOIN invoice_invoicerow ir on ir.product_id=p.id
            JOIN invoice_invoice i ON i.id = ir.invoice_id
            JOIN organization_vat v ON v.id = ir.vat_id
        WHERE 1=1
            %s %s
        GROUP BY year, ir.product_id, p.name
        ORDER BY year, total DESC
        %s
    """ % (where, others_where, limit)
    cursor.execute(query)
    dictionary = dictfetchall(cursor)
    return dictionary

def month_topten_sold_products(years=None, months=None, others_where='', limit=''):
    """
    Returns Top Ten Organizations per year
    years: is a list of years
    """
    where = ''
    if years:
        where = " AND DATE_PART('year', i.date_i) = %s" % years
    if months:
        where += " AND DATE_PART('month', i.date_i) = %s" % months
    
    if limit:
        limit = "LIMIT %i" % limit
    
    cursor = connection.cursor()
    query = """
        SELECT
            COUNT(*) AS products,
            DATE_TRUNC('year', i.date_i) AS year,
            DATE_TRUNC('month', i.date_i) AS month,
            p.name,
            SUM(ir.price * ir.quantity) AS total_income,
            SUM(ir.fee * ir.quantity) AS total_fee,
            SUM(ir.discount * ir.quantity) AS total_discount,
            SUM((ir.price + ir.fee - ir.discount) * ir.quantity * (v.aliquot_perc / 100)) AS total_vat,
            SUM((ir.price + ir.fee - ir.discount) * ir.quantity) AS total_no_vat,
            SUM((ir.price + ir.fee - ir.discount) * ir.quantity * (1 + v.aliquot_perc / 100)) AS total
        FROM
            product_product p
            JOIN invoice_invoicerow ir on ir.product_id=p.id
            JOIN invoice_invoice i ON i.id = ir.invoice_id
            JOIN organization_vat v ON v.id = ir.vat_id
        WHERE 1=1
            %s %s
        GROUP BY year, month, ir.product_id, p.name
        ORDER BY year, month, total DESC
        %s
    """ % (where, others_where, limit)
    cursor.execute(query)
    dictionary = dictfetchall(cursor)
    return dictionary

