import decimal
import datetime
from django import template
from store.models import Store

register = template.Library()

def get_active_pricelistitem(product, price_list_code, decimali=2, with_vat=False):
    """ Return the product price for the price list code.

    Sample usage::

        {% get_active_pricelistitem product 'pub' %}

    """
    price_list_item = product.get_active_pricelistitem(pricelist_code=price_list_code)
    if price_list_item:
        price = price_list_item.price
        if with_vat:
            price = price + (price * product.vat.aliquot_perc / 100)
        return price.quantize(decimal.Decimal('0.01'))
    return decimal.Decimal(0)
register.simple_tag(get_active_pricelistitem)


def get_stock_movements(product, store=None, year=None):
    from store.models import Store

    if not store:
        store = Store.objects.default_store()

    current_year = datetime.date.today().year
    if year:
        current_year = year

    stock = 0

    for ddtrow in product.ddtrows.filter(ddt__date_d__year=current_year, store_cause__store=store):
        stock = stock + (ddtrow.quantity * ddtrow.store_cause.stock)
    return stock
register.simple_tag(get_stock_movements)


def show_stock_qty(product, num_years=None, store_code=None):
    stores = None
    if store_code:
        try:
            stores = [Store.objects.get(code=store_code)]
        except:
            pass
    else:
        stores = Store.objects.all()

    this_year = datetime.date.today().year
    list_years = []
    for y in range(this_year - num_years, this_year+1):
        list_years.append(y)
    list_years.reverse()
    return {'product':product, 'stores': stores, 'years': list_years}
register.inclusion_tag('admin/product/product/tags/stock_quantity.html')(show_stock_qty)