from django.test import TestCase
from django.contrib.auth.models import User
from django.db import IntegrityError
from product.models import Product, Vat

class VatModelTest(TestCase):
    def setUp(self):
        pass
    def tearDown(self):
        pass
    def test_add_a_vat(self):
        l = Vat.objects.count()
        self.assertEquals(l, 0)
        vat = Vat(
            code='0.21',
            name='21.0',
            description='desc',
            aliquot_perc='21.0',
            deduct_perc='21.0',
        )
        vat.save()
        self.assertEquals(Vat.objects.count(), l+1)
    def test_fail_insert_vat_without_name(self):
        l = Vat.objects.count()
        self.assertEquals(l, 0)
        vat = Vat(
            code='TEST1',
            name=None,
            description='desc',
            aliquot_perc='21.0',
            deduct_perc='21.0',
        )
        self.assertRaises(IntegrityError, vat.save)
    def test_fail_insert_vat_with_identical_name(self):
        l = Vat.objects.count()
        self.assertEquals(l, 0)
        vat = Vat(
            code='0.21',
            name='21.0',
            description='desc',
            aliquot_perc='21.0',
            deduct_perc='21.0',
        )
        vat.save()
        self.assertEquals(Vat.objects.count(), l+1)
        vat = Vat(
            code='TEST1',
            name='21.0',
            description='desc',
            aliquot_perc='21.0',
            deduct_perc='21.0',
        )
        self.assertRaises(IntegrityError, vat.save)
    def test_fail_insert_vat_with_identical_code(self):
        l = Vat.objects.count()
        self.assertEquals(l, 0)
        vat = Vat(
            code='0.21',
            name='21.0',
            description='desc',
            aliquot_perc='21.0',
            deduct_perc='21.0',
        )
        vat.save()
        self.assertEquals(Vat.objects.count(), l+1)
        vat = Vat(
            code='0.21',
            name='21perc',
            description='desc',
            aliquot_perc='21.0',
            deduct_perc='21.0',
        )
        self.assertRaises(IntegrityError, vat.save)

