# -*- coding: utf-8 -*-
import datetime

from django.conf import settings
from django.contrib import admin, messages
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.conf.urls.defaults import *
from django import forms
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.contrib import messages
from django.db import transaction
from jumbo.apps.project.models import Project

from jumbo.views.generic.create_oo_file import create_file
from jumbo.forms.models import AdminModelForm
from jumbo.admin.options import OrderedModelAdmin, ExtendibleModelAdmin, SettingsAdmin
from jumbo.core.urlresolvers import reverse
from jumbo.forms import DateWidget
from jumbo.forms.fields import ChainedChoiceField
from jumbo.apps.autocomplete import AutocompleteModelChoiceField, AutocompleteInlineCharField


#from jumbo.utils.csv import render_csv
from quotation import views as quotation_views
from quotation.models import Quotation, QuotationRow, VatRow, Settings

from organization.models import Contact, Office, Vat
from payment.models import PaymentForm
from store.models import StoreCause

### FORM
class QuotationAdminForm(AdminModelForm):
    class Meta:
        model = Quotation
    def __init__(self, *args, **kws):
        super(QuotationAdminForm, self).__init__(*args, **kws)
        self.fields['date_q'].initial = datetime.date.today()
        self.fields['owner'] = AutocompleteModelChoiceField(
            'procedure_owner', label=_("Owner"),
            on_select_token='on_select_owner',
            on_delete_token='on_delete_owner',
        )
        self.fields['recipient'] = AutocompleteModelChoiceField(
            'procedure_recipient', label=_("Recipient"),
            on_select_token='on_select_recipient',
            on_delete_token='on_delete_recipient',
        )
        self.fields['contact'] = ChainedChoiceField(model=Contact, label=_("Contact"))
        self.fields['office'] = ChainedChoiceField(model=Office, label=_("Office"), required=False)
        self.fields['payment_form'] = ChainedChoiceField(model=PaymentForm, label=_("Payment Form"))
        self.fields['store_cause'] = ChainedChoiceField(model=StoreCause, label=_("Store Cause"))

    def clean(self):
        instance = self.instance
        cleaned_data = self.cleaned_data
        changed_data = self.changed_data
        num_changed_data = len(changed_data)
        if num_changed_data > 0:
            if instance.procedure:
                raise forms.ValidationError(_("Quotation already ordered. It's not changeable"))
            if instance.accepted and not (
                (num_changed_data == 1 and 'procedure' in changed_data and self.cleaned_data['procedure'] == 1) or
                (num_changed_data == 1 and 'accepted' in changed_data and self.cleaned_data['accepted'] == False) or
                (num_changed_data == 1 and 'date_accepted' in changed_data and self.cleaned_data['date_accepted'] == None)
            ):
                raise forms.ValidationError(_("Quotation already ordered. It's not changeable"))
            if instance.submitted and not (
                num_changed_data == 1 and 'date_accepted' in changed_data
            ):
                raise forms.ValidationError(_("Quotation already submitted. It's not changeable"))
        return cleaned_data

class QuotationRowAdminForm(AdminModelForm):
    class Meta:
        model = QuotationRow
    class Media:
        js = ('quotation/js/quotationrow_form.js',)

    def __init__(self, *args, **kws):
        super(QuotationRowAdminForm, self).__init__(*args, **kws)

        self.fields['product'] = AutocompleteModelChoiceField(
            'procedure_product', label=_("Product"),
            on_select_token='on_select_product',
            on_delete_token='on_delete_product',
        )

        quotation_id = self.get_object_id('quotation')
        quotation = Quotation.objects.get(pk=quotation_id)
        product_id = self.get_object_id('product')
        projects = Project.objects.filter(status=1)
        vats = Vat.objects.none()
        store_causes = StoreCause.objects.none()
        
        if quotation.store_cause.category == "cus":
            projects = projects.filter(organization__in=(quotation.owner, quotation.recipient))
        if self.fields.get('project'):
            self.fields['project'].queryset = projects
        if not product_id:
            self.fields['vat'].queryset = vats
            self.fields['store_cause'].queryset = store_causes
            self.fields['m_unit'].choices = []

    def clean(self):
        """
        verify if quotation is changeable
        """
        cleaned_data = self.cleaned_data
        changed_data = self.changed_data
        num_changed_data = len(changed_data)
        quotation = cleaned_data['quotation']

        if num_changed_data > 0:
            if quotation.procedure:
                raise forms.ValidationError("Quotation already ordered. It's not changeable")

            if quotation.accepted:
                raise forms.ValidationError("Quotation already accepted. It's not changeable")

            if quotation.submitted:
                raise forms.ValidationError("Quotation already submitted. It's not changeable")
        return cleaned_data

class VatRowAdminForm(AdminModelForm):
    class Meta:
        model = VatRow

### INLINE
class QuotationRowInline(admin.TabularInline):
    model = QuotationRow
    form = QuotationRowAdminForm
    extra = 1
    fields = ('product', 'project', 'vat', 'store_cause', 'quantity', 'price', 'fee', 'discount')

### SEARCH
class QuotationRowSearchForm(forms.ModelForm):
    class Meta:
        model = QuotationRow
        fields = ('store_cause', 'vat')
        display_fields = ('description__icontains',
                         ('store_cause', 'vat'),
                         ('quotation__date_q__year', 'search_in_all_years'),
                         ('procedure__accepted', 'procedure__procedure'))

    def __init__(self, *args, **kws):
        super(QuotationRowSearchForm, self).__init__(*args, **kws)
        BOOLEAN = (('', '---'), ('1', _('Yes')), ('0', _('No')),)

        self.fields['procedure__accepted'] = forms.ChoiceField(label=_("Accepted"), choices=BOOLEAN, required=False)
        self.fields['procedure__procedure'] = forms.ChoiceField(label=_("Procedure Complete"), choices=BOOLEAN, required=False)
        self.fields['description__icontains'] = forms.CharField(label=_("Description"), required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['quotation__date_q__year'] = forms.CharField(label=_("Year"), required=False)

    # Modifico il filtro di ricerca in modo da filtrare sempre in base all'anno corrente, tranne nel caso in cui
    # nei parametri di ricerca venga specificato un anno oppure venga flaggata la checkbox search_in_all_years
    def change_search_filter(self, lookup_params, qs):
        if not lookup_params.has_key('quotation__date_q__year') and not lookup_params.has_key('search_in_all_years'):
            qs = qs.filter(quotation__date_q__year = datetime.date.today().year)

        if lookup_params.has_key('search_in_all_years'):
            del lookup_params['search_in_all_years']
        return lookup_params, qs


class QuotationSearchForm(forms.ModelForm):
    class Meta:
        model = Quotation
        fields = ('payment_form', 'store_cause', 'number')
        display_fields = (
            ('owner__name__istartswith', 'owner__code'),
            ('recipient__name__istartswith', 'recipient__code'),
            ('payment_form', 'store_cause'),
            'number',
            ('date_q__year', 'search_in_all_years'),
            ('date_q__gte', 'date_q__lte'),
            ('accepted', 'procedure')
        )

    def __init__(self, *args, **kws):
        super(QuotationSearchForm, self).__init__(*args, **kws)
        BOOLEAN = (('', '---'), ('1', _('Yes')), ('0', _('No')),)
        ORGANIZATIONTYPE_DEF = (("", _("-----"),), ("cus", _("customer"),), ("sup", _("supplier"),))

        self.fields['owner__name__istartswith'] = forms.CharField(label=_("Owner"), required=False)
        self.fields['owner__code'] = forms.CharField(label=_("Owner Code"), required=False)
        self.fields['recipient__name__istartswith'] = forms.CharField(label=_("Recipient"), required=False)
        self.fields['recipient__code'] = forms.CharField(label=_("Recipient Code"), required=False)
        self.fields['date_q__gte'] = forms.DateField(label=_("Date Begin"), widget= DateWidget, required=False)
        self.fields['date_q__lte'] = forms.DateField(label=_("Date End"), widget= DateWidget, required=False)
        #self.fields['number'] = forms.CharField(label=_("Number"), required=False)
        #self.fields['owner_number'] = forms.CharField(label=_("Owner Number"), required=False)
        #self.fields['number__gte'] = forms.IntegerField(label=_("From Number"), required=False)
        #self.fields['number__lte'] = forms.IntegerField(label=_("To Number"), required=False)
        self.fields['accepted'] = forms.ChoiceField(label=_("Accepted"), choices=BOOLEAN, required=False)
        self.fields['procedure'] = forms.ChoiceField(label=_("Procedure Complete"), choices=BOOLEAN, required=False)
        self.fields['store_cause__category'] = forms.ChoiceField(label=_("Category"), choices=ORGANIZATIONTYPE_DEF, required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['date_q__year'] = forms.CharField(label=_("Year"), required=False)

    # Modifico il filtro di ricerca in modo da filtrare sempre in base all'anno corrente, tranne nel caso in cui
    # nei parametri di ricerca venga specificato un anno oppure venga flaggata la checkbox search_in_all_years
    def change_search_filter(self, lookup_params, qs):
        if not lookup_params.has_key('date_q__year') and not lookup_params.has_key('search_in_all_years'):
            qs = qs.filter(date_q__year = datetime.date.today().year)

        if lookup_params.has_key('search_in_all_years'):
            del lookup_params['search_in_all_years']
        return lookup_params, qs

### GENERAL
########################################
# QUOTATION ROW                        #
########################################
class QuotationRowAdmin(ExtendibleModelAdmin, OrderedModelAdmin):
    class Media:
        css = {"all": ("quotation/css/admin_quotation.css",)}

    form = QuotationRowAdminForm
    search_form = QuotationRowSearchForm
    list_per_page = 30
    fieldsets = (
        (None, {'fields': (
            'status', 'ordering', ('quotation', 'project'), 'product',
            ('vat', 'store_cause'), ('quantity', 'price', 'm_unit'),
            ('fee', 'discount'), 'description', 'long_description',
        ),}),
    )
    search_fields = ('quotation__number', 'product__code', 'product__name')
    default_group_by = 'quotation'
    list_display = [
        'get_date_q', 'get_number', 'get_detail', 'get_product_detail',
        'quantity', 'price', 'fee', 'discount', 'str_total_income',
        'str_total_fee', 'str_total_discount', 'str_total',
        'get_cause_code', 'get_vat_code'
    ]
    list_display_csv = [
        'get_date_q', 'get_number', 'get_detail', 'get_product_detail', 'quantity',
        'price', 'fee', 'discount', 'str_total_income', 'str_total_fee',
        'str_total_discount', 'str_total', 'get_cause_code', 'get_vat_code'
    ]

    # For inline with fancybox
    use_fancybox = True
    father_foreignkey_fancybox = 'quotation'

    def delete_view(self, request, object_id, extra_context=None, js_function='reload_quotation_row'):
        quotation_row = QuotationRow.objects.get(id = object_id)
        quotation = quotation_row.quotation
        if quotation.procedure:
            messages.error(request, _("Quotation already ordered. It's not deletable"))
            return HttpResponseRedirect('..')
        if quotation.accepted:
            messages.error(request, _("Quotation already accepted. It's not deletable"))
            return HttpResponseRedirect('..')
        if quotation.submitted:
            messages.error(request, _("Quotation already submitted. It's not deletable"))
            return HttpResponseRedirect('..')
        return super(QuotationRowAdmin, self).delete_view(request, object_id, extra_context, js_function)

    def get_date_q(self, obj):
        return obj.quotation.date_q.strftime("%d/%m/%Y")
    get_date_q.short_description = _('Date')
    get_date_q.admin_order_field = 'quotation__date_q'

    def get_number(self, obj):
        return obj.quotation.number or ""
    get_number.short_description = _('Number')
    get_number.admin_order_field = 'quotation__number'

    def get_detail(self, obj):
        owner = obj.quotation.owner
        recipient = obj.quotation.recipient or ""
        contact = obj.quotation.contact or ""
        if recipient == owner:
            recipient = ""
        return "%s<br/>%s<br/>%s" % (owner, recipient, contact)
    get_detail.short_description = _("Bill to")
    get_detail.allow_tags = True
    get_detail.admin_order_field = 'quotation__owner'

    def get_product_detail(self, obj):
        product = obj.get_description()
        return """<b><a href="%s/">%s</a></b>""" % (obj.id, product)
    get_product_detail.short_description = _("Product")
    get_product_detail.allow_tags = True
    get_product_detail.admin_order_field = 'description'

    def get_cause_code(sel, obj):
        return obj.store_cause.code
    get_cause_code.short_description = _('SC')

    def get_vat_code(self, obj):
        return obj.vat.code
    get_vat_code.short_description = _('VC')

    def str_total_income(self, obj):
        return str("%.2f" % obj.get_total_income())
    str_total_income.short_description = _('Total Income')

    def str_total_fee(self, obj):
        return str("%.2f" % obj.get_total_fee())
    str_total_fee.short_description = _('Total Fee')

    def str_total_discount(self, obj):
        return str("%.2f" % obj.get_total_discount())
    str_total_discount.short_description = _('Total Discount')

    def str_total(self, obj):
        return str("%.2f" % obj.get_total())
    str_total.short_description = _('Total')


######################################
# QUOTATION                          #
######################################
class QuotationAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("quotation/css/admin_quotation.css",)}
        js = ('quotation/js/quotation.js',)

    form = QuotationAdminForm
    search_form = QuotationSearchForm
    list_per_page = 30
    date_hierarchy = 'date_q'
    fieldsets          = (
        (_('logs admin'), {
            'fields'  : ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None           , {
            'fields'  : (
                'status',
                ('date_q',),
                ('number', 'revision', 'revision_year'),
                ('owner', 'recipient',),
                ('contact', 'project',),
                'payment_form',
                'store_cause',
                'currency',
                'vat',
                'price_list',
                'commision',
                ('vector', 'delivery',),
                ('date_inspected', 'inspected'),
                ('date_submitted', 'submitted'),
                ('date_accepted', 'accepted'),
                ('date_delivery', 'shipping_address',),
                ('delivery_window', 'quote_firm_for',),
                'description',
            )
        }),
        (_('note')     , {'fields'  : ('note',), 'classes' : ('collapse',)}),
    )
    search_fields = ('owner', 'recipient', 'contact', 'payment_form', 'number', 'date_q')
    list_display = (
        'get_date_q', 'get_number', 'get_code', 'get_detail', 'get_total',
        'send_mail', 'create_odt', 'create_pdf', 'inspected', 'submitted', 'accepted',
        'procedure', 'get_document_icon', 'get_detail_icon', 'get_edit_icon'
    )
    list_display_csv = (
        'get_date_q', 'get_number', 'owner', 'contact', 'description',
        'payment_form', 'store_cause', 'get_income', 'get_fee', 'get_discount',
        'get_vat', 'get_total'
    )
    readonly_fields = ('revision', 'revision_year', 'inspected', 'submitted', 'accepted')
    default_group_by = '-date_q'
    actions = [
        "set_inspected", "set_submitted", "set_accepted", "create_procedure",
        "revision_quotation", "clone_quotation"
    ]
    save_on_top = True

    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(QuotationAdmin, self).get_urls()
        my_urls = patterns('',
            url(
                r'^send_mail/(?P<quotation_id>\d+)/$',
                self.admin_site.admin_view(quotation_views.quotation_send_mail),
                name = 'quotation_quotation_sendmail'
            ),
            url(
                r'^create_odt/(?P<obj_id>\d+)/$',
                self.admin_site.admin_view(create_file),
                {
                    'model':            Quotation,
                    'file_model':       'admin/quotation/quotation/quotation.odt',
                    'file_type':        'odt',
                },
                name='quotation_quotation_create_odt'
            ),
            url(
                r'^create_pdf/(?P<obj_id>\d+)/$',
                self.admin_site.admin_view(create_file),
                {
                    'model':            Quotation,
                    'file_model':       'admin/quotation/quotation/quotation.odt',
                    'file_type':        'pdf',
                },
                name='quotation_quotation_create_pdf'
            )
        )
        return my_urls + urls


    def delete_view(self, request, object_id, extra_context=None, js_function=''):
        quotation = Quotation.objects.get(id = object_id)
        if quotation.procedure:
            messages.error(request, _("Quotation already ordered. It's not deletable"))
            return HttpResponseRedirect('..')
        if quotation.accepted:
            messages.error(request, _("Quotation already accepted. It's not deletable"))
            return HttpResponseRedirect('..')
        if quotation.submitted:
            messages.error(request, _("Quotation already submitted. It's not deletable"))
            return HttpResponseRedirect('..')
        return super(QuotationAdmin, self).delete_view(request, object_id, extra_context, js_function)

    def get_date_q(self, obj):
        return obj.date_q.strftime("%d/%m/%Y")
    get_date_q.short_description = _('Date')
    get_date_q.admin_order_field = 'date_q'

    def get_document_icon(self, obj):
        if "document" in settings.INSTALLED_APPS:
            from document.models import Category
            # Recupero il content type della Quotation
            ctq = ContentType.objects.get_for_model(obj)

            ## TODO ##
            # Cerco di recuperare l'id della document category dei preventivi
            # Per ora suppongo che il codice sia PRV. Poi dovra' essere messo come settings personalizzabile dall'utente
            try:
                prv_category = Category.objects.get(code='PRV').pk
            except Category.DoesNotExist:
                prv_category = ""

            link = """<a href="%(reverse_url)s" class="iframe">D</a>""" % {
                    'reverse_url':  '%(add_document)s?_popup=1&content_type=%(content_type)s&object_id=%(object_id)s&category=%(category)s&status=CM&support=W&title=%(title)s' % {
                    'add_document': reverse('admin:document_document_add'),
                    'content_type':ctq.pk,
                    'object_id':obj.id,
                    'category':prv_category,
                    'title':_('Quotation n.%s of %s') % (obj.number, obj.date_q)
                }
            }
            return link
        return ""
    get_document_icon.short_description = _("D")
    get_document_icon.allow_tags = True

    def get_detail(self, obj):
        owner = """<a href=%(reverse_url)s>%(owner)s</a>""" % {
            #'reverse_url': reverse('organization-detail', obj.owner.id),
            'reverse_url':  '',
            'owner':       obj.owner.name
        }
        contact = obj.contact or ""
        description = obj.description
        return "<b>%s</b><br/>%s<br/>%s" % (owner, contact, description)
    get_detail.short_description = _("Bill to")
    get_detail.allow_tags = True
    get_detail.admin_order_field = 'owner'

    def get_payment_form(self, obj):
        return """<span title="%s">%s</span>""" % (obj.payment_form.name, obj.payment_form.code)
    get_payment_form.short_description = _("Payment Form")
    get_payment_form.allow_tags = True
    get_payment_form.admin_order_field = 'payment_form__code'

    def get_store_cause(self, obj):
        return """<span title="%s">%s</span>""" % (obj.store_cause.name, obj.store_cause.code)
    get_store_cause.short_description = _("Store Cause")
    get_store_cause.allow_tags = True
    get_store_cause.admin_order_field = 'store_cause__code'

    def get_income(self, obj):
        return obj.totals['income']
    get_income.short_description = _('Total Income')

    def get_fee(self, obj):
        return obj.totals['fee']
    get_fee.short_description = _('Total Fee')

    def get_discount(self, obj):
        return obj.totals['discount']
    get_discount.short_description = _('Total Discount')

    def get_vat(self, obj):
        return obj.totals['vat']
    get_vat.short_description = _('Total VAT')

    def get_total(self, obj):
        return obj.totals['total']
    get_total.short_description = _('Total')

    def get_code(self,obj):
        return obj.owner.code
    get_code.short_description = _('Code')

    def create_odt(self, obj):
        # fa andare in errore apache
        # <a href="%(href)s" onclick="javascript:print_conditions('%(confirm_message)s', '%(href)s');" title="%(title)s"><img src='%(url)sadmin/images/writer.gif' alt='%(title)s' title='%(title)s'/></a>
        return """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/writer.gif' alt='%(title)s' title='%(title)s'/></a>""" % {
            'href' : reverse('admin:quotation_quotation_create_odt', obj.id),
            'url':settings.MEDIA_URL,
            'title': ugt("Create Quotation ODT"),
        }
    create_odt.short_description = _("ODT")
    create_odt.allow_tags = True

    def create_pdf(self, obj):
        return """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/pdf_icon.png' alt='%(title)s' title='%(title)s'/></a>""" % {
            'href' :reverse('admin:quotation_quotation_create_pdf', obj.id),
            'url':settings.MEDIA_URL,
            'title': ugt("Create Quotation PDF")
        }
    create_pdf.short_description = _("PDF")
    create_pdf.allow_tags = True

    def get_quotation_rows(self, obj):
        n_rows = obj.rows.count()
        link = ""
        if n_rows > 0:
            link = """<a href="%(reverse_url)s" class="link_quotationrows" rel="#overlay">%(num_quotation)s</a>""" % {
                #'reverse_url': reverse('procedure-procedure-procedurerows', obj.id),
                'reverse_url': '',
                'num_quotation': n_rows,
            }

        return link
    get_quotation_rows.short_description = _('QR')
    get_quotation_rows.allow_tags = True

    def send_mail(self, obj):
        return """<a href="%(href)s" title="%(title)s"><img src='%(url)sadmin/images/mail.png' alt='%(title)s' title='%(title)s'/></a>""" % {
            'href' : reverse('admin:quotation_quotation_sendmail', obj.id),
            'url':settings.MEDIA_URL,
            'title': ugt("Send Mail")
        }
    send_mail.short_description = _("Mail")
    send_mail.allow_tags = True

    def clone_quotation(self, request, queryset):
        """Clone quotation"""
        rows_updated = 0
        for obj in queryset:
            try:
                self.clone(request, obj)
                rows_updated += 1
            except Exception, e:
                messages.error(request, "%s: %s" % (obj.get_number, e))
        if rows_updated == 0:
            messages.error(request, _("ATTENTION! Nothing was cloned"))
        elif rows_updated == 1:
            message_bit = _("1 object was")
            messages.info(request, _("%s successfully cloned.") % message_bit)
        else:
            message_bit = _("%s objects were") % rows_updated
            messages.info(request, _("%s successfully cloned.") % message_bit)
    clone_quotation.short_description = _("Clone Quotation")

    @transaction.commit_manually
    def clone(self, request, obj):
        try:
            old_obj = Quotation.objects.get(pk=obj.pk)
            obj.pk = None
            obj.recipient = None
            obj.procedure = False
            obj.payment_form = None
            obj.office = None
            obj.contact = None
            obj.currency = None
            obj.delivery = None
            obj.vat = None
            obj.agent = None
            obj.commision = None
            obj.price_list = None
            obj.vector = None
            obj.shipping_address = None
            obj.number = None
            obj.revision = None
            obj.date_q = datetime.date.today()
            obj.inspected = None
            obj.date_inspected = None
            obj.date_delivery = None
            obj.submitted = False
            obj.date_submitted = None
            obj.accepted = False
            obj.date_accepted = None
            obj.note = ""
            obj.save(force_insert=True)

            ## salvo many to many collegate
            for m2m in obj._meta.many_to_many:
                setattr(obj, m2m.name, getattr(old_obj, m2m.name).all())

            # salvo related
            for row in old_obj.rows.all():
                row.pk = None
                row.project = None
                obj.rows.add(row)

            obj.save()
            transaction.commit()
            return ""
        except Exception, e:
            transaction.rollback()
            raise Exception(e)


    def revision_quotation(self, request, queryset):
        """ Revision quotation """
        rows_updated = 0
        for obj in queryset:
            try:
                self.revision(request, obj)
                rows_updated += 1
            except Exception, e:
                messages.error(request, "%s: %s" % (obj.get_number, e))
        if rows_updated == 0:
            messages.error(request, _("ATTENTION! Nothing was revisioned"))
        elif rows_updated == 1:
            message_bit = _("1 object was")
            messages.info(request, _("%s successfully revisioned.") % message_bit)
        else:
            message_bit = _("%s objects were") % rows_updated
            messages.info(request, _("%s successfully revisioned.") % message_bit)
    revision_quotation.short_description = _("Revision Quotation")

    @transaction.commit_manually
    def revision(self, request, obj):
        try:
            old_obj = Quotation.objects.get(pk=obj.pk)
            obj.pk = None
            obj.procedure = False
            obj.date_delivery = None
            obj.submitted = False
            obj.date_submitted = None
            obj.accepted = False
            obj.date_accepted = None

            obj.date_q = datetime.date.today()
            # controllo revisioni precedenti con:
            # - stesso numero
            # - stesso anno di prima revisione
            # - stesso intestatario
            revision = Quotation.objects.filter(
                number=obj.number, revision_year=obj.revision_year,
                owner=obj.owner
            ).count()
            obj.revision = revision + 1
            obj.save(force_insert=True)

            ## salvo many to many collegate
            for m2m in obj._meta.many_to_many:
                setattr(obj, m2m.name, getattr(old_obj, m2m.name).all())

            # salvo related
            for row in old_obj.rows.all():
                row.pk = None
                obj.rows.add(row)

            obj.save()
            transaction.commit()
            return ""
        except Exception, e:
            transaction.rollback()
            raise Exception(e)

    def set_inspected(self, request, queryset):
        rows_updated = 0
        for row in queryset:
            rows_updated += 1
            row.inspected = True
            row.date_inspected = datetime.date.today()
            row.save()
        if rows_updated == 1:
            message_bit = "1 Quotantion was"
        else:
            message_bit = "%s Quotation were" % rows_updated
        messages.info(request, "%s successfully inspected." % message_bit)
    set_inspected.short_description = _("Set Quotations as inspected")

    def set_accepted(self, request, queryset):
        rows_updated = 0
        for row in queryset:
            rows_updated += 1
            row.accepted = True
            row.date_accepted = datetime.date.today()
            row.save()
        if rows_updated == 1:
            message_bit = "1 Quotantion was"
        else:
            message_bit = "%s Quotation were" % rows_updated
        messages.info(request, "%s successfully accepted." % message_bit)
    set_accepted.short_description = _("Set Quotations as accepted")

    def set_submitted(self, request, queryset):
        rows_updated = 0
        for row in queryset:
            rows_updated += 1
            row.submitted = True
            row.date_submitted = datetime.date.today()
            row.save()
        if rows_updated == 1:
            message_bit = "1 Quotantion was"
        else:
            message_bit = "%s Quotation were" % rows_updated
        messages.info(request, "%s successfully submitted." % message_bit)
    set_submitted.short_description = _("Set Quotations as submitted")


    def create_procedure(self, request, queryset):
        rows_updated = 0
        error_message = ""
        for obj in queryset:
            resp = obj.create_or_append_to_procedure()
            if resp[0]:
                rows_updated += 1
            else:
                messages.error(request, "Quotation n. %s: %s!" % (obj.number, resp[1]))
        if rows_updated == 1:
            message_bit = "1 Procedure was"
        else:
            message_bit = "%s Procedures were" % rows_updated
        if resp[0]:
            self.message_user(request, "%s successfully created." % message_bit)
    create_procedure.short_description = _("Create Procedure")

admin.site.register(Settings, SettingsAdmin)
admin.site.register(Quotation, QuotationAdmin)
admin.site.register(QuotationRow, QuotationRowAdmin)
