import decimal
from dajax.core import Dajax
from dajaxice.core import dajaxice_functions
from django.http import Http404
from django.utils.html import strip_tags

from jumbo.shortcuts import render_to_string
from jumbo.apps.project.models import Project

from organization.models import Organization, Office, Contact
from organization.models import OrganizationAddress, Currency, Vat
from payment.models import PaymentForm
from store.models import StoreCause
from quotation.models import Quotation
from product.models import Product, UNIT_TYPE
from jumbo.apps.project.models import Project

##################################
# OWNER                          #
##################################
def ajax_on_select_owner(
    request, org_id, office_id=None, contact_id=None, payment_form_id=None,
    store_cause_id=None, project_id=False, set_default=False
):
    dajax = Dajax()
    if not org_id:
        return dajax.json()

    try:
        o = Organization.objects.get(id=org_id)

        # CONTACT
        option = "<option value=''></option>"
        for i, contact in enumerate(o.contacts.all()):
            selected = ""
            if contact_id:
                if contact.id == int(contact_id):
                    selected = " selected"
            elif set_default and i == 0:
                selected = " selected"
            option += "<option value='%s'%s>%s</option>" % (contact.id, selected, unicode(contact))
        dajax.assign('#id_contact', 'innerHTML', option)

        # PROJECT
        option = "<option value=''></option>"
        category = o.organization_type.category
        if category == "cus":
            projects = o.projects.filter(status=1)
        else:
            projects = Project.objects.filter(status=1)
        for i, project in enumerate(projects):
            selected = ""
            if project_id:
                if int(project_id) == project.id:
                    selected = " selected"
            elif set_default and i == 0:
                selected = " selected"
            option += "<option value='%s'%s>%s</option>" % (project.id, selected, unicode(project))

        dajax.assign('#id_project', 'innerHTML', option)

        # OFFICE
        option = "<option value=''></option>"
        office = []
        if o.office:
            office = [o.office]

        for i, office in enumerate(office):
            selected = ""
            if office_id:
                if office.id == int(office_id):
                    selected = " selected"
            elif set_default and i == 0:
                selected = " selected"
            option += "<option value='%s'%s>%s</option>" % (office.id, selected, unicode(office))
        dajax.assign('#id_office', 'innerHTML', option)

        # PAYMENT FORM
        option = "<option value=''></option>"

        if o.payment_form:
            org_payment_id = o.payment_form.id
        else:
            org_payment_id = 0

        payments = PaymentForm.objects.filter(status=1)

        for i, pay in enumerate(payments):
            selected = ""
            if payment_form_id:
                if pay.id == int(payment_form_id):
                    selected = " selected"
            elif set_default and pay.id == org_payment_id:
                selected = " selected"
            option += "<option value='%s'%s>%s</option>" % (pay.id, selected, unicode(pay))
        dajax.assign('#id_payment_form', 'innerHTML', option)

        # STORE CAUSE
        option = "<option value=''></option>"
        category = o.organization_type.category
        if category == "com":
            category = "cus"
        cats = StoreCause.objects.filter(category=category, status=1)
        cat_default = None
        try:
            cat_default = StoreCause.objects.get(default=True, status=1, category=category)
        except:
            cat_defaults = StoreCause.objects.filter(status=1, category=category)
            if cat_defaults:
                cat_default = cat_defaults[0]

        for i, cat in enumerate(cats):
            selected = ""
            if store_cause_id:
                if cat.id == int(store_cause_id):
                    selected = " selected"
            elif set_default and cat == cat_default:
                selected = " selected"
            option += "<option value='%s'%s>%s</option>" % (cat.id, selected, unicode(cat.name))
        dajax.assign('#id_store_cause', 'innerHTML', option)

        # CURRENCY
        option = "<option value=''></option>"
        if o.currency:
            currency_id = o.currency.id
        else:
            currency_id = 0

        currencies = Currency.objects.filter(status=1)
        for i, curr in enumerate(currencies):
            selected = ""
            if currency_id:
                if curr.id == int(currency_id):
                    selected = " selected"
            elif set_default and curr.id == currency_id:
                selected = " selected"
            option += "<option value='%s'%s>%s</option>" % (curr.id, selected, unicode(curr))
        dajax.assign('#id_currency', 'innerHTML', option)

        # VAT
        option = "<option value=''></option>"
        if o.vat:
            vat_id = o.vat.id
        else:
            vat_id = 0

        vats = Vat.objects.filter(status=1)
        for i, vat in enumerate(vats):
            selected = ""
            if vat_id:
                if vat.id == int(vat_id):
                    selected = " selected"
            elif set_default and vat.id == vat_id:
                selected = " selected"
            option += "<option value='%s'%s>%s</option>" % (vat.id, selected, unicode(vat))
        dajax.assign('#id_vat', 'innerHTML', option)

    except Exception, e:
        pass

    return dajax.json()
dajaxice_functions.register(ajax_on_select_owner)

def ajax_on_delete_owner(request):
    dajax = Dajax()
    option = "<option value=''></option>"
    dajax.assign('#id_contact', 'innerHTML', option)
    dajax.assign('#id_project', 'innerHTML', option)
    dajax.assign('#id_office', 'innerHTML', option)
    dajax.assign('#id_payment_form', 'innerHTML', option)
    dajax.assign('#id_store_cause', 'innerHTML', option)
    dajax.assign('#id_currency', 'innerHTML', option)
    dajax.assign('#id_vat', 'innerHTML', option)
    return dajax.json()
dajaxice_functions.register(ajax_on_delete_owner)

##################################
# RECIPIENT                      #
##################################
def ajax_on_select_recipient(
    request, recipient_id, shipping_address_id=None, project_id=None, set_default=False):

    dajax = Dajax()
    if not recipient_id:
        return dajax.json()

    o = Organization.objects.get(id=recipient_id)

    option = "<option value=''></option>"
    # SHIPPING ADDRESS - CLIENTE
    if o.organization_type.category == "cus":
        for i, shipping_address in enumerate(o.get_shipping_addresses()):
            selected = ""
            if shipping_address_id:
                if shipping_address.id == int(shipping_address_id):
                    selected = " selected"
            elif set_default and i == 0:
                selected = " selected"
            option += "<option value='%s'%s>%s</option>" % (shipping_address.id, selected, unicode(shipping_address))

    # SHIPPING ADDRESS - FORNITORE
    if o.organization_type.category == "sup":
        org_adrs = OrganizationAddress.objects.filter(
            organization__organization_type__category="cus",
            address_scopes__is_shipping_address=True
        ).order_by('name')
        for i, shipping_address in enumerate(org_adrs):
            selected = ""
            if shipping_address_id:
                if shipping_address.id == int(shipping_address_id):
                    selected = " selected"
            elif set_default and i == 0:
                selected = " selected"
            option += "<option value='%s'%s>%s</option>" % (shipping_address.id, selected, unicode(shipping_address))

    dajax.assign('#id_shipping_address', 'innerHTML', option)

    # PROJECT
    option = "<option value=''></option>"
    category = o.organization_type.category
    if category == "cus":
        projects = o.projects.filter(status=1)
    else:
        projects = Project.objects.filter(status=1)
    for i, project in enumerate(projects):
        selected = ""
        if project_id:
            if int(project_id) == project.id:
                selected = " selected"
        elif set_default and i == 0:
            selected = " selected"
        option += "<option value='%s'%s>%s</option>" % (project.id, selected, unicode(project))
    dajax.assign('#id_project', 'innerHTML', option)

    return dajax.json()
dajaxice_functions.register(ajax_on_select_recipient)

def ajax_on_delete_recipient(request):
    dajax = Dajax()
    option = "<option value=''></option>"
    dajax.assign('#id_shipping_address', 'innerHTML', option)
    dajax.assign('#id_project', 'innerHTML', option)
    return dajax.json()
dajaxice_functions.register(ajax_on_delete_recipient)


##################################
# PRODUCT                        #
##################################
def ajax_on_select_product(
        request, prod_id, quotation_id=None, vat_id=None, m_unit=None,
        store_cause_id=None, description=None, long_description=None,
        project_id=None, set_default=False
):
    dajax = Dajax()

    if not prod_id:
        return dajax.json()

    product = Product.objects.get(id=prod_id)
    quotation = Quotation.objects.get(id=quotation_id)
    pricelist = None
    perc_discount1 = 0
    perc_discount2 = 0
    try:
        if quotation.price_list:
            pricelist = quotation.price_list.price_list
            discount1 = quotation.price_list.discount1
            discount2 = quotation.price_list.discount2
    except Quotation.DoesNotExist:
        raise Http404

    # PRICELISTITEM
    try:
        # procedure.price_list.price_list -> il primo price_list e' l'organization_price_list,
        # mentre il secondo e' il listino associato

        pricelistitem = product.get_active_pricelistitem(pricelist=pricelist)
        price = pricelistitem.last_price
    except Exception, e:
        price = decimal.Decimal(0)
    dajax.assign('#id_price', 'value', price.quantize(decimal.Decimal('0.01')))
    dajax.assign('#id_perc_discount1', 'value', perc_discount1)
    dajax.assign('#id_perc_discount2', 'value', perc_discount2)
    dajax.assign('#id_fee', 'value', 0)
    dajax.assign('#id_discount', 'value', 0)

    # VAT
    option = "<option value=''></option>"
    if product.vat and not vat_id:
        vat_id = product.vat.id
    vat = Vat.objects.filter(status=1)
    for i, v in enumerate(vat):
        selected = ""
        if vat_id:
            if v.id == int(vat_id):
                selected = " selected"
        elif set_default and i == 0:
            selected = " selected"
        option += u"<option value='%s'%s>%s</option>" % (v.id, selected, v)
    dajax.assign('#id_vat', 'innerHTML', option)

    # PROJECT
    option = "<option value=''></option>"
    owner = quotation.owner
    recipient = quotation.recipient
    category = owner.organization_type.category
    projects = Project.objects.filter(status=1)
    if category == "cus":
        projects = projects.filter(organization__in=(owner, recipient))
    for i, proj in enumerate(projects):
        selected = ""
        if project_id:
            if proj.id == int(project_id):
                selected = " selected"
        option += u"<option value='%s'%s>%s</option>" % (proj.id, selected, proj)
    dajax.assign('#id_project', 'innerHTML', option)

    # M_UNIT
    option = "<option value=''></option>"
    if product.m_unit and not m_unit:
        m_unit = product.m_unit
    for i, mu in enumerate(UNIT_TYPE):
        selected = ""
        if m_unit:
            if mu[0] == m_unit:
                selected = " selected"
        elif set_default and i == 0:
            selected = " selected"
        option += u"<option value='%s'%s>%s</option>" % (mu[0], selected, mu[1])
    dajax.assign('#id_m_unit', 'innerHTML', option)

    # STORE CAUSE
    option = "<option value=''></option>"
    quot_store_cause = quotation.store_cause
    prod_store_cause = product.store_cause
    category = quotation.owner.organization_type.category

    if prod_store_cause and not store_cause_id and category == "cus":
        store_cause_id = prod_store_cause.id
    if quot_store_cause and not store_cause_id:
        store_cause_id = quot_store_cause.id

    store_causes = StoreCause.objects.filter(status=1, category=category)
    for i, sc in enumerate(store_causes):
        selected = ""
        if store_cause_id:
            if sc.id == int(store_cause_id):
                selected = " selected"
        elif set_default and i == 0:
            selected = " selected"
        option += u"<option value='%s'%s>%s</option>" % (sc.id, selected, sc)
    dajax.assign('#id_store_cause', 'innerHTML', option)

    # DESCRIPTIONS
    if product.short_description and not description:
        dajax.assign('#id_description', 'value', product.short_description)
    if product.short_description and not long_description:
        full_description = strip_tags(product.full_description)
        dajax.assign('#id_long_description', 'value', full_description)

    return dajax.json()
dajaxice_functions.register(ajax_on_select_product)

def ajax_on_delete_product(request):
    dajax = Dajax()
    dajax.assign('#id_price', 'value', "")
    dajax.assign('#id_quantity', 'value', "")
    dajax.assign('#id_fee', 'value', "")
    dajax.assign('#id_discount', 'value', "")
    dajax.assign('#id_description', 'value', "")
    dajax.assign('#id_long_description', 'value', "")

    option = "<option value=''></option>"
    dajax.assign('#id_vat', 'innerHTML', option)
    dajax.assign('#id_store_cause', 'innerHTML', option)
    dajax.assign('#id_m_unit', 'innerHTML', option)

    return dajax.json()
dajaxice_functions.register(ajax_on_delete_product)

def reload_procedure_rows(request, quotation_id):
    dajax = Dajax()
    p = Quotation.objects.get(id=quotation_id)
    str_quotation = render_to_string("admin/quotation/quotation/quotationrow_inline.html", original=p)
    dajax.assign("#quotation_rows", "innerHTML", str_quotation)

    return dajax.json()
dajaxice_functions.register(reload_procedure_rows)
#
#
#def ajax_on_select_product(request, prod_id, set_default=False):
#    dajax = Dajax()
#
#    if not prod_id:
#        return dajax.json()
#
#    p = Product.objects.get(id=prod_id)
#
#    # PRICELISTITEM
#    try:
#        last_price = p.get_active_pricelistitem()[0].price
#    except:
#        last_price = 0
#
#    dajax.assign('#id_price', 'value', last_price)
#    return dajax.json()
#dajaxice_functions.register(ajax_on_select_product)
#
#def ajax_on_delete_product(request):
#    dajax = Dajax()
#    dajax.assign('#id_price', 'value', 0)
#    return dajax.json()
#dajaxice_functions.register(ajax_on_delete_product)
