from django.utils.translation import ugettext_lazy as _

from admin_tools.dashboard import modules
from quotation.models import Quotation

class QuotationDashboard(modules.Group):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': "Quotations",
            'display': "tabs",
            'children': [
                QuotationModels(),
                QuotationSettings(),
            ]
        }
        super(QuotationDashboard, self).__init__(**kwargs)

class QuotationModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Models"),
            'models': ('quotation.models.Quotation', 'quotation.models.QuotationRow'),
            }
        super(QuotationModels, self).__init__(**kwargs)

class QuotationSettings(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Settings"),
            'models':['quotation.models.Settings',],
            }
        super(QuotationSettings, self).__init__(**kwargs)

class CustomerQuotations(modules.Group):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': "Customer Quotations",
            'children': [
                Quotations(title='Last'),
                Quotations(title='Last To Do', submitted=False),
                Quotations(title='Last Submitted', submitted=True),
                Quotations(title='Last Accepted', procedure=True),
            ],
        }   
        super(CustomerQuotations, self).__init__(**kwargs)

class Quotations(modules.DashboardModule):
    def __init__(self, title, **kwargs):
        self.limit = 10
        self.category = None
        self.submitted = None
        self.procedure = None
        self.order_by = None
        self.template = 'admin_tools/dashboard/quotations.html'
        self.children = ''

        # creo o sovrascrivo con attributi passati
        for key in kwargs:
            setattr(self, key, kwargs[key])

        kwargs =  {
            'title': title,
            'template': self.template,
            'children': self.children
        }
        super(Quotations, self).__init__(**kwargs)

    def init_with_context(self, context):
        request = context['request']
        qs = Quotation.objects.all()
        if self.order_by: qs = qs.order_by(self.order_by) 
        if self.category: qs = qs.filter(store_cause__category=self.category)
        if self.submitted != None: qs = qs.filter(submitted=self.submitted)
        if self.procedure != None: qs = qs.filter(procedure=self.procedure)
        if self.limit: qs = qs[:self.limit]
        self.children = qs

