# -*- coding: utf-8 -*-
from datetime import date, timedelta
import decimal

from django.conf import settings
from django.db import models, transaction
from django.contrib.auth.models import User
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy as ugl

from jumbo.db.fields import HTMLField
from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jumbo.db.models import Settings as JSettings
from jumbo.apps.project.models import Project
from jumbo.core.urlresolvers import reverse
from jumbo.utils.functions import zerofill

from address.models import Address
from organization.models import Organization, Office, Contact, Bank, OrganizationAddress
from organization.models import Currency, Delivery, Vat, Agent, Commision, M2MtoVector as Vector
from payment.models import PaymentForm
from store.models import StoreCause
from product.models import Product, OrganizationPriceList
from organization.models import Vat
from procedure.models import ProcedureRow

###############################
# PREVENTIVO                  #
###############################
class Quotation (DateModel, UserModel, StatusModel):
    ## ForeignKey
    owner = models.ForeignKey (
        Organization,
        related_name = "owner_quotations",
        db_index = True,
        verbose_name = _("Bill to"),
    )
    recipient = models.ForeignKey (
        Organization,
        null = True, blank = True,
        related_name = "recipient_quotations",
        db_index = True,
        verbose_name = _("Ship to"),
    )
    project = models.ForeignKey (
        Project,
        blank = True, null = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Project"),
    )
    payment_form = models.ForeignKey (
        PaymentForm,
        blank = True, null = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Payment Form"),
    )
    office = models.ForeignKey (
        Office,
        blank = True, null = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Office"),
    )
    contact = models.ForeignKey (
        Contact,
        blank = True, null = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Contact"),
    )
    #bank = models.ForeignKey (
    #)
    currency = models.ForeignKey (
        Currency,
        blank = True, null = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Currency")
    )
    delivery = models.ForeignKey (
        Delivery,
        blank = True, null = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Delivery")
    )
    vat = models.ForeignKey (
        Vat,
        blank = True, null = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Vat")
    )
    agent = models.ForeignKey (
        Agent,
        null = True, blank = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Agent"),
    )
    commision = models.ForeignKey (
        Commision,
        blank = True, null = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Agent & Commision")
    )
    price_list = models.ForeignKey (
        OrganizationPriceList,
        blank = True, null = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Prices List & Discount")
    )
    vector = models.ForeignKey (
        Vector,
        blank = True, null = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Vector")
    )
    store_cause = models.ForeignKey (
        StoreCause,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Store Cause")
    )
    shipping_address = models.ForeignKey (
        OrganizationAddress,
        blank = True, null = True,
        related_name = "quotations",
        db_index = True,
        verbose_name = _("Shipping Address")
    )
    ## Campi
    description = models.CharField(
        max_length = 255,
        db_index = True,
        verbose_name = _("Description")
    )
    full_description = HTMLField(
        blank=True,
        null=True,
        verbose_name=_("Full description"),
    )
    number = models.IntegerField (
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Number"),
        db_column = "qnumber"
    )
    revision = models.IntegerField (
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Revision"),
    )
    revision_year = models.IntegerField(
#        blank = True, null = True,
        verbose_name = _("First revision year")
    )
    date_q = models.DateField (
        db_index = True,
        verbose_name = _("Date")
    )
    procedure = models.BooleanField (
        ## preventivo già trasformato in procedure
        default = False,
        db_index = True,
        verbose_name = _("Ordered"),
    )
    # data prevista consegna
    date_delivery = models.DateField (
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Date Delivery"),
    )
    ## campi condizioni generali
    # validita offerta
    quote_firm_for = models.CharField (
        max_length = 255,
        blank = True, null = True,
        verbose_name = _("Quote Firm For"),
    )
    # tempi di consegna
    delivery_window = models.CharField (
        max_length = 255,
        blank = True, null = True,
        verbose_name = _("Delivery Window"),
    )

    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("Note"),
    )
    inspected = models.BooleanField (
        ## fatto sopralluogo
        default = False,
        db_index = True,
        verbose_name = _("Inspected"),
    )
    date_inspected = models.DateField (
        # data sopralluogo
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Inspected Date"),
    )
    submitted = models.BooleanField (
        ## preventivo presentato
        default = False,
        db_index = True,
        verbose_name = _("Submitted"),
    )
    date_submitted = models.DateField (
        # data presentazione
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Submitted Date"),
    )
    accepted = models.BooleanField (
        ## preventivo accettato
        default = False,
        db_index = True,
        verbose_name = _("Accepted"),
    )
    date_accepted = models.DateField (
        # data accettazione
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Accepted Date"),
    )
    total_income = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Income"),
    )
    total_fee = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Fee"),
    )
    total_discount = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Discount"),
    )
    total_vat = models.DecimalField (
        default = 0,
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total VAT"),
    )
    # campi print
#    print_discount = models.BooleanField (
#        default = True,
#        verbose_name = _("Print Discount"),
#    )

    def save(self, force_insert=False, force_update=False):
        if not self.number:
            # number
            from store.utils import create_update_storedocumentnumber
            self.number = create_update_storedocumentnumber(
                'q', curr_date=self.date_q, store_document=self.store_cause.store_document
            )
        if not self.revision:
            self.revision = 0

        if not self.revision_year:
            self.revision_year = date.today().year

        # date inspected
        if self.date_inspected and not self.inspected:
            self.inspected = True
        if not self.date_inspected:
            self.inspected = False

        # date accepted
        if self.date_accepted and not self.accepted:
            self.accepted = True
        if not self.date_accepted:
            self.accepted = False

        # date submitted
        if self.date_submitted and not self.submitted:
            self.submitted = True
        if not self.date_submitted:
            self.submitted = False

        super(Quotation, self).save(force_insert=force_insert, force_update=force_update)

    def get_number(self):
        number = ""
        if self.number:
            number = zerofill(self.number, 3)
        if self.revision == 0:
            return "%s-%s" % (number, self.revision_year)
        return "%s/%s-%s" % (number, self.revision, self.revision_year)
    get_number.short_description = _("Number")


    def default_invoice_address(self):
        """
        restituisce il primo indirizzo di fatturazione
        """
        invoice_addresses = self.owner.get_addresses().filter(address_scopes__is_invoice_address=True)
        if invoice_addresses:
            return invoice_addresses[0]
        return None

    def default_invoice_address1(self):
        """
        restituisce il primo indirizzo di fatturazione: prima riga
        """
        addr = self.default_invoice_address()
        return "%s %s, %s" % (addr.get_address_type_display() or "", addr.name, addr.number)

    def default_invoice_address2(self):
        """
        restituisce il primo indirizzo di fatturazione: seconda riga
        """
        addr = self.default_invoice_address()
        return "%s %s (%s)" % (addr.zip_code, addr.locality, addr.province)

    def default_invoice_address3(self):
        """
        restituisce il primo indirizzo di fatturazione: terza riga
        """
        addr = self.default_invoice_address()
        return "%s" % addr.country

    def shipping_address1(self):
        """
        restituisce indirizzo di spedizione: prima riga
        """
        return "%s %s, %s" % (self.shipping_address.get_address_type_display() or "", self.shipping_address.name, self.shipping_address.number)

    def shipping_address2(self):
        """
        restituisce indirizzo di spedizione: seconda riga
        """
        return "%s %s (%s)" % (self.shipping_address.zip_code, self.shipping_address.locality, self.shipping_address.province)

    def shipping_address3(self):
        """
        restituisce indirizzo di spedizione: terza riga
        """
        return "%s" % self.shipping_address.country

    def update_totals(self):
        try:
            first_quotationrow=self.rows.all()[0]
            first_quotationrow.update_totals()
        except:
            pass

    if "procedure" in settings.INSTALLED_APPS:
        @transaction.commit_manually
        def create_or_append_to_procedure(self, procedure = None, date_p = None):
            """
            crea o aggiunge ad una Pratica le Righe Preventivo
            """
            if not date_p:
                date_p = date.today()
            try:
                ## se il Preventivo non è accettato esco
                if not self.accepted:
                    transaction.rollback()
                    return (False, "Quotation is not accepted")

                ## se il Preventivo è già stato trasformato esco
                if self.procedure:
                    transaction.rollback()
                    return (False, "Quotation already ordered")

                quotationrows = self.rows.all()
                ## se non ci sono righe da trasformare esco con errore
                if not quotationrows:
                    transaction.rollback()
                    return (False, "There are not ordering rows")

                if not procedure:
                    from procedure.models import Procedure
                    procedure = Procedure(
                        owner=self.owner, recipient=self.recipient, project=self.project,
                        payment_form=self.payment_form, office=self.office,
                        contact=self.contact, currency=self.currency, delivery=self.delivery,
                        vat=self.vat, agent=self.agent, commision=self.commision,
                        price_list=self.price_list, vector=self.vector, store_cause=self.store_cause,
                        description=self.description, date_p=date_p, note=self.note,
                        total_income=self.total_income, total_fee=self.total_fee,
                        total_discount=self.total_discount, total_vat=self.total_vat,
                        date_delivery=self.date_delivery, quote_firm_for=self.quote_firm_for,
                        delivery_window=self.delivery_window
                    )
                    procedure.save()

                for row in quotationrows:
                    message_row = row.add_to_procedure(procedure)

                if message_row[0] == False:
                    transaction.rollback()
                    return message_row

                if procedure.rows.count() == 0:
                    transaction.rollback()
                    return (False, "Error!!!")
            except Exception, e:
                transaction.rollback()
                return (False, e)

            ## Preventivo Trasformato correttamente ed interamente
            try:
                self.procedure = True
                self.save()
                transaction.commit()
            except Exception, e:
                pass
            return (True, "")

    def get_totals(self):
        totals = {}
        totals['income'] = decimal.Decimal(self.total_income)
        totals['fee'] = decimal.Decimal(self.total_fee)
        totals['discount'] = decimal.Decimal(self.total_discount)
        totals['vat'] = decimal.Decimal(self.total_vat)
        totals['total_no_vat'] = decimal.Decimal(self.total_income) + \
                                 decimal.Decimal(self.total_fee) - \
                                 decimal.Decimal(self.total_discount)
        totals['total'] = totals['total_no_vat'] + decimal.Decimal(self.total_vat)
        return totals
    totals = property(get_totals)

    class Meta:
        verbose_name = _("quotation")
        verbose_name_plural = _("quotations")
        permissions = (
            ('add_customer_quotation',          'Can ADD Customer quotation'),
            ('add_supplier_quotation',          'Can ADD Supplier quotation'),

            ('change_customer_quotation',       'Can CHANGE Customer quotation'),
            ('change_supplier_quotation',       'Can CHANGE Supplier quotation'),

            ('view_quotation',                  'Can VIEW quotation'),
            ('view_customer_quotation',         'Can VIEW Customer quotation'),
            ('view_supplier_quotation',         'Can VIEW Supplier quotation'),

            ('list_customer_quotation',         'Can LIST Customer Invoices'),
            ('list_supplier_quotation',         'Can LIST Supplier Invoices'),
        )

    def __unicode__(self):
        return u"%(date)s, %(number)s" % {'date': self.date_q, 'number': self.number}

##################################
# RIGA PREVENTIVO                #
##################################
UNIT_TYPE = (
    ("pz", _("Pezzi")),
    ("ml", _("Metro lineare")),
    ("mq", _("Metro quadro")),
    ("mc", _("Metro cubo")),
    ("kg", _("Chili")),
    ("t", _("Tonnellate")),
    ("l", _("Litri")),
    ("cp", _("a corpo")),
    ("hh", _("ore")),
    ("gg", _("giorni")),
)

class QuotationRow (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    quotation = models.ForeignKey (
        Quotation,
        related_name = "rows",
        db_index = True,
        verbose_name = _("Quotation"),
    )
    ### da aggiungere anche anno
    product = models.ForeignKey (
        Product,
        related_name = "quotationrows",
        db_index = True,
        verbose_name = _("Product"),
    )
    vat = models.ForeignKey (
        Vat,
        related_name = "quotationrows",
        db_index = True,
        verbose_name = _("Vat"),
    )
    store_cause = models.ForeignKey (
        StoreCause,
        related_name = "quotationrows",
        db_index = True,
        verbose_name = _("Store Cause"),
    )
    project = models.ForeignKey(
        Project,
        null = True, blank = True,
        db_index = True,
        related_name = "quotationrows",
        verbose_name = _("Project"),
    )
    ## Campi
    quantity = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Quantity"),
    )
    m_unit = models.CharField (
        max_length = 2,
        choices = UNIT_TYPE,
        default = "pz",
        db_index = True,
        verbose_name = _("Unit"),
    )
    price = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Price"),
    )
    # costo manodopera
    labor_cost = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Labor cost"),
    )
    # costo materiale
    material_cost = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Matrial cost"),
    )
    # costo dei servizi - noleggio e trasporti
    services_cost = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Services cost"),
    )
    # ricarico da applicare sui precedenti costi per determinare il prezzo
    # di vendita
    recharge = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("recharge %"),
    )
    fee = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Fee"),
    )
    discount = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        default = 0,
        verbose_name = _("Discount"),
    )
    description = models.CharField(
        max_length = 200,
        blank=True, null=True,
        verbose_name = _("Description")
    )
    long_description = models.TextField (
        blank = True, null = True,
        verbose_name = _("Long Description"),
    )
    to_print = models.BooleanField (
        db_index = True,
        default = True,
        verbose_name = _("To Print"),
    )
    # riepilogo
    # campo booleano utilizzato per inserire la riga nel report di riepilogo
    summary = models.BooleanField (
        db_index = True,
        default = True,
        verbose_name = _("Summary"),
    )

    def save(self, force_insert=False, force_update=False):
        super(QuotationRow, self).save(force_insert=force_insert, force_update=force_update)
        self.update_totals()

    def save_not_update(self, force_insert=False, force_update=False):
        super(QuotationRow, self).save(force_insert=force_insert, force_update=force_update)

    def delete(self):
        super(QuotationRow, self).delete()
        self.update_totals()

    def get_costs(self):
        return self.labor_cost + self.material_cost + self.services_cost
    costs = property(get_costs)

    def get_profit(self):
        return self.price - self.costs
    profit = property(get_profit)

    ## aggiunto per TETTI e AFFINI
    def get_copertura(self):
        try:
            return self.copertura
        except:
            return None
    get_copertura = property(get_copertura)

    def get_link_copertura(self):
        if self.get_rimozione:
            return ""
        if self.get_copertura:
            return """
                <a href="%s?quotation_row=%s&_return_to=%s" title="%s">
                    <img src="%sadmin/images/c.png">
                </a>
            """ % (
                reverse('admin:copertura_copertura_change', self.get_copertura.id),
                self.id,
                reverse('admin:quotation_quotation_change', self.quotation.id),
                _('Copertura'),
                settings.MEDIA_URL
            )
        return """
            <a href="%s?quotation_row=%s&_return_to=%s" title="%s">
                <img src="%sadmin/images/c.png">
            </a>
        """ % (
            reverse('admin:copertura_copertura_add'),
            self.id,
            reverse('admin:quotation_quotation_change', self.quotation.id),
            _('Copertura'),
            settings.MEDIA_URL
        )

    def get_total_material_cost(self):
        """
        restituisce il costo totale dei materiali
        """
        return self.material_cost * self.quantity

    def get_total_services_cost(self):
        """
        restituisce il costo totale dei servizi
        """
        return self.services_cost * self.quantity

    def get_total_labor_cost(self):
        """
        restituisce il costo totale dei manodopera
        """
        return self.labor_cost * self.quantity

    def get_total_costs(self):
        """
        restituisce il totale dei costi
        """
        return self.costs * self.quantity

    def get_total_profit(self):
        """
        restituisce l'utile totale
        """
        return self.profit * self.quantity

    ## aggiunto per TETTI e AFFINI
    def get_rimozione(self):
        try:
            return self.rimozione
        except:
            return None
    get_rimozione = property(get_rimozione)

    def get_link_rimozione(self):
        if self.get_copertura:
            return ""
        if self.get_rimozione:
            rimozione = self.rimozione
            return """
                <a href="%s?quotation_row=%s&_return_to=%s" title="%s">
                    <img src="%sadmin/images/a.png">
                </a>
            """ % (
                reverse('admin:rimozione_rimozione_change', self.get_rimozione.id),
                self.id,
                reverse('admin:quotation_quotation_change', self.quotation.id),
                _('Rimozione'),
                settings.MEDIA_URL
            )
        return """
            <a href="%s?quotation_row=%s&_return_to=%s" title="%s">
                <img src="%sadmin/images/a.png">
            </a>
        """ % (
            reverse('admin:rimozione_rimozione_add'),
            self.id,
            reverse('admin:quotation_quotation_change', self.quotation.id),
            _('Rimozione'),
            settings.MEDIA_URL
        )

    def get_description(self):
        """
        Restituisce la descrizione della riga o, se non esistente, il nome del prodotto
        """
        return self.description or  self.long_description or "%s %s" % (self.product.code, self.product.name)

    def get_description_wo_code(self):
        """
        Restituisce la descrizione della riga o, se non esistente, il nome del prodotto
        """
        return self.description or self.product.name

    def get_long_description(self):
        """
        Restituisce la descrizione estesa della riga o la descrizione estesa del prodotto
        """
        return self.long_description or self.product.full_description

    def get_net_price(self):
        """
        restituisce il prezzo unitario con sconto e spesa aggiuntivi
        """
        return self.price - self.discount + self.fee
    net_price = property(get_net_price)
    get_net_price.short_description = _("Net Price")

    def get_total_income(self):
        return self.quantity * self.price
    total_income = property(get_total_income)
    get_total_income.short_description = _("Total Income")

    def get_total_fee(self):
        return self.quantity * self.fee
    total_fee = property(get_total_fee)
    get_total_fee.short_description = _("Total Fee")

    def get_total_discount(self):
        return self.quantity * self.discount
    total_discount = property(get_total_discount)
    get_total_discount.short_description = _("Total Discount")

    def get_total(self):
        return self.total_income + self.total_fee - self.total_discount
    total = property(get_total)
    get_total.short_description = _("Total")

    def update_totals(self):
        quotation = self.quotation
        rows=self.quotation.rows.all()
        VatRow.objects.filter(quotation=quotation).delete()
        total = {'income': 0, 'fee': 0, 'discount': 0, 'total': 0, 'vat': 0}
        subtotal = {'income': {}, 'fee': {}, 'discount': {}, 'vat': {}, 'total': {}}
        for row in rows:
            subtotal['income'][row.vat]=subtotal['income'].get(row.vat, 0)
            subtotal['income'][row.vat]+=row.total_income

            subtotal['fee'][row.vat]=subtotal['fee'].get(row.vat, 0)
            subtotal['fee'][row.vat]+=row.total_fee
            subtotal['discount'][row.vat]=subtotal['discount'].get(row.vat, 0)
            subtotal['discount'][row.vat]+=row.total_discount

            subtotal['total'][row.vat]=subtotal['total'].get(row.vat, 0)
            subtotal['total'][row.vat]+=row.total

        for vat in subtotal['income'].keys():
            ## calcolo Totali Procedura
            total['income']+=subtotal['income'][vat]
            total['fee']+=subtotal['fee'][vat]
            total['discount']+=subtotal['discount'][vat]
            total['total']+=subtotal['total'][vat]
            ## calcolo Subtotale IVA per ogni aliquota
            subtotal['vat'][vat]=subtotal['total'][vat]*vat.aliquot_perc/100
            total['vat']+=subtotal['vat'][vat]
            ## creo nuovo oggetto VatRow
            vatrow = VatRow()
            vatrow.quotation = quotation
            vatrow.vat = vat
            vatrow.total_income = str(subtotal['income'][vat])
            vatrow.total_fee = str(subtotal['fee'][vat])
            vatrow.total_discount = str(subtotal['discount'][vat])
            vatrow.total_vat = str(subtotal['vat'][vat])
            vatrow.save()

        total['totals']=total['total']+total['vat']

        quotation.total_income = str(total['income'])
        quotation.total_fee = str(total['fee'])
        quotation.total_discount = str(total['discount'])
        quotation.total_vat = str(total['vat'])
        quotation.save()

    if "procedure" in settings.INSTALLED_APPS:
        @transaction.commit_manually
        def add_to_procedure(self, procedure):
            # se il preventivo è già stato trasformato in ordine esco
            if self.quotation.procedure:
                transaction.rollback()
                return (False, "Quotation already Ordered")
            try:
                procedurerow = ProcedureRow(
                    procedure=procedure, product=self.product, vat=self.vat, store_cause=self.store_cause,
                    quantity=self.quantity, price=self.price, m_unit=self.m_unit, fee=self.fee, discount=self.discount,
                    description=self.description, long_description=self.long_description
                )
                procedurerow.save()
            except Exception, e:
                transaction.rollback()
                return (False, e)
            transaction.commit()
            return (True, "")


    class Meta:
        verbose_name = _("quotation row")
        verbose_name_plural = _("quotation rows")
        ordering = ['-quotation__date_q', '-quotation__number', 'ordering', 'id']
        permissions = (
            ('view_customer_quotationrow',         'Can VIEW Customer quotation Row'),
            ('view_supplier_quotationrow',         'Can VIEW Supplier quotation ROw'),)

    def __unicode__(self):
        return u"%(quotation)s, %(product)s" % {'quotation': self.quotation, 'product': self.product}

##################################
# RIGA IVA                       #
##################################
class VatRow (DateModel, UserModel, StatusModel):
    ## ForeignKey
    quotation = models.ForeignKey (
        Quotation,
        related_name = "vattotalquotationrows",
        db_index = True,
        verbose_name = _("Quotation"),
    )
    vat = models.ForeignKey (
        Vat,
        related_name = "vattotalquotationrows",
        db_index = True,
        verbose_name = _("Vat"),
    )
    total_income = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Income"),
    )
    total_fee = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Fee"),
    )
    total_discount = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Discount"),
    )
    total_vat = models.DecimalField (
        max_digits = 9,
        decimal_places = 2,
        verbose_name = _("Total Vat"),
    )

    def update_totals(self):
        quotation = self.quotation
        try:
            first_quotationrow=quotation.rows.all()[0]
            first_quotationrow.update_totals()
        except:
            pass

    class Meta:
        verbose_name = _("vat row")
        verbose_name_plural = _("vat rows")
        ordering = ('vat__ordering', 'vat__code')

    def __unicode__(self):
        return u"%s %s %s %s" % (self.quotation, self.vat, self.total_income, self.total_vat)

class Settings(JSettings):
    pass