from django.http import Http404, HttpResponse, HttpResponseForbidden, HttpResponseRedirect
from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode
from django.core.urlresolvers import reverse
from django.core.mail import EmailMessage
from django.contrib import messages
from django.shortcuts import render_to_response
from django import forms
from django import template
from django.template.loader import render_to_string

from jumbo.forms.fields import MultiEmailField
from jumbo.views.generic.create_oo_file import create_file
from jumbo.utils.functions import get_settings

from quotation.models import Quotation, QuotationRow

class SendMailForm(forms.Form):
    mails       = forms.MultipleChoiceField(required=False, label=_("Send To"))
    other_mails = MultiEmailField(required=False, label=_("Other Mail Addresses"))
    attention_of = forms.CharField(required=False, label=_("Attention of"))

    def clean(self):
        cleaned_data = self.cleaned_data

        mails = cleaned_data.get('mails', [])
        other_mails = cleaned_data.get('other_mails', '')

        if not mails and not other_mails:
            raise forms.ValidationError("Field Mails or Other Mail Addresses is required.")

        return cleaned_data

def quotation_send_mail(request, quotation_id):
    try:
        qu = Quotation.objects.get(pk=quotation_id)
        opts = qu._meta
    except Exception, e:
        raise Http404(_('The object with primary key %s does not exist.') % quotation_id)

    mails = set(qu.owner.get_all_mails_for_quotation())
    if qu.owner != qu.recipient:
        for rec_email in qu.recipient.get_all_mails_for_quotation():
            mails.add(rec_email)

    # Recupero la lingua del primo contatto per decidere in che lingua
    # creare e mandare la fattura
    odt_language = 'en'
    contacts = qu.owner.get_quotation_contacts()[:1]

    if contacts and contacts[0].language:
        odt_language = contacts[0].language.code

    lmails = [ (email[0], email[1]) for email in mails ]

    if request.POST:
        form = SendMailForm(request.POST)
        form.fields['mails'].choices = lmails
        if form.is_valid():
            f = create_file(
                request, Quotation, quotation_id,
                file_model = 'admin/quotation/quotation/quotation.odt',
                file_type  = 'pdf', get_raw_string = True,
                language = odt_language, extra_context={'form':form}
            )
            all_mails = set()

            fromail = get_settings("quotation.settings", "QUOTATION_SEND_MAIL_FROM_EMAIL")
            if qu.creator.email:
                fromail = qu.creator.email
                all_mails.add(qu.creator.email)

            for mail in form.cleaned_data['mails']:
                if mail:
                    all_mails.add(mail)

            for mail in form.cleaned_data['other_mails']:
                if mail:
                    all_mails.add(mail)

            if all_mails:
                subject = render_to_string("admin/quotation/quotation/mail_subject.txt", {'qu':qu, 'form':form})
                # Email subject *must not* contain newlines
                subject = ''.join(subject.splitlines())
                body    = render_to_string("admin/quotation/quotation/mail_body.txt", {'qu':qu, 'form':form})

                email_msg = EmailMessage(subject, body, fromail, [], all_mails)

                email_msg.attach(
                    'preventivo_%s.pdf' % qu.date_q.strftime("%Y%m%d"),
                    f.read(), 'application/pdf'
                )

                email_msg.send(fail_silently=False)
            messages.success(request, _('Mail Sent Successfully.'))
            return HttpResponseRedirect(reverse('admin:quotation_quotation_changelist'))
    else:
        form = SendMailForm()
        form.fields['mails'].choices = lmails

    context = {
        'title': _('Send Mail - quotation : %s') % force_unicode(qu),
        'object_id': quotation_id,
        'obj': qu,
        'is_popup': request.REQUEST.has_key('_popup'),
        'app_label': opts.app_label,
        'opts': opts,
        'form':form
    }
    context_instance = template.RequestContext(request, current_app='quotation')
    return render_to_response('admin/quotation/quotation/sendmail.html', context,
                              context_instance=context_instance)
