import re
import math
from django.template import Library, Node, Variable, VariableDoesNotExist, TemplateSyntaxError
from django.conf import settings
from django.utils.encoding import force_unicode
from jumbo.shortcuts import render_to_string

from django import template
register = template.Library()

def show_product_icon(product, *args, **kwargs):
    icon = ''
    try:
        if product.shop.icon.size > 0:
            icon = product.shop.icon
    except:
        try:
            if product.parent.shop.icon.size > 0:
                icon = product.parent.icon
        except:
            pass
    return icon

def show_product_icon_url(product, *args, **kwargs):
    icon = ''
    try:
        if product.shop.icon.size > 0:
            icon = product.shop.icon.url
    except:
        try:
            if product.parent.shop.icon.size > 0:
                icon = product.parent.shop.icon.url
        except:
            pass
    return icon

def show_popup_info(product):
    info = ''
    try:
        if product.shop.popup_info and product.shop.popup_info > 0:
            info = product.shop.popup_info
    except:
        try:
            if product.parent.shop.popup_info and product.parent.shop.popup_info > 0:
                info = product.parent.shop.popup_info 
        except:
            pass
    return locals()

try:
    from sorl.thumbnail.main import DjangoThumbnail, get_thumbnail_setting
    from sorl.thumbnail.processors import dynamic_import, get_valid_options

    size_pat = re.compile(r'(\d+)x(\d+)$')
    quality_pat = re.compile(r'quality=([1-9]\d?|100)$')

    try:
        PROCESSORS = dynamic_import(get_thumbnail_setting('PROCESSORS'))
        VALID_OPTIONS = get_valid_options(PROCESSORS)
    except:
        if get_thumbnail_setting('DEBUG'):
            raise
        else:
            PROCESSORS = []
            VALID_OPTIONS = []

    class ThumbnailNode(Node):
        def __init__(self, source_var, size_var, opts=None,
                     context_name=None, **kwargs):
            self.source_var = source_var
            self.size_var = size_var
            self.opts = opts
            self.context_name = context_name
            self.kwargs = kwargs

        def render(self, context):
            # Note that this isn't a global constant because we need to change the
            # value for tests.
            DEBUG = get_thumbnail_setting('DEBUG')
            try:
                # A file object will be allowed in DjangoThumbnail class
                relative_source = self.source_var.resolve(context)
            except VariableDoesNotExist:
                if DEBUG:
                    raise VariableDoesNotExist("Variable '%s' does not exist." %
                            self.source_var)
                else:
                    relative_source = None
            try:
                requested_size = self.size_var.resolve(context)
            except VariableDoesNotExist:
                if DEBUG:
                    raise TemplateSyntaxError("Size argument '%s' is not a"
                            " valid size nor a valid variable." % self.size_var)
                else:
                    requested_size = None
            # Size variable can be either a tuple/list of two integers or a valid
            # string, only the string is checked.
            else:
                if isinstance(requested_size, basestring):
                    m = size_pat.match(requested_size)
                    if m:
                        requested_size = (int(m.group(1)), int(m.group(2)))
                    elif DEBUG:
                        raise TemplateSyntaxError("Variable '%s' was resolved but "
                                "'%s' is not a valid size." %
                                (self.size_var, requested_size))
                    else:
                        requested_size = None
            if relative_source is None or requested_size is None:
                thumbnail = ''
            else:
                try:
                    thumbnail = DjangoThumbnail(relative_source, requested_size,
                            opts=self.opts, processors=PROCESSORS, **self.kwargs)
                except:
                    if DEBUG:
                        raise
                    else:
                        thumbnail = ''
            # Return the thumbnail class, or put it on the context
            if self.context_name is None:
                return thumbnail
            # We need to get here so we don't have old values in the context
            # variable.
            context[self.context_name] = thumbnail
            return ''

    def show_product_icon_thumbnail_url(parser, token):
        args = token.split_contents()
        tag = args[0]
        # Check to see if we're setting to a context variable.
        if len(args) in (5, 6) and args[-2] == 'as':
            context_name = args[-1]
            args = args[:-2]
        else:
            context_name = None

        if len(args) not in (3, 4):
            raise TemplateSyntaxError("Invalid syntax. Expected "
                "'{%% %s source size [option#1,option#2,...] %%}' or "
                "'{%% %s source size [option#1,option#2,...] as variable %%}'" % (tag, tag))

        # Get the source image path and requested size.
        source_var = parser.compile_filter(args[1])
        # Since we changed to allow filters we have to make an exception for our syntax
        m = size_pat.match(args[2])
        if m:
            args[2] = '"%s"' % args[2]
        size_var = parser.compile_filter(args[2])

        # Get the options.
        if len(args) == 4:
            args_list = args[3].split(',')
        else:
            args_list = []

        # Check the options.
        opts = []
        kwargs = {} # key,values here override settings and defaults

        for arg in args_list:
            if arg in VALID_OPTIONS:
                opts.append(arg)
            else:
                m = quality_pat.match(arg)
                if not m:
                    raise TemplateSyntaxError("'%s' tag received a bad argument: "
                            "'%s'" % (tag, arg))
                kwargs['quality'] = int(m.group(1))
        return ThumbnailNode(source_var, size_var, opts=opts,
                context_name=context_name, **kwargs)

    show_product_icon_thumbnail = show_product_icon_thumbnail_url

    register.simple_tag(show_product_icon)
    register.simple_tag(show_product_icon_url)
    register.tag(show_product_icon_thumbnail)
    register.tag(show_product_icon_thumbnail_url)

except ImportError:
    show_product_icon_thumbnail = show_product_icon
    show_product_icon_thumnail_url = show_product_icon_url

    register.simple_tag(show_product_icon)
    register.simple_tag(show_product_icon_url)
    register.simple_tag(show_product_icon_thumbnail)
    register.simple_tag(show_product_icon_thumbnail_url)

register.simple_tag(show_popup_info)

