from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from jumbo.admin.options import ExtendibleModelAdmin, OrderedModelAdmin, SettingsAdmin
from jumbo.forms.models import AdminModelForm

from store.models import Store, StoreDocument, StoreCause, StoreDocumentNumber, Settings

class StoreAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("store/css/admin_store.css",)}
    fieldsets = (
        (_('logs admin'), {'fields': (('creator', 'date_create',), ('last_modifier', 'date_last_modify',)), 'classes': ('collapse',)}),
        (_('general admin'), {'fields': ('status','default','name','code','address','description',),}),
    )
    list_display = ('get_status_icon', 'code', 'name', 'get_edit_icon', 'get_delete_icon')
    search_fields = ['code', 'name']

class StoreDocumentAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("store/css/admin_store.css",)}
    fieldsets = (
        (_('logs admin'), {'fields': (('creator', 'date_create',), ('last_modifier', 'date_last_modify',)), 'classes': ('collapse',)}),
        (_('general admin'), {'fields': ('status','name','code',),}),
        (_('description admin'), {'fields': ('description',),}),
    )
    list_display = ('get_status_icon', 'code', 'name', 'get_edit_icon', 'get_delete_icon')
    search_fields = ['code', 'name']

class StoreCauseAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("store/css/admin_store.css",)}
    fieldsets = (
        (_('logs admin'), {'fields': (('creator', 'date_create',), ('last_modifier', 'date_last_modify',)), 'classes': ('collapse',)}),
        (_('general admin'), {'fields': (
            'status','name','code','store', 'store_document',
            'category', 'default', 'default_for_shop', 'model_template'
        ),}),
        (_('description admin'), {'fields': ('description',),}),
        (_('operation admin'), {'fields': ('move','stock','busy','ddt','invoice',),}),
    )
    list_display = (
        'get_status_icon', 'code', 'name', 'store', 'store_document', 'move', 'stock', 'busy', 'ddt', 'invoice', 'default', 'default_for_shop', 'get_edit_icon', 'get_delete_icon'
    )
    search_fields = ['code', 'name']

class StoreDocumentNumberAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("store/css/admin_store.css",)}
    fieldsets = (
        (_('logs admin'), {'fields': (('date_create', 'date_last_modify',)), 'classes': ('collapse',)}),
        (_('general admin'), {'fields': ('store_document', 'application', 'year', 'last_date', 'last_number'),}),
    )
    list_display = (
        'store_document', 'application', 'year', 'last_date', 'last_number', 'get_edit_icon', 'get_delete_icon'
    )
    search_fields = ['year', 'application']

admin.site.register(Settings, SettingsAdmin)
admin.site.register(Store, StoreAdmin)
admin.site.register(StoreDocument, StoreDocumentAdmin)
admin.site.register(StoreCause, StoreCauseAdmin)
admin.site.register(StoreDocumentNumber, StoreDocumentNumberAdmin)
