from django.utils.translation import ugettext_lazy as _

from admin_tools.dashboard import modules
from product.models import Product
from store.models import Store

class StoreDashboard(modules.Group):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': "Stores",
            'display': "tabs",
            'children': [
                StoreModels(),
                StoreSettings(),
            ]
        }
        super(StoreDashboard, self).__init__(**kwargs)

class StoreModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Models"),
            'models': (
                'store.models.Store', 'store.models.StoreAddress', 'store.models.StoreDocument',
                'store.models.StoreDocumentNumber', 'store.models.StoreCause',
            ),
        }
        super(StoreModels, self).__init__(**kwargs)

class StoreSettings(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs =  {
            'title': _("Settings"),
            'models':['store.models.Settings',],
            }
        super(StoreSettings, self).__init__(**kwargs)

class Stores(modules.DashboardModule):
    def __init__(self, title, template='admin_tools/dashboard/stores.html',
                 order_by=None, limit=10, qs=Store.objects.all(), extra_filters={},
                 children=None, **kwargs
    ):
        self.order_by = order_by
        self.template = template
        self.qs = qs
        self.extra_filters = extra_filters
        self.limit = limit
        self.custom_children = children

        kwargs =  {
            'title': title,
            'template': self.template,
            'children': self.custom_children
        }
        super(Stores, self).__init__(**kwargs)

    def init_with_context(self, context):
        request = context['request']
        if self.custom_children is None:
            qs = self.qs
            if self.extra_filters: qs = qs.filter(**self.extra_filters)
            if self.order_by: qs = qs.order_by(self.order_by)
            if self.limit: qs = qs[:self.limit]
            self.children = qs

