from django.conf import settings
from django.utils.translation import ugettext as _
from jumbo.core.urlresolvers import reverse
from store.models import Store, StoreCause, StoreDocument, StoreDocumentNumber

class StoreList(object):
    list_display = ['get_status_icon', 'code', 'get_detail', 'get_edit_icon']
    default_order_by = 'code'
    model = Store

    def get_status_icon(self, result):
        return "<img title='%(name_status)s' src='%(url)sjumbo/img/status_%(status)s.gif'/>" % {
            'name_status':  result.get_status_display(),
            'url':          settings.MEDIA_URL,
            'status':       result.status
        }
    get_status_icon.short_description = _("Status")

    def get_detail(self, result):
        return """
            %s<br/>
            %s
        """ % (result.name, result.description or "")
    get_detail.short_description = _("Name")

    def get_edit_icon(self, result):
        return "<a href=%(reverse_url)s><img src='%(url)sjumbo/img/edit.gif' alt='%(window_title)s' /></a>" % {
            'reverse_url':reverse('store-store-edit', result.id),
            'url':settings.MEDIA_URL,
            'window_title':_("Edit Store")
        }
    get_edit_icon.short_description = _("Edit")

class StoreDocumentList(object):
    list_display = ['get_status_icon', 'code', 'name', 'get_edit_icon']
    default_order_by = ('name',)
    model = StoreDocument

    def get_status_icon(self, result):
        return "<img title='%(name_status)s' src='%(url)sjumbo/img/status_%(status)s.gif'/>" % {
            'name_status':  result.get_status_display(),
            'url':          settings.MEDIA_URL,
            'status':       result.status
        }
    get_status_icon.short_description = _("Status")

    def get_edit_icon(self, result):
        return "<a href=%(reverse_url)s><img src='%(url)sjumbo/img/edit.gif' alt='%(window_title)s' /></a>" % {
            'reverse_url':reverse('store-storedocument-edit', result.id),
            'url':settings.MEDIA_URL,
            'window_title':_("Edit Store Document")
        }
    get_edit_icon.short_description = _("Edit")

class StoreDocumentNumberList(object):
    list_display = ['store_document', 'application', 'year', 'last_number', 'get_edit_icon']
    default_order_by = ('year', 'application',)
    model = StoreDocumentNumber

    def get_edit_icon(self, result):
        return "<a href=%(reverse_url)s><img src='%(url)sjumbo/img/edit.gif' alt='%(window_title)s' /></a>" % {
            'reverse_url':reverse('store-storedocumentnumber-edit', result.id),
            'url':settings.MEDIA_URL,
            'window_title':_("Edit Store Document Number")
        }
    get_edit_icon.short_description = _("Edit")


class StoreCauseList(object):
    list_display = ['get_status_icon', 'store', 'store_document', 'code', 'get_detail', 'category', 'move', 'ddt', 'invoice', 'stock', 'busy', 'default', 'get_edit_icon']
    default_order_by = ('store', 'code')
    model = StoreCause

    def get_status_icon(self, result):
        return "<img title='%(name_status)s' src='%(url)sjumbo/img/status_%(status)s.gif'/>" % {
            'name_status':  result.get_status_display(),
            'url':          settings.MEDIA_URL,
            'status':       result.status
        }
    get_status_icon.short_description = _("Status")

    def get_detail(self, result):
        return """
            %s<br/>
            %s
        """ % (result.name, result.description or "")
    get_detail.short_description = _("Name")

    def get_edit_icon(self, result):
        return "<a href=%(reverse_url)s><img src='%(url)sjumbo/img/edit.gif' alt='%(window_title)s' /></a>" % {
            'reverse_url':reverse('store-storecause-edit', result.id),
            'url':settings.MEDIA_URL,
            'window_title':_("Edit Store Cause")
        }
    get_edit_icon.short_description = _("Edit")
