# -*- coding: utf-8 -*-

from django.conf.urls.defaults import *

from jumbo.views.generic.list_detail import object_list, object_detail
from jumbo.views.generic.create_update import create_object, update_object

from store.models import Store, StoreCause, StoreDocument, StoreDocumentNumber
from store.list import StoreList, StoreCauseList, StoreDocumentList, StoreDocumentNumberList
from store.forms import StoreForm, StoreCauseForm, StoreDocumentForm, StoreDocumentNumberForm
from store.forms import StoreSearchForm, StoreCauseSearchForm, StoreDocumentSearchForm, StoreDocumentNumberSearchForm

urlpatterns = patterns('store.views',
    ### STORE ###
    url(r'^$', object_list, {
        'search_form':                          StoreSearchForm,
        'data_list':                            StoreList,
        'template_name':                        'store/store_list.html'
    }),
    url(r'^store/$', object_list, {
        'search_form':                          StoreSearchForm,
        'data_list':                            StoreList,
        'template_name':                        'store/store_list.html'
    },
    name='store-store-list'),
    url(r'^store/add/$', create_object, {
        'model':                                Store,
        'form_class':                           StoreForm,
        'post_save_redirect':                   'store-store-list',
        'post_save_continue_redirect':          'store-store-edit',
        'post_save_add_another':                'store-store-add',
        'template_name':                        'store/store_form.html'
    },
    name='store-store-add'),
    url(r'^store/(?P<object_id>\d+)/$', update_object, {
        'model':                                Store,
        'form_class':                           StoreForm,
        'post_save_redirect':                   'store-store-list',
        'post_save_continue_redirect':          'store-store-edit',
        'post_save_add_another':                'store-store-add',
        'template_name':                        'store/store_form.html'
    },
    name='store-store-edit'),
    url(r'^store/detail/(?P<object_id>\d+)/$', object_detail, {
        'model':                                Store,
        'template_name':                        'store/store_detail.html'
    },
    name='store-store-detail'),

    ### STORECAUSE ###
    url(r'^storecause/$', object_list, {
        'search_form':                          StoreCauseSearchForm,
        'data_list':                            StoreCauseList,
        'template_name':                        'store/storecause_list.html'
    },
    name='store-storecause-list'),
    url(r'^storecause/add/$', create_object, {
        'model':                                StoreCause,
        'form_class':                           StoreCauseForm,
        'post_save_redirect':                   'store-storecause-list',
        'post_save_continue_redirect':          'store-storecause-edit',
        'post_save_add_another':                'store-storecause-add',
        'template_name':                        'store/storecause_form.html'
    },
    name='store-storecause-add'),
    url(r'^storecause/(?P<object_id>\d+)/$', update_object, {
        'model':                                StoreCause,
        'form_class':                           StoreCauseForm,
        'post_save_redirect':                   'store-storecause-list',
        'post_save_continue_redirect':          'store-storecause-edit',
        'post_save_add_another':                'store-storecause-add',
        'template_name':                        'store/storecause_form.html'
    },
    name='store-storecause-edit'),
    url(r'^storecause/detail/(?P<object_id>\d+)/$', object_detail, {
        'model':                                StoreCause,
        'template_name':                        'store/storecause_detail.html'
    },
    name='store-storecause-detail'),

    ### STORE DOCUMENT ###
    url(r'^storedocument/$', object_list, {
        'search_form':                          StoreDocumentSearchForm,
        'data_list':                            StoreDocumentList,
        'template_name':                        'store/storedocument_list.html'
    },
    name='store-storedocument-list'),
    url(r'^storedocument/add/$', create_object, {
        'model':                                StoreDocument,
        'form_class':                           StoreDocumentForm,
        'post_save_redirect':                   'store-storedocument-list',
        'post_save_continue_redirect':          'store-storedocument-edit',
        'post_save_add_another':                'store-storedocument-add',
        'template_name':                        'store/storedocument_form.html'
    },
    name='store-storedocument-add'),
    url(r'^storedocument/(?P<object_id>\d+)/$', update_object, {
        'model':                                StoreDocument,
        'form_class':                           StoreDocumentForm,
        'post_save_redirect':                   'store-storedocument-list',
        'post_save_continue_redirect':          'store-storedocument-edit',
        'post_save_add_another':                'store-storedocument-add',
        'template_name':                        'store/storedocument_form.html'
    },
    name='store-storedocument-edit'),
    url(r'^storedocument/detail/(?P<object_id>\d+)/$', object_detail, {
        'model':                                StoreDocument,
        'template_name':                        'store/storestoredocument_detail.html'
    },
    name='store-storedocument-detail'),

    ### STORE DOCUMENT NUMBER ###
    url(r'^storedocumentnumber/$', object_list, {
        'search_form':                          StoreDocumentNumberSearchForm,
        'data_list':                            StoreDocumentNumberList,
        'template_name':                        'store/storedocumentnumber_list.html'
    },
    name='store-storedocumentnumber-list'),
    url(r'^storedocumentnumber/add/$', create_object, {
        'model':                                StoreDocumentNumber,
        'form_class':                           StoreDocumentNumberForm,
        'post_save_redirect':                   'store-storedocumentnumber-list',
        'post_save_continue_redirect':          'store-storedocumentnumber-edit',
        'post_save_add_another':                'store-storedocumentnumber-add',
        'template_name':                        'store/storedocumentnumber_form.html'
    },
    name='store-storedocumentnumber-add'),
    url(r'^storedocumentnumber/(?P<object_id>\d+)/$', update_object, {
        'model':                                StoreDocumentNumber,
        'form_class':                           StoreDocumentNumberForm,
        'post_save_redirect':                   'store-storedocumentnumber-list',
        'post_save_continue_redirect':          'store-storedocumentnumber-edit',
        'post_save_add_another':                'store-storedocumentnumber-add',
        'template_name':                        'store/storedocumentnumber_form.html'
    },
    name='store-storedocumentnumber-edit'),
    url(r'^storedocumentnumber/detail/(?P<object_id>\d+)/$', object_detail, {
        'model':                                StoreDocumentNumber,
        'template_name':                        'store/storestoredocumentnumber_detail.html'
    },
    name='store-storedocumentnumber-detail')
)
