from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from jumbo.db.forms import AdminModelForm, OrderedModelAdmin
from suspension.models import Suspension, SuspensionCause

class SuspensionAdminForm(AdminModelForm):
    class Meta:
        model = Suspension

class SuspensionAdmin(admin.ModelAdmin):
    form = SuspensionAdminForm
    list_per_page = 30
    fieldsets = (
        (_('logs admin'), {'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',), 'classes': ('collapse',)}),
        (None, {'fields': ('organization', 'product', 'procedure', 'procedure_row', ('disable_cause', 'date_begin', 'date_end'))}),
    )
    search_fields = ('organization', 'procedure', 'product', 'disable_cause', 'date_begin', 'date_end')
    list_display = ('date_begin', 'date_end', 'organization', 'product', 'procedure', 'procedure_row', 'disable_cause')
    list_display_links = ('date_begin', 'date_end')

admin.site.register(Suspension, SuspensionAdmin)

class SuspensionCauseAdminForm(AdminModelForm):
    class Meta:
        model = SuspensionCause

class SuspensionCauseAdmin(admin.ModelAdmin):
    form = SuspensionCauseAdminForm
    list_per_page = 30
    fieldsets = (
        (_('logs admin'), {'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',), 'classes': ('collapse',)}),
        (None, {'fields': ('code', 'name')}),
    )
    search_fields = ('code', 'name')
    list_display = ('code', 'name')
    list_display_links = ('code', 'name')

admin.site.register(SuspensionCause, SuspensionCauseAdmin)
