# -*- coding: utf-8 -*-

from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel

from procedure.models import Procedure, ProcedureRow
from product.models import Product
from organization.models import Organization

###############################
# SOSPENSIONI                 #
###############################
class Suspension (DateModel, UserModel, StatusModel):
    ## ForeignKey
    procedure = models.ForeignKey(
        Procedure,
        blank = True, null = True,
        related_name = "suspensions",
        db_index = True,
        verbose_name = _("Procedure"),
    )
    product = models.ForeignKey(
        Product,
        blank = True, null = True,
        related_name = "suspensions",
        db_index = True,
        verbose_name = _("Product"),
    )
    organization = models.ForeignKey(
        Organization,
        blank = True, null = True,
        related_name = "suspensions",
        db_index = True,
        verbose_name = _("Organization"),
    )
    procedure_row = models.ForeignKey(
        ProcedureRow,
        blank = True, null = True,
        related_name = "suspensions",
        db_index = True,
        verbose_name = _("Procedure row"),
    )
    disable_cause = models.ForeignKey(
        "SuspensionCause",
        related_name = "suspensions",
        db_index = True,
        verbose_name = _("Disable cause"),
    )
    date_begin = models.DateField(
        db_index = True,
        blank = True, null = True,
        verbose_name = _("Date begin"),
    )
    date_end = models.DateField(
        db_index = True,
        blank = True, null = True,
        verbose_name = _("Date end"),
    )
    note = models.TextField(
        blank = True, null = True,
        verbose_name = _("Notes"),
    )

    class Meta:
        verbose_name = _("suspension")
        verbose_name_plural = _("suspensions")

#    def __unicode__(self):

###############################
# CAUSALI SOSPENSIONE         #
###############################
class SuspensionCause (DateModel, UserModel, StatusModel, OrderedModel):
    code = models.CharField(
        max_length = 10,
        unique = True,
        db_index = True,
        verbose_name = _("Code"),
    )
    name = models.CharField(
        max_length = 100,
        unique = True,
        db_index = True,
        verbose_name = _("Nome"),
    )

    class Meta:
        verbose_name = _("suspension cause")
        verbose_name_plural = _("suspension causes")

    def __unicode__ (self):
        return u"%s - %s" % (self.code, self.name)
