# -*- coding: utf-8 -*-

from datetime import date, timedelta

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel, PeriodicityModel
from product.models import Product
from procedure.models import ProcedureRow
#####################################
# PRODOTTO SERIALE                  #
#####################################
class SerialNumber (DateModel, UserModel, StatusModel, OrderedModel):
    # si ricorda che: 
    # - "Pratica" movimenta la "Disponibilita"
    # - "DDT" movimenta la "Giacenza"
    # - "Fattura" genera il "Credito" o il "Debito" ed e" utilizzata per la "Garanzia"
    ## Campi Generici
    ## ForeignKey
    product = models.ForeignKey (
        Product,
        related_name = "serials",
        db_index = True,
        verbose_name = _("Product"),
    )
    procedurerow = models.ForeignKey (
        ProcedureRow,
        related_name = "serials",
        db_index = True,
        verbose_name = _("Procedure Row"),
    )
    ## Campi
    serial_number = models.CharField (
        max_length = 100,
        db_index = True,
        verbose_name = _("Serial number"),
    )

    class Meta:
        verbose_name = _('Serial number')
        verbose_name_plural = _('Serial numbers')
        unique_together = (('product', 'procedurerow', 'serial_number'))

    def __unicode__(self):
        return u"%s" % self.serial_number

    def save(self):
        self.product = self.procedurerow.product
        super(SerialNumber, self).save()

    def is_available(self):
        serial = Serial.object.filter(serial_number = self.serial_number).order_by('procedurerow__procedure__date_p')[0]

#####################################
# PRODOTTO LOTTO                    #
#####################################
class BatchNumber (DateModel, UserModel, StatusModel, OrderedModel):
    # si ricorda che: 
    # - "Pratica" movimenta la "Disponibilita"
    # - "DDT" movimenta la "Giacenza"
    # - "Fattura" genera il "Credito" o il "Debito" ed e" utilizzata per la "Garanzia"
    product = models.ForeignKey (
        Product,
        related_name = "batches",
        db_index = True,
        verbose_name = _("Product"),
    )
    procedurerow = models.ForeignKey (
        ProcedureRow,
        related_name = "batches",
        db_index = True,
        verbose_name = _("Procedure Row"),
    )
    ## Campi
    batch_number = models.CharField (
        max_length = 100,
        db_index = True,
        verbose_name = _("Batch number"),
    )

    class Meta:
        verbose_name = _('Batch number')
        verbose_name_plural = _('Batches numbers')
        unique_together = (('product', 'procedurerow', 'batch_number'))

    def __unicode__(self):
        return u"%s" % self.batch_number
