from django.test import TestCase
from django.db import IntegrityError
from django.contrib.auth.models import User
from wallet.models import Wallet

class WalletTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user('jsmith', 'jsmith@example.com', 'passwd')
    def tearDown(self):
        pass

    def test_create_wallet(self):
        wallet = Wallet(
            user=self.user,
        )
        wallet.save()
        self.assertNotEquals(wallet.id, 0)

    def test_fail_if_not_provided_user(self):
        wallet = Wallet()
        self.assertRaises(IntegrityError, wallet.save)

    def test_create_a_filled_wallet(self):
        wallet = Wallet(
            user=self.user,
            total=150,
        )
        wallet.save()
        self.assertNotEquals(wallet.id, 0)
        self.assertNotEquals(wallet.total, 0)

    def test_cannot_save_wallet_with_negative_total(self):
        wallet = Wallet(
            user=self.user,
            total=-10,
        )
        self.assertRaises(IntegrityError, wallet.save)

    def test_cannot_create_more_than_one_wallet_for_one_user(self):
        wallet1 = Wallet(
            user=self.user,
            total=150,
        )
        wallet1.save()
        self.assertNotEquals(wallet1.id, 0)
        self.assertNotEquals(wallet1.total, 0)

        wallet2 = Wallet(
            user=self.user,
        )
        self.assertRaises(IntegrityError, wallet2.save)


